@@ -18,12 +18,14 @@
 #include "pktdrvr.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: ax25.c,v 1.24 1997/05/24 12:34:05 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: ax25.c,v 1.27 1997/09/07 00:31:16 root Exp root $";
 #endif
 
 static int axsend (struct iface * iface, const char *dest, char *source,
 			   int cmdrsp, int ctl, struct mbuf * data);
 
+static void incCounter (struct iface *iface);
+			   
 #ifdef AXIP
 static int axip_stop (struct iface * iface);
 static int axip_raw (struct iface * iface, struct mbuf * bp);
@@ -38,6 +40,10 @@ extern int AXSmartRoute;
 extern int Axui_sock;
 #endif
 
+#ifdef NODECALL
+extern int NodeAlias, NodeNetrom;
+#endif
+
 
 /* List of AX.25 multicast addresses in network format (shifted ascii).
  * Only the first entry is used for transmissions, but any incoming
@@ -84,6 +90,15 @@ char Ccall[AXALEN], Calias[AXALEN];
 char Ttycall[AXALEN];
 #endif
 
+#ifdef NODECALL
+char Nodecall[AXALEN];
+char Nodealias[AXALEN];
+#endif
+
+#ifdef MBFWD
+extern char FWDCall[AXALEN];
+#endif
+
 char Tcall[AXALEN];
 char Icall[AXALEN];
 char Ncall[AXALEN];
@@ -139,6 +154,15 @@ int16 fcstab[256] =
 #endif
 
 
+static void
+incCounter (struct iface *iface)
+{
+	/* update the counters for messages out */
+	if (iface)
+		iface->axcnt.msgout++;
+}
+
+
 
 /* Send IP datagrams across an AX.25 link */
 int
@@ -207,7 +231,7 @@ char mode = AX_DEFMODE;	/* default to interface mode */
 	}
 	bp = tbp;
 	bp->data[0] = PID_IP;
-	if ((tbp = segmenter (bp, axp->paclen)) == NULLBUF) {
+	if ((tbp = segmenter (bp, axp->paclen, axp)) == NULLBUF) {
 		free_p (bp);
 		return -1;
 	}
@@ -229,6 +253,7 @@ int16 pid;			/* Protocol ID */
 struct mbuf *data;		/* Data field (follows PID) */
 {
 struct mbuf *bp;
+int retval;
 
 	/* Prepend pid to data */
 	bp = pushdown (data, 1);
@@ -237,7 +262,14 @@ struct mbuf *bp;
 		return -1;
 	}
 	bp->data[0] = uchar(pid);
-	return axsend (iface, dest, source, LAPB_COMMAND, UI, bp);
+
+	retval = axsend (iface, dest, source, LAPB_COMMAND, UI, bp);
+
+	/* update the counters for messages out */
+	if (retval != -1)
+		incCounter (iface);
+
+	return retval;
 }
 
 
@@ -291,6 +323,7 @@ int rval;
 		free_p (cbp);	/* Also frees data */
 		return -1;
 	}
+
 	/* This shouldn't be necessary because redirection has already been
 	 * done at the IP router layer, but just to be safe...
 	 */
@@ -410,6 +443,7 @@ char (*mpp)[AXALEN];
 int mcast;
 char *isrc, *idest;	/* "immediate" source and destination */
 int To_us = 0;		/* Is this a link to us ? */
+int send2axui = 0;
 
 	/* Pull header off packet and convert to host structure */
 	if (ntohax25 (&hdr, &bp) < 0) {
@@ -525,9 +559,17 @@ int To_us = 0;		/* Is this a link to us ? */
 	if (!To_us) {
 #ifdef NETROM
 		if (addreq (idest, Myalias))
-			To_us = ALIAS_LINK;	/* this is for the alias */
+			To_us =
+#ifdef NODECALL
+				(NodeAlias) ? NODE_LINK :
+#endif
+				ALIAS_LINK;	/* this is for the alias */
 		else if ((iface->flags & IS_NR_IFACE) && addreq (idest, Nr_iface->hwaddr))
-			To_us = NETROM_LINK;	/* this is for the netrom callsign! */
+			To_us =
+#ifdef NODECALL
+				(NodeNetrom) ? NODE_LINK :
+#endif
+				NETROM_LINK;	/* this is for the netrom callsign! */
 		else
 #endif
 		if (addreq (idest, iface->hwaddr))
@@ -546,12 +588,24 @@ int To_us = 0;		/* Is this a link to us ? */
 		else if (addreq (hdr.dest, Ncall))
 			To_us = NEWS_LINK;	/* this is for the news server call */
 #endif
+#ifdef NODECALL
+		else if (addreq (hdr.dest, Nodecall))
+			To_us = NODE_LINK;	/* this is for the news server call */
+		else if (addreq (hdr.dest, Nodealias))
+			To_us = NODE_LINK;	/* this is for the news server call */
+#endif
 #ifdef TTYCALL
 		else if (addreq (idest, Ttycall))
 			To_us = TTY_LINK;
 #endif
 		else if (addreq (idest, MyBBS))
 			To_us = ALIAS_LINK;
+		else if (addreq (idest, Mycall))
+			To_us = IFACE_LINK;
+#ifdef MBFWD
+		else if (addreq (idest, FWDCall))
+			To_us = IFACE_LINK;
+#endif
 	}
 	if (addreq (idest, iface->ipcall)) {
 		To_us |= IP_LINK;
@@ -580,10 +634,10 @@ int To_us = 0;		/* Is this a link to us ? */
 		nomark = (int) cross;	/* if not found here, then == 0 */
 	}
 #ifdef AXUI		/* K5JB: for axui, mustn't discard other UI frames */
-	if (Axui_sock != -1 && !mcast && !To_us) {	/* UI Session active? */
+	if (Axui_sock != -1 && !mcast) {	/* UI Session active? */
 		control = *bp->data & ~PF;
 		if (control == UI)
-			mcast = 1;	/* will fake it */
+			send2axui = 1;	/* will fake it */
 	}
 #endif
 
@@ -637,7 +691,11 @@ int To_us = 0;		/* Is this a link to us ? */
 				}
 			}
 		}
+#ifdef AXUI
+		axui_input (iface, NULLAX25, hdr.source, hdr.dest, bp, (mcast + send2axui));
+#else
 		free_p (bp);	/* Dispose if not forwarded */
+#endif
 		return;
 	}
 	/* If we reach this point, then the packet has passed all digis,
@@ -681,7 +739,7 @@ int To_us = 0;		/* Is this a link to us ? */
 				break;
 		}
 		if (ipp->funct != NULL)
-			(*ipp->funct) (iface, NULLAX25, hdr.source, hdr.dest, bp, mcast);
+			(*ipp->funct) (iface, NULLAX25, hdr.source, hdr.dest, bp, (mcast + send2axui));
 		else
 			free_p (bp);
 		return;
@@ -689,7 +747,7 @@ int To_us = 0;		/* Is this a link to us ? */
 	/* Everything from here down is connected-mode LAPB, so ignore
 	 * multicasts
 	 */
-	if (mcast) {
+	if ((mcast + send2axui) != 0) {
 		free_p (bp);
 		return;
 	}
@@ -745,7 +803,15 @@ int To_us = 0;		/* Is this a link to us ? */
 int
 sendframe (struct ax25_cb *axp, int cmdrsp, int ctl, struct mbuf *data)
 {
-	return axsend (axp->iface, axp->remote, axp->local, cmdrsp, ctl, data);
+int retval;
+
+	retval = axsend (axp->iface, axp->remote, axp->local, cmdrsp, ctl, data);
+
+	/* update the counters for messages out */
+	if (retval != -1)
+		incCounter(axp->iface);
+
+	return retval;
 }
 
 
