@@ -35,7 +35,7 @@
 #include "pool.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: ax25cmd.c,v 1.24 1997/06/28 16:46:13 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: ax25cmd.c,v 1.28 1997/09/14 14:37:46 root Exp root $";
 #endif
 
 static int doaxdest (int argc,char *argv[],void *p);
@@ -82,6 +82,7 @@ static int doax25window (int argc, char *argv[], void *p);
 static int doax25blimit (int argc, char *argv[], void *p);
 static int doax25maxwait (int argc, char *argv[], void *p);
 static void dobroadtick (void);
+static void displaycounters (struct iface *ifp, int addseparator);
 
 #ifdef TUTOR
 static int dotutorcall (int argc, char *argv[], void *p);
@@ -100,11 +101,6 @@ static int dottycall (int argc, char *argv[], void *p);
 extern char Ttycall[AXALEN];	/* the ttylink call in 'call' form */
 #endif
 
-#ifdef AXNEGOTIATION
-int AXNegotiation = 0;
-static int donegotiate (int argc, char *argv[], void *p);
-#endif
-
 extern int axheard_filter_flag;	/* in axheard.c */
 extern int lapbtimertype;
 extern char Tcall[AXALEN];
@@ -297,6 +293,7 @@ static struct cmds Axcmds[] =
 	{ "bcport",	dobcport,	0, 0, NULLCHAR },
 	{ "blimit",	doblimit,	0, 0, NULLCHAR },
 	{ "bctext",	dobctext,	0, 0, NULLCHAR },
+	{ "counters",	doaxcounters,	0, 0, NULLCHAR },
 	{ "dest",	doaxdest,	0, 0, NULLCHAR },
 	{ "digipeat",	dodigipeat,	0, 0, NULLCHAR },
 	{ "filter",	doaxfilter,	0, 0, NULLCHAR },
@@ -313,9 +310,6 @@ static struct cmds Axcmds[] =
 	{ "maxframe",	domaxframe,	0, 0, NULLCHAR },
 	{ "maxwait",	doaxmaxwait,	0, 0, NULLCHAR },
 	{ "mycall",	domycall,	0, 0, NULLCHAR },
-#ifdef AXNEGOTIATION
-	{ "negotiation",donegotiate,	0, 0, NULLCHAR },
-#endif
 #ifdef TUTOR
 	{ "newscall",	donewscall,	0, 0, NULLCHAR },
 #endif
@@ -353,16 +347,6 @@ doax25 (int argc, char *argv[], void *p)
 
 
 
-#ifdef AXNEGOTIATION
-int
-donegotiate (int argc, char *argv[], void *p OPTIONAL)
-{
-	return setbool (&AXNegotiation, "Use AX25 Negotiation Extensions", argc, argv);
-}
-#endif
-
-
-
 int
 dosmartroute (int argc, char *argv[], void *p OPTIONAL)
 {
@@ -428,8 +412,7 @@ struct iface *ifa;
 #else
 		ax_bc (ifa);
 #endif
-		stop_timer (&Broadtimer);	/* in case it's already running */
-		start_timer (&Broadtimer);	/* and fire it up */
+		start_detached_timer (&Broadtimer);	/* stops if running and fire it up */
 	}
 	return 0;
 }
@@ -478,7 +461,7 @@ dobcint (int argc, char *argv[], void *p OPTIONAL)
 	Broadtimer.func = (void (*)(void *)) dobroadtick;	/* what to call on timeout */
 	Broadtimer.arg = NULLCHAR;	/* dummy value */
 	set_timer (&Broadtimer, atoi (argv[1]) * 1000L);	/* set timer duration */
-	start_timer (&Broadtimer);	/* and fire it up */
+	start_detached_timer (&Broadtimer);	/* and fire it up */
 	return 0;
 }
 
@@ -530,7 +513,7 @@ struct iface *ifa;
 	}
 
 	/* Restart timer */
-	start_timer (&Broadtimer);
+	start_detached_timer (&Broadtimer);
 }
 
 
@@ -796,6 +779,71 @@ char tmp2[AXBUF];
 
 
 
+static void
+displaycounters (struct iface *ifp, int addseparator)
+{
+struct ax25_counters *ac;
+
+	ac = &ifp->axcnt;
+#if 0
+	if (addseparator)
+		tprintf ("---------------------------------------------------------------------\n");
+	tprintf ("\nAX25 Counters for Interface: %s\n\n", ifp->name);
+
+	tprintf ("Total Frames: %10ld in %10ld out\n Data Frames: %10ld in %10ld out\n",
+		ac->msgin, ac->msgout, ac->datain, ac->dataout);
+	tprintf ("    Segments: %10ld in %10ld out %10ld errors\n        RNRs: %10ld in %10ld out\n",
+		ac->segin, ac->segout, ac->segerr, ac->rnrin, ac->rnrout);
+	tprintf ("        REJs: %10ld in %10ld out\n     Retries: %10ld\n   FRMR Recv: %10ld\n\n",
+		ac->rejin, ac->rejout, ac->retries, ac->frmerr);
+#else
+	if (!addseparator)
+		tprintf ("\nAX25 Counters:\n\n"
+			"       <-Frame Count-> <-Data Frames-> <-Segmented-> <--RNR--> <-REJ->     FRMR\n"
+			"name        in     out      in     out   in  out err   in  out  in out Rtry  in\n"
+			"------ ------- ------- ------- ------- ---- ---- --- ---- ---- --- --- ---- ---\n");
+	else if ((addseparator % 5) == 0)
+		tprintf ("-------------------------------------------------------------------------------\n");
+	
+	tprintf ("%-7.7s%7ld %7ld %7ld %7ld %4ld %4ld %3ld %4ld %4ld %3ld %3ld %4ld %3ld\n",
+		ifp->name, ac->msgin, ac->msgout, ac->datain, ac->dataout, ac->segin,
+		ac->segout, ac->segerr, ac->rnrin, ac->rnrout, ac->rejin, ac->rejout,
+		ac->retries, ac->frmerr);
+#endif
+}
+
+
+
+/* Display AX.25 link level counters */
+int
+doaxcounters (int argc, char *argv[], void *p OPTIONAL)
+{
+struct iface *ifp;
+int count = 0;
+int k;
+
+	if (argc < 2) {
+		for (ifp = Ifaces; ifp != NULLIF; ifp = ifp->next)	{
+			if (ifp->iftype->type == CL_AX25)	{
+				displaycounters (ifp, count);
+				count++;
+			}
+		}
+		return 0;
+	}
+	for (k = 1; k < argc; k++)	{
+		if ((ifp = if_lookup (argv[k])) == NULLIF) {
+			tprintf (Badinterface, argv[k]);
+			return 1;
+		}
+		displaycounters (ifp, count);
+		count++;
+	}
+	return 0;
+}
+
+
+
 /* Dump one control block */
 void
 st_ax25 (register struct ax25_cb *axp)
@@ -1276,11 +1324,8 @@ char tmp[AXBUF];
 		return 1;
 	}
 	fsocket.sax_family = AF_AX25;
+
 	memcpy (fsocket.ax25_addr, AXuser, AXALEN);
-#ifdef AXNEGOTIATION
-	if (AXNegotiation)
-		fsocket.ax25_addr[AXALEN - 1] &= ~AXN;
-#endif
 	(void) bind (sp->s, (char *) &fsocket, sizeof (struct sockaddr_ax));
 
 	(void) setcall (fsocket.ax25_addr, argv[2]);
