@@ -34,7 +34,7 @@
 #include "color.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: bmutil.c,v 1.38 1997/05/31 14:42:31 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: bmutil.c,v 1.40 1997/08/19 01:19:22 root Exp root $";
 #endif
 
 #ifndef UNIX
@@ -137,7 +137,7 @@ int i;
 #endif
 
 	if (m->mfile != NULLFILE) {
-		fclose (m->mfile);
+		(void) fclose (m->mfile);
 		wasit = m->nmsgs;
 	}
 	sprintf (buf, "%s/%s.txt", Mailspool, m->area);
@@ -180,7 +180,7 @@ static int
 readnotes (register struct mbx *m, int already)
 {
 register FILE *fp;
-char mailbox[100];
+char mailbox[128];
 long size;
 register int i, isit;
 register struct let *cmsg;
@@ -197,7 +197,7 @@ struct let *new;
 		new = (struct let *) mallocw ((unsigned long) size + sizeof (struct let));
 
 		(void) fread (&new[1], (unsigned) size, 1, fp);
-		fclose (fp);
+		(void) fclose (fp);
 		if (already && !strcmp (m->area, m->name))
 			memcpy (new, m->mbox, (unsigned) (already + 1) * sizeof (struct let));
 
@@ -833,7 +833,7 @@ long msgid;
 	sprintf (buf, "%s/control/%s.ctl", Mailspool, cl->area);
 	(void) remove (buf);
 
-	fclose (cl->mfile);
+	(void) fclose (cl->mfile);
 	sprintf (buf, "%s/%s.txt", Mailspool, cl->area);
 	sprintf (buf2, "%s/%s.bak", Mailspool, cl->area);
 	(void) remove (buf2);	/* delete it here, just in case! */
@@ -844,7 +844,7 @@ long msgid;
 		return;
 	}
 	if ((nfile = subdir_fopen (buf, WRITE_TEXT)) == NULLFILE) {
-		fclose (cl->mfile);
+		(void) fclose (cl->mfile);
 		rmlock (Mailspool, cl->area);
 		free (cl);
 		return;
@@ -912,7 +912,7 @@ long msgid;
 			kwait (NULL);	/* dont want no damaged files */
 			if (ferror (nfile)) {
 				(void) fclose (nfile);
-				fclose (cl->mfile);
+				(void) fclose (cl->mfile);
 				rmlock (Mailspool, cl->area);
 				free (cl);
 				return;
@@ -933,7 +933,7 @@ long msgid;
 		(void) unlink (buf);
 	} else
 		(void) fclose (nfile);
-	fclose (cl->mfile);
+	(void) fclose (cl->mfile);
 	(void) remove (buf2);	/* leave it around, for now! NOT! */
 	rmlock (Mailspool, cl->area);
 	free (cl->mbox);
@@ -1000,8 +1000,8 @@ struct clparms *cl;
 		cl->hostsize = &cl->_hostsize;
 	}
 	cl->isbbs = (m->sid & MBX_SID);
-	strcpy (cl->name, m->name);
-	strcpy (cl->area, m->area);
+	strncpy (cl->name, m->name, 20);
+	strncpy (cl->area, m->area, 64);
 	cl->mfile = m->mfile;
 	cl->nmsgs = m->nmsgs;
 	cl->mbox = m->mbox;
@@ -1034,12 +1034,12 @@ FILE *fp;
 				*cp = '\0';
 			(void) nntp_name_expansion (buf);
 			if (stricmp (name, buf) == 0) {	/* found it */
-				fclose (fp);
+				(void) fclose (fp);
 				return 1;
 			}
 		}
 	}
-	fclose (fp);
+	(void) fclose (fp);
 	return 0;
 }
 
@@ -1070,7 +1070,7 @@ lockit (struct mbx *m)
 int c, cnt = 0;
 
 	while (mlock (Mailspool, m->area)) {
-		(void) kpause (1000L / MSPTICK);	/* Wait one second */
+		(void) kpause (1000L);	/* Wait one second */
 		if (++cnt == 10) {
 			char buf[80];
 
@@ -1394,7 +1394,8 @@ long msgid;
 						break;
 #endif
 					case TO:
-						theto = strdup ((char *) &buf[strlen (Hdrs[TO])]);	/*lint !e616 */
+						theto = strdup ((char *) &buf[strlen (Hdrs[TO])]);
+						/* fall through */
 					case FROM:
 						{
 							char *nm;
@@ -1475,7 +1476,7 @@ long msgid;
 			subj[2] = ':';
 			subj[3] = ' ';
 			mm->stype = 'P';
-			strcpy (mm->name, m->name);
+			strncpy (mm->name, m->name, 20);
 			if ((mm->tfile = tmpfile ()) != NULLFILE) {
 				struct list *cclist = NULLLIST;
 				char fullfrom[MBXLINE];
@@ -1489,7 +1490,7 @@ long msgid;
 				fprintf (mm->tfile, "Message received - Originally addressed to: '%s'\n", theto);
 				fseek (mm->tfile, 0L, 0);	/* rewind file */
 				(void) queuejob (mm->tfile, Hostname, cclist, fullfrom);
-				fclose (mm->tfile);
+				(void) fclose (mm->tfile);
 				del_list (cclist);
 				smtptick (NULL);	/* wake up SMTP to send mail */
 			}
@@ -1581,9 +1582,11 @@ long size;
 			case SUBJECT:
 				trimright (m->line);
 				if (strlen (m->line) > 11 && !strnicmp (&m->line[9], "Re:", 3))
-					strcpy (subject, &m->line[9]);
-				else
-					sprintf (subject, "Re: %s", &m->line[9]);
+					strncpy (subject, &m->line[9], MBXLINE);
+				else	{
+					strcpy (subject, "Re: ");
+					strncpy (&subject[4], &m->line[9], MBXLINE - 4);
+				}
 				break;
 			case FROM:
 				trimright (m->line);
@@ -1617,7 +1620,7 @@ long size;
 				k = (int) (strlen (cp) + 1);
 				m->line[min(k, MBXLINE)] = '\0';	/* add extra null at end */
 				for (;;) {
-					if ((cp = getaddress (cp, lastheader == header || cp != m->line)) == NULLCHAR)	/*lint !e730 */
+					if ((cp = getaddress (cp, lastheader == header || cp != m->line)) == NULLCHAR)
 						break;
 					(void) addlist (cclist, cp, 0, cp);
 					/* skip to next address, if any */
@@ -1644,7 +1647,7 @@ long size;
 		free (msgid);
 		free (date);
 	}
-	strcpy (m->line, subject);
+	strncpy (m->line, subject, MBXLINE);
 	return 0;
 }
 
@@ -1675,7 +1678,7 @@ int found = 0;
 		/* Add this user as first one */
 		sprintf (buf, "%s 0\n", m->name);
 		fputs (buf, Alog);
-		fclose (Alog);
+		(void) fclose (Alog);
 		return;
 	}
 	/*Find user in the usr file for this area*/
@@ -1700,7 +1703,7 @@ int found = 0;
 		sprintf (buf, "%s 0\n", m->name);
 		fputs (buf, Alog);
 	}
-	fclose (Alog);
+	(void) fclose (Alog);
 	return;
 }
 
@@ -1772,9 +1775,9 @@ int doit = 0;
 				*cp = ' ';	/* restore the space !*/
 			fputs (buf, Alog);
 		}
-		fclose (tfile);
+		(void) fclose (tfile);
 		unlink (tmpname);
-		fclose (Alog);
+		(void) fclose (Alog);
 	}
 	return;
 }
@@ -1854,7 +1857,7 @@ FILE *fp;
 	fseek (fp, 0L, 2);
 	cnt = ftell (fp);
 	/*	cnt = filelength (fileno(fp)); */
-	fclose (fp);
+	(void) fclose (fp);
 	return cnt;
 }
 
@@ -1866,7 +1869,7 @@ static void
 mfclose (struct mbx *m)
 {
 	if (m->mfile != NULLFILE)
-		fclose (m->mfile);
+		(void) fclose (m->mfile);
 	m->mfile = NULLFILE;
 #ifdef SETVBUF
 	free (m->stdoutbuf);
@@ -2011,7 +2014,7 @@ char *cp, buf[FILE_PATH_SIZE];
 			ca = na;
 		}
 	}
-	fclose (fp);
+	(void) fclose (fp);
 }
 
 
