@@ -31,7 +31,7 @@
 
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: bootpdip.c,v 1.15 1997/06/28 16:46:13 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: bootpdip.c,v 1.17 1997/09/07 21:18:28 root Exp root $";
 #endif
 
 #define E_NOMEM 3101
@@ -332,7 +332,7 @@ da_runtask (void *p OPTIONAL)
 	stop_timer(&da_timer);
 	da_task();
 	set_timer(&da_timer,TIME_RWAIT*1000L);
-	start_timer(&da_timer);
+	start_detached_timer(&da_timer);
 }
 
 /*
@@ -810,7 +810,7 @@ uint32 rend			/* Last address in range. */
 		 *
  		 * Allocate the memory I need.
  		 */
-		dr = (struct drange_desc *) calloc(1, sizeof(*dr));
+		dr = (struct drange_desc *) callocw(1, sizeof(*dr));
 		if(dr == NULLDRANGE) 
 			return E_NOMEM;
 	} else if((dr->dr_start != rstart) || (dr->dr_end != rend)) 
@@ -821,7 +821,7 @@ uint32 rend			/* Last address in range. */
 
 
 	rcount = (long) ((rend - rstart) + 1);
-	da = (struct daddr *) calloc(1,(sizeof (*da) + (uint32) iface->iftype->hwalen) * (uint32) rcount);
+	da = (struct daddr *) callocw(1,(sizeof (*da) + (uint32) iface->iftype->hwalen) * (uint32) rcount);
 	if(da == NULLDADDR) 
 		return E_NOMEM;
 
@@ -859,7 +859,7 @@ uint32 rend			/* Last address in range. */
 		set_timer(&da_timer,TIME_RWAIT*1000L);
        		da_timer.func = da_runtask;
 	       	da_timer.arg = (void *) 0;
-		start_timer(&da_timer);
+		start_detached_timer(&da_timer);
 	}
 	q_enqueue(&rtabq,(struct q_elt *)dr);
 	da_enter_critical(dr);	/* Start reclaiming some of these addresses. */
