@@ -26,7 +26,7 @@ void kwait (void *);
 
 
 #if !defined(_lint) && !defined(TEST)
-static char rcsid[] OPTIONAL = "$Id: browser.c,v 1.9 1997/05/24 13:25:09 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: browser.c,v 1.12 1997/09/07 00:31:16 root Exp root $";
 #endif
 
 #ifdef BROWSER
@@ -323,7 +323,7 @@ redirection:
 			clear_browser (br);
 			br->current_url = build_url (br, type, host, port, component);
 			parse_url (br, fp, 0);
-			fclose (fp);
+			(void) fclose (fp);
 			return 0;
 		}
 		return -1;
@@ -343,7 +343,7 @@ redirection:
 	if ((retval = http_url_to_file (br, refered, host, port, component, br->auth, fp, post, postmode, enctype, 0)) == 0)
 		parse_url (br, fp, 1);
 
-	fclose (fp);
+	(void) fclose (fp);
 	free (refered);
 
 	/* check to see if there was a redirection to a new URL */
@@ -379,7 +379,7 @@ redirection:
 static char *
 build_url (struct browser *br, char *type, char *host, int port, char *component)
 {
-char buf[512], portstr[10], namebuf[128];
+char buf[512], portstr[16], namebuf[128];
 char *usehost;
 
 	if (port != 80)
@@ -694,7 +694,7 @@ char *buf;
 int len, k;
 struct url_links *lnk;
 
-	buf = (char *) malloc (((strlen(str) > SCREENwidth) ? strlen(str) : SCREENwidth) + 2);
+	buf = (char *) mallocw (((strlen(str) > SCREENwidth) ? strlen(str) : SCREENwidth) + 2);
 	if (centerit)	{
 		len = (int) (SCREENwidth - strlen (str));
 		if (len < 0)
@@ -1516,7 +1516,7 @@ do_a_p:						break_line (br, p);
 
 #ifndef TEST
 	if (savefp != NULLFILE)
-		fclose (savefp);
+		(void) fclose (savefp);
 #endif
 
 	/* if both an ISINDEX and an MARQUEE found, ISINDEX wins the status line */
@@ -1535,14 +1535,57 @@ int theindex, c;
 char buf[1024], *cp;
 struct tag *tag;
 int done = 0;
+int eat_semicolon = 0;
 
 	theindex = 0;
 	buf[0] = 0;
 
 	do	{
 		while ((c = fgetc (fp)) != '<')	{
+#if 0
 			if (c != '\n')
 				buf[theindex++] = (char) c;
+#else
+			switch (c)	{
+				case '&':
+					switch (tolower (fgetc (fp)))	{
+						case 'l':
+							if (tolower (fgetc (fp)) == 't')	{
+								buf[theindex++] = '<';
+								eat_semicolon = 1;
+							}
+							break;
+						case 'g':
+							if (tolower (fgetc (fp)) == 't')	{
+								buf[theindex++] = '>';
+								eat_semicolon = 1;
+							}
+							break;
+						case 'n':
+							if ((tolower (fgetc (fp)) == 'b') &&
+							    (tolower (fgetc (fp)) == 's') &&
+							    (tolower (fgetc (fp)) == 'p'))	{
+							    	buf[theindex++] = ' ';
+							    	eat_semicolon = 1;
+							}
+							break;
+						default:
+							break;
+					}
+					break;
+				case '\n':
+					break;	/* nothing, eat it */
+				case ';':
+					if (eat_semicolon)	{
+						eat_semicolon = 0;
+						break;
+					}
+					/* else, fall-through */
+				default:
+					buf[theindex++] = (char) c;
+					eat_semicolon = 0;
+			}
+#endif
 		}
 		tag = parse_tag (fp);
 		switch (opcode_lookup (supported_opcodes, tag->name))	{
@@ -1941,7 +1984,7 @@ FILE *fp;
 		trim_trailing_white (name);
 		trim_trailing_white (value);
 		fprintf (fp, "%s %s %ld %s %s\n", host, path, expires, name, value);
-		fclose (fp);
+		(void) fclose (fp);
 	}
 }
 
@@ -1991,7 +2034,7 @@ time_t expires;
 			/* need to add code to consolidate multiple cookies to 1 line */
 			usprintf (s, "Cookie: %s=%s\n", cp, tmp);
 		}
-		fclose (fp);
+		(void) fclose (fp);
 	}
 }
 
@@ -2199,7 +2242,7 @@ FILE *fp;
 		if ((cp = strrchr (file, '/')) != NULLCHAR)
 			*cp++ = 0;
 		fprintf (fp, "anonymous\nwwwuser@%s\nbinary\ncd %s\nget %s\nbye\n", Hostname, (char *) file, cp);
-		fclose (fp);
+		(void) fclose (fp);
 
 		Curproc->input = Command->input;
 		args[0] = strdup ("ftp");
@@ -2269,7 +2312,7 @@ char subject[128];
 			fputs (buf, fp);
 			kwait (NULL);
 		}
-		fclose (fp);
+		(void) fclose (fp);
 
 		args[0] = strdup ("sendmail");
 		args[1] = addr;
@@ -2351,8 +2394,10 @@ struct sockaddr_in fsocket;
 		return -1;	/* shouldn't happen */
 		
 	tmp = strdup (url);
-	if (disect_url (tmp, type, host, &port, component))
+	if (disect_url (tmp, type, host, &port, component))	{
+		(void) fclose (fp);
 		return 0;
+	}
 	if (!*host)	{
 		strncpy (host, component, 100);
 		strcpy (component, "/");
@@ -2361,12 +2406,15 @@ struct sockaddr_in fsocket;
 
 	if (strcasecmp (type, "http"))	{
 		/* don't do anything, for now */
+		(void) fclose (fp);
 		return 0;
 	}
 
 	s = http_connect (host, port, &fsocket);
-	if (s == -1)
+	if (s == -1)	{
+		(void) fclose (fp);
 		return -1;
+	}
 
 	if ((cp = strchr (component, '#')) != NULLCHAR)
 		*cp = 0;
@@ -2399,7 +2447,7 @@ struct sockaddr_in fsocket;
 
 		/* else, bitbucket any other headers, for now */
 	}
-	fclose (fp);
+	(void) fclose (fp);
 	return 1;
 }
 
@@ -2470,7 +2518,7 @@ char *bareurl, *origurl;
 							doit = 1;
 					}
 				}
-				fclose (fp);
+				(void) fclose (fp);
 			}
 			/* if doit != 0, it is new or newer */
 			if (doit)	{
@@ -2510,11 +2558,11 @@ char *bareurl, *origurl;
 							*cp = ' ';
 							fputs (buf, fpout);
 						}
-						fclose (fp);
+						(void) fclose (fp);
 					}
 					if (!found)
 						fprintf (fpout, "%s %s\n", bareurl, modified);
-					fclose (fpout);
+					(void) fclose (fpout);
 				}
 				(void) unlink (CheckedURLs);
 				(void) rename (tname, CheckedURLs);
@@ -2543,7 +2591,7 @@ struct browser *br;
 char buf[200], *cp;
 FILE *fp;
 
-	br = (struct browser *) calloc (1, sizeof (struct browser));
+	br = (struct browser *) callocw (1, sizeof (struct browser));
 	fp = tmpfile ();
 	if (fp == NULLFILE || br == NULLBROWSER)
 		return NULLCHAR;	/* shouldn't happen */
@@ -2555,7 +2603,7 @@ FILE *fp;
 		parse_url (br, fp, 1);
 	kalarm (0);
 
-	fclose (fp);
+	(void) fclose (fp);
 	cp = br->marquee;
 	br->marquee = NULLCHAR;
 	clear_browser (br);
@@ -3069,7 +3117,7 @@ struct formlink *fl;
 	brow.current_url = strdup ("/index.html");
 	fp = fopen ("/tmp/index.html", "r");
 	parse_url (&brow, fp, 0);
-	fclose (fp);
+	(void) fclose (fp);
 
 	printf ("Current URL: %s\nHost: %s\nPort: %d\nUrlLines: %d\n", brow.current_url,
 		brow.host, brow.port, brow.lines_in_url);
