@@ -49,7 +49,7 @@
 #endif
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: config.c,v 1.41 1997/05/24 22:13:02 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: config.c,v 1.46 1997/09/14 14:37:46 root Exp root $";
 #endif
 
 #ifndef TRACE
@@ -79,6 +79,7 @@ static void axnr (struct iface * iface, struct ax25_cb * axp, char *src,
 
 struct mbuf *Hopper;
 unsigned Nsessions = NSESSIONS;
+extern int cmdparse_newproc_type;
 
 #ifndef UNIX
 /* Free memory threshold, below which things start to happen to conserve
@@ -374,6 +375,9 @@ struct cmds Cmds[] =
 #ifdef ALLCMD
 	{ "copy",	docopy,		0,	3,	"copy <file> <newfile>" },
 #endif
+#if defined(BETA) && defined(ALPHATEST)	/* not for production usage - diagnostic only */
+	{ "crashprotect", docrashprot,	0, 	0, 	NULLCHAR },
+#endif
 #ifdef CRONTAB
 	{ "crontab",	docrontab,	0,	0,	NULLCHAR },
 #endif
@@ -444,6 +448,7 @@ struct cmds Cmds[] =
 #ifdef FORTH
 	{ "forth",	doforth,	0,	0,	NULLCHAR },
 #endif
+	{ "fstat",	dofstat,	0,	0,	NULLCHAR },
 #ifdef ALLSESSIONS
 	{ "ftp",	doftp,		2048,	2,	"ftp <address> [[inputfile] [pause]]" },
 	{ "ftype",	doftype,	0,	0,	NULLCHAR },
@@ -546,6 +551,9 @@ struct cmds Cmds[] =
 #if defined(NNTP) || defined(NNTPS)
 	{ "nntp",	donntp,		0,	0,	NULLCHAR },
 #endif
+#ifdef NODECALL
+	{ "node",	donode,		0,	0,	NULLCHAR },
+#endif
 #ifdef NRS
 	{ "nrstat",	donrstat,	0,	0,	NULLCHAR },
 #endif
@@ -558,7 +566,7 @@ struct cmds Cmds[] =
 #endif
 	{ "ping",	doping,		0,	2,
 	 "ping <hostid> [<length> [<interval> [incflag]]]" },
-#if defined(UNIX) && defined(BETA)
+#ifdef PIPECMD
 	{ "pipe",	dopipe,		0,	2,	"pipe cmdname" },
 #endif
 #ifdef	PI
@@ -576,6 +584,7 @@ struct cmds Cmds[] =
 #ifdef PPP
 	{ "ppp",	doppp_commands,	0,	0,	NULLCHAR },
 #endif
+	{ "profile",	doprofile,	0,	0,	NULLCHAR },
 	{ "prompt",	doprompt,	0,	0,	NULLCHAR },
 	{ "ps",		ps,		0,	0,	NULLCHAR },
 #ifndef UNIX
@@ -600,8 +609,11 @@ struct cmds Cmds[] =
 	{ "record",	dorecord,	0,	0,	NULLCHAR },
 #endif
 	{ "register",	doregister,	0,	0,	NULLCHAR },
-	{ "remote",	doremote,	0,	3,
-       "remote [-p port] [-k key] [-a kickaddr] <address> exit|reset|kick" },
+	{ "remote",	doremote,	0,	3,	/*lint -save -e786 */
+	"remote [-p <port#>] [-a <kickaddress>] <hostname> kickme\n"
+	"   or: remote [-p <port#>] -k <key> <hostname> reset|exit\n"
+	"   or: remote [-p <port#>] -k <key> -r <destIPaddr>[/<bits>] <hostname> add|drop\n"
+	"   or: remote -s <password>\n   or: remote -g <gatewaypassword>" },	/*lint -restore */
 #ifdef ALLCMD
 	{ "rename",	dorename,	0,	3,	"rename <oldfile> <newfile>" },
 #endif
@@ -639,6 +651,7 @@ struct cmds Cmds[] =
 #ifdef SHELL
 	{ "shell",	doshell,	0,	0,	NULLCHAR },
 #endif
+	{ "shutdown",	doshutdown,	256,	3,	"shutdown <delay | \"now\"> \"reason\"" },
 	{ "skick",	dokicksocket,	0,	2,	"skick <socket#>" },
 #ifdef SCRIPTING
 	{ "sleep",	dosleep,	0,	0,	NULLCHAR },
@@ -689,6 +702,7 @@ struct cmds Cmds[] =
 	{ "thirdparty",	dothirdparty,	0,	0,	NULLCHAR },
 #endif
 	{ "time",	dosystime,	0,	0,	NULLCHAR },
+	{ "timers",	dotimers,	0,	0,	NULLCHAR },
 	{ "tip",	dotip,		0,	2,	"tip <iface (asyc only!)>" },
 #ifdef  TRACE
 	{ "trace",	dotrace,	0,	0,	NULLCHAR },
@@ -790,14 +804,13 @@ struct cmds Attab[] =
 		" <label> <bufsize> <mtu> <speed> [options]" },
 #endif /* ASY */
 
-#ifdef	KISS
-/* Multi-port KISS interfaer piggy-backing on ASY interface. */
-	{ "kiss", kiss_attach, 0, 4, "attach kiss <asy_interface_label> <port> <label> [mtu]" },
-#endif /* KISS */
+#ifdef KISS
+	/* fake ax25 loopback interface */
+	{ "axloop", axloop_attach, 0, 1, "attach axloop" },
+#endif
 
-#ifdef	PC100
-/* PACCOMM PC-100 8530 HDLC adaptor */
-	{ "pc100", pc_attach, 0, 8, "attach pc100 <address> <vector> ax25 <label> <buffers> <mtu> <speed> [ip_addra] [ip_addrb]" },
+#ifdef AXIP
+	{ "axip", axip_attach, 0, 4, "attach axip <name> <mtu> <remote-hostid> [callsign] [rmtcall]" },
 #endif
 
 #ifdef	DRSI
@@ -805,6 +818,9 @@ struct cmds Attab[] =
 	{ "drsi", dr_attach, 0, 8, "attach drsi <address> <vector> ax25 <label> <bufsize> <mtu> <chan a speed> <chan b speed> [ip addr a] [ip addr b]" },
 #endif
 
+/* dummy interfaces */
+	{ "dummy", dummy_attach, 0, 3, "attach dummy <label> <ip_addr>" },
+
 #ifdef	EAGLE
 /* EAGLE RS-232C 8530 HDLC adaptor */
 	{ "eagle", eg_attach, 0, 8, "attach eagle <address> <vector> ax25 <label> <buffers> <mtu> <speed> [ip_addra] [ip_addrb]" },
@@ -814,6 +830,20 @@ struct cmds Attab[] =
 	{ "hapn", hapn_attach, 0, 8, "attach hapn <address> <vector> ax25 <label> <rx bufsize> <mtu> csma|full [ip_addr]" },
 #endif
 
+#ifdef	HS
+/* Special high speed driver for DRSI PCPA or Eagle cards */
+	{ "hs", hs_attach, 0, 7, "attach hs <address> <vector> ax25 <label> <buffers> <mtu> <txdelay> <persistence> [ip_addra] [ip_addrb]" },
+#endif
+
+#ifdef linux
+	{ "kernel", krnlif_attach, 0, 2, "attach kernel <iface> [nopromisc]" },
+#endif
+
+#ifdef	KISS
+/* Multi-port KISS interfaer piggy-backing on ASY interface. */
+	{ "kiss", kiss_attach, 0, 4, "attach kiss <asy_interface_label> <port> <label> [mtu]" },
+#endif /* KISS */
+
 #ifdef NETROM
 /* fake netrom interface */
 	{ "netrom", nr_attach, 0, 1, "attach netrom [ip_addr]" },
@@ -824,9 +854,9 @@ struct cmds Attab[] =
 	{ "packet", pk_attach, 0, 4, "attach packet <int#> <label> <buffers_in_kb> <mtu> [ip_addr]" },
 #endif
 
-#ifdef	HS
-/* Special high speed driver for DRSI PCPA or Eagle cards */
-	{ "hs", hs_attach, 0, 7, "attach hs <address> <vector> ax25 <label> <buffers> <mtu> <txdelay> <persistence> [ip_addra] [ip_addrb]" },
+#ifdef	PC100
+/* PACCOMM PC-100 8530 HDLC adaptor */
+	{ "pc100", pc_attach, 0, 8, "attach pc100 <address> <vector> ax25 <label> <buffers> <mtu> <speed> [ip_addra] [ip_addrb]" },
 #endif
 
 #ifdef	PI
@@ -838,10 +868,6 @@ struct cmds Attab[] =
 	{ "scc", scc_attach, 0, 7, "attach scc <devices> init <addr> <spacing> <Aoff> <Boff> <Dataoff>\n   <intack> <vec> [p]<clock> [hdwe] [param]\n"
 		"attach scc <chan> slip|kiss|nrs|ax25 <label> <mtu> <speed> <bufsize> [call]" },
 #endif
-
-#ifdef AXIP
-	{ "axip", axip_attach, 0, 4, "attach axip <name> <mtu> <remote-hostid> [callsign] [rmtcall]" },
-#endif
 	{ NULLCHAR, NULL, 0, 4, NULLCHAR }
 };
 /*lint -restore */
@@ -1083,8 +1109,14 @@ static struct cmds Startcmds[] =
 #ifdef ALLSERV
         { "daytime",    daytime1,       256,    0, NULLCHAR },
         { "discard",    dis1,           256,    0, NULLCHAR },
+#ifdef ENCAP
+	{ "dynamic",	routecli1,	256,	2, "start dynamic <hostname> [<port>]" },
+#endif
         { "echo",       echo1,          256,    0, NULLCHAR },
 #endif
+#ifdef MAILBOX
+	{ "fbbtelnet",	fbbtelnet1,	256,	0, NULLCHAR },
+#endif /* MAILBOX */
 #ifdef FIFOSERVER
         { "fifo",       fifo1,          256,    0, NULLCHAR },
 #endif
@@ -1127,6 +1159,9 @@ static struct cmds Startcmds[] =
         { "quote",      quote1,         256,    0, NULLCHAR },
 #endif
         { "remote",     rem1,           768,    0, NULLCHAR },
+#ifdef ROUTESERVER
+	{ "router",	route1,		256,	0, NULLCHAR },
+#endif
 #ifdef  RIP
         { "rip",        doripinit,      0,      0, NULLCHAR },
 #endif
@@ -1189,8 +1224,14 @@ static struct cmds Stopcmds[] =
 #ifdef ALLSERV
 	{ "daytime",	daytime0,	0, 0, NULLCHAR },
 	{ "discard",	dis0,		0, 0, NULLCHAR },
+#ifdef ENCAP
+	{ "dynamic",	routecli0,	0, 0, NULLCHAR },
+#endif
 	{ "echo",	echo0,		0, 0, NULLCHAR },
 #endif
+#ifdef	MAILBOX
+	{ "fbbtelnet",	fbbtelnet0,	0, 0, NULLCHAR },
+#endif
 #ifdef FIFOSERVER
         { "fifo",       fifo0,          0, 0, NULLCHAR },
 #endif
@@ -1233,6 +1274,9 @@ static struct cmds Stopcmds[] =
 	{ "quote",	quote0,		0, 0, NULLCHAR },
 #endif
 	{ "remote",	rem0,		0, 0, NULLCHAR },
+#ifdef ROUTESERVER
+	{ "router",	route0,		0, 0, NULLCHAR },
+#endif
 #ifdef  RIP
 	{ "rip",	doripstop,	0, 0, NULLCHAR },
 #endif
@@ -1291,7 +1335,9 @@ int insave = 0, outsave = 0, swapped = 0;
 		Curproc->output = Cmdpp->output;
 		swapped = 1;
 	}
+	cmdparse_newproc_type = PTYPE_SERVER;
 	(void) subcmd (Startcmds, argc, argv, p);
+	cmdparse_newproc_type = PTYPE_NORMAL;
 
 	if (swapped) {
 		Curproc->input = insave;
@@ -1322,12 +1368,17 @@ net_route (struct iface *ifp, int type, struct mbuf *bp)
 struct mbuf *nbp;
 struct phdr phdr;
 
+	if (ifp == NULLIF)	{
+		kwait (NULL);
+		return 0;
+	}
+
 	phdr.iface = ifp;
 	phdr.type = (int16) type;
 
-	if ((nbp = pushdown (bp, sizeof (phdr))) == NULLBUF) {
+	if ((nbp = pushdown (bp, sizeof (phdr))) == NULLBUF)
 		return -1;
-	}
+
 	memcpy (&nbp->data[0], (char *) &phdr, sizeof (phdr));
 	enqueue (&Hopper, nbp);
 	/* Especially on slow machines, serial I/O can be quite
@@ -1336,6 +1387,7 @@ struct phdr phdr;
 	 * go on toward its ultimate destination. [Karn]
 	 */
 	kwait (NULL);
+
 	return 0;
 }
 
@@ -1351,6 +1403,7 @@ int i_state;
 struct mbuf *bbp;
 #endif
 
+	server_disconnect_io ();
 loop:
 	while (Hopper == NULLBUF)
 		kwait (&Hopper);
