@@ -7,37 +7,39 @@
  * No copyright, copyleft, copysideways, copytwisted, ....
  */
 
+#include <sys/types.h>
+#include <errno.h>
+#include <sys/param.h>
+#include "system.h"
 /*lint -save -e659 */
+#ifdef HAVE_NCURSES_H
 #include <ncurses.h>
+#else
+#include <curses.h>
+#endif
 /*lint -restore */
 #ifndef MSDOS
 #include <term.h>
 #endif
-#include <errno.h>
-#ifdef BSD
-#undef BSD
-#endif
-#include <sys/param.h>
 #undef FALSE
 #undef TRUE
 
 #ifndef _lint
-static char rcsid[] OPTIONAL = "$Id: curses.c,v 1.18 1997/05/24 12:35:47 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: curses.c,v 1.20 1997/08/19 01:19:22 root Exp root $";
 #endif
 
-/* This nonsense WILL go away... */
-#if defined(linux) || defined(BSD) || defined(sun)
-# define _tmarg _regtop
-# define _bmarg _regbottom
 # ifndef	wbkgdset
 #  define wbkgdset(w,attr) ((w)->_bkgd = (attr))
 # endif
 # ifndef getattrs
 #  define getattrs(w) ((w)->_attrs)
 # endif
-#endif
 
 
+#ifdef memcpy
+#undef memcpy
+#endif
+
 #include "global.h"
 #include "hardware.h"
 #include "proc.h"
@@ -46,13 +48,18 @@ static char rcsid[] OPTIONAL = "$Id: curses.c,v 1.18 1997/05/24 12:35:47 root Ex
 /*lint -e123 */
 #include "sessmgr.h"
 
+/* This nonsense WILL go away... */
+#ifdef NO_CURSES_TMARG
+# define _tmarg _regtop
+# define _bmarg _regbottom
+#endif
+
 #ifdef SM_CURSES
 
 #ifdef SCREENSAVER
 extern void sskick (void);
 #endif
 
-extern int isatty (int);
 extern struct sessmgr_sw curses_sessmgr; /* forward declaration */
 
 static int Suspense, initted;
@@ -89,7 +96,7 @@ struct curses_data
     struct
     {
 	int x, y, tmarg, bmarg, valid;
-	chtype attr, bkgd;
+	chtype attr, bkgrnd;
     } save;
     char parm[8];
     char nparm;
@@ -129,10 +136,6 @@ static chtype ibm_map[128] =
 };
 
 
-#ifdef M_UNIX
-#define Getmaxy(w) getmaxy(w)
-#define Getmaxx(w) getmaxx(w)
-#else
 
 /*lint -save -e550 */
 static int
@@ -152,8 +155,7 @@ int y, x;
 	getmaxyx(w, y, x);
 	return x;
 }
-
-#endif
+/*lint -restore */
 
 
 
@@ -176,7 +178,6 @@ int i;
 static void
 key_add (const char *str, int val)
 {
-const char *old;
 struct keytrie *t;
 int c;
 
@@ -188,7 +189,6 @@ int c;
 	 */
 	if (!str || !*str)
 		return;			/* no key to define */
-	old = str;
 	t = keys;
 	while (str[1])    {
 		c = uchar(*str++);
@@ -301,25 +301,13 @@ curses_init(const struct sessmgr_sw *sm OPTIONAL)
 		key_add(key_f7, -9);
 		key_add(key_f8, -10);
 		key_add(key_f9, -11);
-		/* the next one is NOT portable, but since I can't find the PC's
-		   INSERT key in the table of ncurses strings, I will give it a
-		   fixed one.......... ko4ks  */
-#ifdef sun
-		key_add("\033[2z", -105);
-#else
-		key_add("\033[2~", -105);
-#endif
+		key_add(key_ic, -105);
 		key_add(key_dc, -106);
 		key_add(key_home, -107);
 		key_add(key_end, -108);
 		key_add(key_ppage, -109);
 		key_add(key_npage, -110);
-#ifdef M_UNIX
-		/* SCO botches it, as per usual */
-		key_add(key_f0, -2);
-#else
 		key_add(key_f10, -2);
-#endif
 		key_add(key_left, LTARROW);
 		key_add(key_right, RTARROW);
 		key_add(key_up, UPARROW);
@@ -558,7 +546,7 @@ chtype attrs;
 			    sp->save.attr = getattrs(sp->win);
 			    sp->save.tmarg = sp->win->_tmarg;
 			    sp->save.bmarg = sp->win->_bmarg;
-			    sp->save.bkgd = sp->win->_bkgd; /* should be a macro for this */
+			    sp->save.bkgrnd = sp->win->_bkgd; /* should be a macro for this */
 			    sp->save.valid = 1;
 			    sp->flags &= ~(C_ESC|C_CSI);
 			    break;
@@ -566,7 +554,7 @@ chtype attrs;
 			    if (sp->save.valid)	    {
 				(void) wmove(sp->win, sp->save.y, sp->save.x);
 				wattrset(sp->win, sp->save.attr);
-				wbkgdset(sp->win, sp->save.bkgd);
+				wbkgdset(sp->win, sp->save.bkgrnd);
 #ifdef linux
 				(void) werase(sp->win); /* wbkgdset() doesn't do anything */
 #endif
@@ -753,7 +741,7 @@ chtype attrs;
 		sp->save.attr = getattrs(sp->win);
 		sp->save.tmarg = sp->win->_tmarg;
 		sp->save.bmarg = sp->win->_bmarg;
-		sp->save.bkgd = sp->win->_bkgd; /* should be a macro for this */
+		sp->save.bkgrnd = sp->win->_bkgd; /* should be a macro for this */
 		sp->save.valid = 1;
 		sp->flags &= ~(C_ESC|C_CSI);
 		break;
@@ -761,7 +749,7 @@ chtype attrs;
 		if (sp->save.valid)	{
 		    (void) wmove(sp->win, sp->save.y, sp->save.x);
 		    wattrset(sp->win, sp->save.attr);
-		    wbkgdset(sp->win, sp->save.bkgd);
+		    wbkgdset(sp->win, sp->save.bkgrnd);
 		    (void) wsetscrreg(sp->win, sp->save.tmarg, sp->save.bmarg);
 		    sp->save.valid = 0;
 		}
