@@ -14,37 +14,41 @@
 #include "global.h"
 #include "ctype.h"
 #include "commands.h"
+#include "hardware.h"
 #include <sys/stat.h>
-#ifndef UNIX
-#ifndef S_ISDIR
-#define S_ISDIR(m)      (((m) & S_IFMT) == S_IFDIR)
-#endif
-#else
-/* There are at least FOUR variants of statfs()... we handle SCO and Linux */
-#ifdef M_UNIX
-#include <sys/statfs.h>
-#define f_bavail f_bfree
-#else
-#if ( defined(BSD) || defined(__FreeBSD__) ) && !defined(sun)
-#ifdef __FreeBSD__
-#include <sys/param.h>
-#endif
-#include <sys/mount.h>
-#else
-#include <sys/vfs.h>
-#endif /* BSD */
-#endif /* M_UNIX */
-#endif /* UNIX */
 
-#ifndef MSDOS
+#ifndef UNIX
+# ifndef S_ISDIR
+#  define S_ISDIR(m)      (((m) & S_IFMT) == S_IFDIR)
+# endif
+#else
+# ifdef HAVE_SYS_STATFS_H
+#  include <sys/statfs.h>
+#  define f_bavail f_bfree
+# else
+#  if defined(HAVE_SYS_PARAM_H) && !defined(_lint)
+#   include <sys/param.h>
+#  endif
+#  if defined(HAVE_SYS_MOUNT_H) && !defined(_lint)
+#   include <sys/mount.h>
+#  endif
+#  ifdef HAVE_SYS_VFS_H
+#   include <sys/vfs.h>
+#  endif
+# endif
+#endif	/* UNIX */
+
+#ifndef _lint
 #include <time.h>
 #endif
+#if !defined (_lint) || !defined(MSDOS)
 #include "proc.h"
 #include "session.h"
+#endif
 #include "smtp.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: dirutil.c,v 1.30 1997/05/24 22:13:02 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: dirutil.c,v 1.33 1997/09/14 14:37:46 root Exp root $";
 #endif
 
 
@@ -73,9 +77,6 @@ static void free_clist (struct dirsort * this);
 static void print_free_space (const char *path, FILE * file, int n);
 char *defpath (struct cur_dirs * curdirs, char *path);
 extern void crunch (char *buf, const char *path);
-#if defined(UNIX) && !defined(__FreeBSD__)
-extern int readlink (const char *path, char *buf, size_t bufsiz);
-#endif
 
 #ifdef TNOS_68K
 #define REGFILE	(FA_DIREC)
@@ -159,7 +160,7 @@ int rval;
 			rval = findnext (&sbuf);
 			break;
 	}
-	if (rval == -1)
+	if (rval != 0)
 		ret_str[0] = '\0';
 	else			/* Copy result to output */
 		strcpy (ret_str, sbuf.ff_name);
@@ -324,7 +325,7 @@ char **margv;
 	if (fp) {
 		(void) getdir (path, 1, fp);
 		free (path);
-		fclose (fp);
+		(void) fclose (fp);
 	}
 	margv[1] = strdup (tmpname);
 	(void) morecmd (2, margv, p);
@@ -368,10 +369,10 @@ domkd (int argc OPTIONAL, char *argv[], void *p OPTIONAL)
 {
 char path[128];
 
-	strcpy (path, make_fname (Command->curdirs->dir, argv[1]));
+	strncpy (path, make_fname (Command->curdirs->dir, argv[1]), 128);
 
 	if (mkdir (path, 0777) == -1)
-		tprintf ("Can't make %s: %s\n", path, sys_errlist[errno]);
+		tprintf ("Can't make %s: %s\n", path, SYS_ERRLIST(errno));
 	return 0;
 }
 
@@ -383,9 +384,9 @@ dormd (int argc OPTIONAL, char *argv[], void *p OPTIONAL)
 {
 char path[128];
 
-	strcpy (path, make_fname (Command->curdirs->dir, argv[1]));
+	strncpy (path, make_fname (Command->curdirs->dir, argv[1]), 128);
 	if (rmdir (path) == -1)
-		tprintf ("Can't remove %s: %s\n", path, sys_errlist[errno]);
+		tprintf ("Can't remove %s: %s\n", path, SYS_ERRLIST(errno));
 	return 0;
 }
 
@@ -485,7 +486,7 @@ static char ourpath[THISSIZE];
 		/* if there isn't enough room, give up -- it's invalid anyway */
 		if (strlen (path) + 4 > (THISSIZE - 1))
 			return path;
-		strcpy (ourpath, path);
+		strncpy (ourpath, path, THISSIZE - 4);
 #ifndef MSDOS
 		strcat (ourpath, "/*");
 #else
@@ -508,7 +509,7 @@ struct stat sb;
 char *symlinkpath = NULLCHAR;
 char *tcbuf;
 
-	strcpy (cbuf, sbuf->ff_name);
+	strncpy (cbuf, sbuf->ff_name, sizeof (sbuf->ff_name));
 	if (sbuf->ff_attrib & FA_DIREC)
 		strcat (cbuf, "/");
 	if (full == 1) {
@@ -588,7 +589,7 @@ char line_buf[80];	/* for long dirlist */
 char cbuf[20];		/* for making line_buf */
 char which;
 
-	strcpy (cbuf, sbuf->ff_name);
+	strncpy (cbuf, sbuf->ff_name, 20);
 	if (sbuf->ff_attrib & FA_DIREC)
 		strcat (cbuf, "/");
 	if (full == 1) {
@@ -649,14 +650,14 @@ struct statfs vfsb;
 char *cp;
 
 	pbuf = mallocw (1024);
-	strcpy (pbuf, path);
+	strncpy (pbuf, path, 1024 - 2);
 	if ((cp = strrchr (pbuf, '/')) == 0)
 		strcat (pbuf, "/.");
 	else {
 		*++cp = '.';
 		*++cp = '\0';
 	}
-#ifdef M_UNIX
+#if defined(HAVE_SYS_STATFS_H) && !defined(linux)
 	statfs (pbuf, &vfsb, sizeof vfsb, 0);
 #else
 	(void) statfs (pbuf, &vfsb);
@@ -753,12 +754,12 @@ char path[1024], *q;
 		if (*q == '/')
 #endif
 		{
-			strcpy (path, arg);
+			strncpy (path, arg, 1024 - strlen (ALLFILES));
 			strcat (path, ALLFILES);
 		} else
-			strcpy (path, arg);
+			strncpy (path, arg, 1024);
 	} else
-		strcpy (path, ALLFILES);
+		strncpy (path, ALLFILES, 1024);
 
 	retval = make_fname (tmpdir, path);
 	free (tmpdir);
@@ -773,7 +774,7 @@ make_fname (const char *curdir, const char *fname0)
 char *p, *fname;
 static char new_name[1024];
 
-	strcpy (new_name, curdir);
+	strncpy (new_name, curdir, 1024);
 	fname = strdup (fname0);
 	if (fname)
 		undosify (fname);
@@ -833,7 +834,7 @@ int new = 0;
 	}
 	return (result);
 #else	/* either MSDOS or UNIX */
-char *a, curpath[1024];
+char curpath[1024];
 char buf[128];
 
 #ifdef MSDOS
@@ -843,20 +844,27 @@ struct stat sbuf;
 
 
 	undosify (newpath);
-	strcpy (buf, newpath);
-	strcpy (curpath, dirs->dir);
+	strncpy (buf, newpath, 128);
+	strncpy (curpath, dirs->dir, 1024);
 
-	if (*buf != '/')
-		crunch (curpath, buf);
-	else
-		strcpy (curpath, buf + 1);
-
-	a = curpath;
-#ifdef UNIX
-	sprintf (fullpath, "%s%s", (*a != '/' ? "/" : ""), curpath);
-#else
-	sprintf (fullpath, "%c:%s%s", drive + '`', (*a != '/' ? "/" : ""), curpath);
+#ifdef MSDOS
+	if (buf[1] == ':')	{
+		strncpy (fullpath, buf, 1024);
+	} else
 #endif
+	{
+		if (*buf != '/')
+			crunch (curpath, buf);
+		else
+			strncpy (curpath, buf + 1, 1024);
+
+#ifdef UNIX
+		sprintf (fullpath, "%s%s", (*curpath != '/' ? "/" : ""), curpath);
+#else
+		strncpy (fullpath, curpath, 1024);
+#endif
+	}
+
 	if (stat (fullpath, &sbuf) == -1 || !S_ISDIR (sbuf.st_mode))
 		result = 0;
 	else if ((result = access (fullpath, 0) + 1) == 1) {
