@@ -34,7 +34,7 @@
 #include "session.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: domain.c,v 1.32 1997/06/28 16:46:13 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: domain.c,v 1.34 1997/08/19 01:19:22 root Exp root $";
 #endif
 
 #ifdef UNIX
@@ -411,7 +411,7 @@ int count = 0;
 			++count;
 			put_rr (fp, rrp);
 		}
-		fclose (fp);
+		(void) fclose (fp);
 	}
 	if (count) {
 		margv[1] = strdup (tmpname);
@@ -1390,7 +1390,7 @@ struct stat dstat;
 
 	if (fstat (fileno (dbase), &dstat) != 0) {
 		tputs ("Dfile_Search: can't get file status\n");
-		fclose (dbase);
+		(void) fclose (dbase);
 		return NULLRR;
 	}
 	if ((elapsed = (int32) (Dcache_time - (time_t) dstat.st_ctime)) < 0L)
@@ -1432,7 +1432,7 @@ struct stat dstat;
 	free_rr (oldrrp);
 	*rrpp = NULLRR;
 
-	fclose (dbase);
+	(void) fclose (dbase);
 	return result_rrlp;
 }
 
@@ -1510,7 +1510,7 @@ struct stat old_stat, new_stat;
 		}
 		if (fstat (fileno (new_fp), &new_stat) != 0) {
 			printf ("Dfile_Update: can't get new_file status!\n");
-			fclose (new_fp);
+			(void) fclose (new_fp);
 			break;
 		}
 		kwait (NULL);	/* file operations can be slow */
@@ -1543,15 +1543,15 @@ struct stat old_stat, new_stat;
 		/* open up the old file, concurrently with everyone else */
 		if ((old_fp = fopen (DomainFile, READ_TEXT)) == NULLFILE) {
 			/* great! no old file, so we're ready to go. */
-			fclose (new_fp);
+			(void) fclose (new_fp);
 			(void) rename (newname, DomainFile);
 			free_rr (rrlp);
 			break;
 		}
 		if (fstat (fileno (old_fp), &old_stat) != 0) {
 			printf ("Dfile_Update: can't get old_file status!\n");
-			fclose (new_fp);
-			fclose (old_fp);
+			(void) fclose (new_fp);
+			(void) fclose (old_fp);
 			free_rr (rrlp);
 			break;
 		}
@@ -1578,8 +1578,8 @@ struct stat old_stat, new_stat;
 				kwait (NULL);	/* run in background */
 		}
 		free_rr (oldrrp);
-		fclose (new_fp);
-		fclose (old_fp);
+		(void) fclose (new_fp);
+		(void) fclose (old_fp);
 		free_rr (rrlp);
 
 		/* wait for everyone else to finish reading */
@@ -2492,6 +2492,7 @@ struct dhdr *dhdr;
 struct dserver *dp;
 int foo;
 
+	server_disconnect_io ();
 	Dsocket = socket (AF_INET, SOCK_DGRAM, 0);
 	sock.sin_family = AF_INET;
 	sock.sin_addr.s_addr = INADDR_ANY;
@@ -2686,7 +2687,7 @@ FILE *fp;
 		goto exit;
 	}
 	if (argc > 3) {
-		strcpy (rec_type, argv[3]);
+		strncpy (rec_type, argv[3], 20);
 		(void) strupr (rec_type);
 
 		for (i = 1; i < NDTYPES1; i += 2) {
@@ -2780,7 +2781,7 @@ FILE *fp;
 				count++;
 				put_rr (fp, rrp);
 			}
-			fclose (fp);
+			(void) fclose (fp);
 		}
 		free_dhdr (dhp);
 		if (count) {
