@@ -12,7 +12,7 @@ extern int FWDpersonal, FWDbulletins;
 
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: fbbfwd.c,v 1.24 1997/06/13 13:21:37 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: fbbfwd.c,v 1.27 1997/09/07 00:31:16 root Exp root $";
 #endif
 
 extern int indexFwdBbs (char *name);
@@ -204,7 +204,7 @@ unsigned int len;
 
 #ifdef REJECT
 	/* first, before we strip off the 'FA/FB', lets pass the string to reject() */
-	msglst->reject = reject (fbline, 1);
+	msglst->reject = reject (fbline, 1, 0);
 #endif
 	/* FB/FA */
 	/* At present... FA and FB are treated equally. The FBB Specs say that 'FA' is an */
@@ -213,7 +213,7 @@ unsigned int len;
 	if ((cp = strtok (fbline, " ")) == NULLCHAR)
 		return 0;
 	if (strlen (cp) < sizeof (msglst->fbbcmd))
-		strcpy (msglst->fbbcmd, cp);
+		strncpy (msglst->fbbcmd, cp, 3);
 
 	/* Message type ( P, B, or T ) */
 	if ((cp = strtok (NULLCHAR, " ")) == NULLCHAR)
@@ -387,6 +387,7 @@ eatprompt:
 			 */
 
 			anysent++;
+			f->sentThisArea++;
 			if (FBBtrace)
 				tcmdprintf (sendingmessageto, fwd_bbsname (m));
 			msglst[i].accept = fbbYES;
@@ -407,7 +408,7 @@ eatprompt:
 				sendmsg (f->m, msglst[i].number, msglst[i].bid);
 
 				/* close the file. send_yapp() will re-open it. */
-				fclose (m->quickfile);
+				(void) fclose (m->quickfile);
 				m->quickfile = NULLFILE;
 
 				/* And Send it. */
@@ -453,6 +454,9 @@ eatprompt:
 		}
 	}
 
+	if (!anysent)
+		goto goback;
+		
 	/* Now we will get a line from the remote system to make sure that the
 	   messages were received ok.
 	 */
@@ -510,6 +514,7 @@ eatprompt:
 		}
 	}
 	/* Change forward direction */
+goback:
 	m->state = MBX_REVFWD;
 	if (strnicmp (m->line, "FF", 2) == 0)
 		return 1;
@@ -598,12 +603,7 @@ char tempname[80];
 			thechecksum += 13;
 		}
 		/* lowercase command. */
-		cp = m->line;
-		while (*cp) {
-			if (isupper (*cp))
-				*cp = (char) tolower (*cp);
-			++cp;
-		}
+		(void) strlwr (m->line);
 
 		if (strncmp (m->line, "ff", 2) == 0)
 			/* End of FBB Packet */
@@ -788,8 +788,8 @@ char tempname[80];
 				(void) tmpnam (tempname);
 
 				/* Receive the message and uncompress it. */
-				strcpy (f->iFile, tmpnam (NULL));
-				strcpy (f->oFile, tempname);
+				strncpy (f->iFile, tmpnam (NULL), FWDFILELEN);
+				strncpy (f->oFile, tempname, FWDFILELEN);
 				if (recv_yapp (f) == 0)
 					return 0;
 
@@ -811,7 +811,7 @@ char tempname[80];
 			retval = mbx_parse (m);
 #ifdef FBBCMP
 			if (m->quickfile) {
-				fclose (m->quickfile);
+				(void) fclose (m->quickfile);
 				m->quickfile = NULLFILE;
 			}
 #endif
@@ -830,7 +830,7 @@ char tempname[80];
 		tcmdprintf (messagesreceived, fwd_bbsname (m));
 #ifdef FBBCMP
 	if (m->quickfile) {
-		fclose (m->quickfile);
+		(void) fclose (m->quickfile);
 		m->quickfile = NULLFILE;
 	}
 #endif
@@ -1009,7 +1009,7 @@ struct arealist *a;
 							msglst[msgcnt].number = i;
 
 							/* Keep track of makecl() modified bid. */
-							strcpy (msglst[msgcnt].bid, &thebid[1]);
+							strncpy (msglst[msgcnt].bid, &thebid[1], 15);
 
 							/* uppercase the FB line and send it. */
 							(void) strupr (line2);
@@ -1076,7 +1076,6 @@ int FBBdone = 0;
 int FBBok = 0;
 long pos;
 int areasused;
-int lastcount;
 int equals, bang;
 char name[256];
 struct arealist *fwdarea;
@@ -1092,7 +1091,7 @@ struct mbx *m;
 	(void) nntp_name_expansion (name);
 
 	/* Save data */
-	strcpy (f->savefsline, m->line);
+	strncpy (f->savefsline, m->line, MBXLINE);
 
 	/* figure out .fwd file name and see if it exist. It it doesn't exist...   */
 	/* .... we have nothing for this user.                                     */
@@ -1124,7 +1123,7 @@ struct mbx *m;
 	 */
 
 	/* restore data. */
-	strcpy (m->line, f->savefsline);
+	strncpy (m->line, f->savefsline, MBXLINE);
 
 checkloop:
 	kwait (NULL);
@@ -1142,20 +1141,20 @@ checkloop:
 			FBBok = 0;
 			break;
 		}
-		lastcount = Tmsgcnt;
+		f->sentThisArea = 0;
 		Tmsgcnt += fbbfwdthisarea (f, fwdarea->name, &FBBok, &FBBdone);
 		if (FBBdone || Tmsgcnt)
 			break;
 		if (isnewprivmail (m, "fwd") > 0)
 			goto checkloop;
-		if (lastcount != Tmsgcnt)	/* if we got something in this area */
+		if (f->sentThisArea)	/* if we got something in this area */
 			areasused += 1;
 		fwdarea = fwdarea->next;	/* Do next area. */
 	}			/* end while() */
 
 	if (!FBBok) {
 		/* We had an error. */
-		fclose (f->fwdfile);
+		(void) fclose (f->fwdfile);
 		return 3;
 	}
 	/* this next piece (and the sections above that affect areasused), make
@@ -1204,7 +1203,7 @@ checkloop:
 		fwdunlockit (m->name);
 		goto checkloop;
 	}
-	fclose (f->fwdfile);
+	(void) fclose (f->fwdfile);
 	if (!err)	{
 		if (FBBtrace)
 			tcmdprintf ("FBBFWD: Removing completed .fwd file for %s\n", fwd_bbsname (m));
@@ -1264,7 +1263,7 @@ int sendFQ = 0;
 		NeedData = TRUE;
 		FBBfwd = TRUE;
 	} else if (strnicmp (argv[0], "FA", 2) == 0 || strnicmp (argv[0], "FB", 2) == 0) {
-		if (argc > 7) {
+		if (argc != 7) {
 			tprintf ("Huh?\n");
 			usflush (m->user);
 			Done = TRUE;
@@ -1305,6 +1304,7 @@ int sendFQ = 0;
 			}
 			/* Change status. */
 			f.m->state = MBX_FORWARD;
+			smtptick (NULL);	/* wake SMTP to send that mail */
 		} else {
 			if (FBBtrace)
 				tcmdprintf ("FBBFWD: Sending data to %s.\n", fwd_bbsname (m));
