@@ -26,10 +26,10 @@
 #endif
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: ftpcli.c,v 1.26 1997/06/28 16:46:13 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: ftpcli.c,v 1.28 1997/08/19 01:19:22 root Exp root $";
 #endif
 
-#define	DIRBUF	256
+#define	FTPDIRBUF	256
 
 #ifdef ALLSESSIONS
 static int compsub (struct ftpcli * ftp, char *localname, char *remotename);
@@ -222,10 +222,8 @@ int pauseonexit = 0;
 
 	if (resp >= 400)
 		goto quit;
-#ifdef UNIX
 	if ((un = getenv ("LOGNAME")) == NULLCHAR)
-#endif
-		un = getenv ("USER");		/*lint !e539 */
+		un = getenv ("USER");
 	if (un != NULLCHAR)
 		sprintf (prmt, "Enter user name (%s): ", un);
 	else
@@ -398,7 +396,7 @@ quit:	cp = sockerr (control);
 		 cp != NULLCHAR ? cp : "EOF");
 
 	if (ftp.fp != NULLFILE && ftp.fp != stdout)
-		fclose (ftp.fp);
+		(void) fclose (ftp.fp);
 	if (ftp.data != -1)
 		close_s (ftp.data);
 	if (ftp.control != -1)
@@ -406,7 +404,7 @@ quit:	cp = sockerr (control);
 	if (pauseonexit)
 		(void) keywait (NULLCHAR, 1);
 	if (fp1 != NULLFILE)
-		fclose (fp1);
+		(void) fclose (fp1);
 	if (ftp.session != NULLSESSION)
 		freesession (ftp.session);
 	free_dirs (&dirs);
@@ -552,10 +550,10 @@ char fname2[128];
 	ftp = (struct ftpcli *) p;
 	if (ftp == NULLFTP)
 		return -1;
-	strcpy (fname1, make_fname (ftp->curdirs->dir, argv[1]));
-	strcpy (fname2, make_fname (ftp->curdirs->dir, argv[2]));
+	strncpy (fname1, make_fname (ftp->curdirs->dir, argv[1]), 128);
+	strncpy (fname2, make_fname (ftp->curdirs->dir, argv[2]), 128);
 	if (rename (fname1, fname2) == -1)
-		tprintf ("Can't rename: %s\n", sys_errlist[errno]);
+		tprintf ("Can't rename: %s\n", SYS_ERRLIST(errno));
 	else
 		tprintf ("Local file renamed to '%s'\n", fname2);
 	return 0;
@@ -814,7 +812,7 @@ long r;
 		return 1;
 	}
 	(void) tmpnam (tmpname);
-	buf = mallocw (DIRBUF);
+	buf = mallocw (FTPDIRBUF);
 	ftp->state = RECEIVING_STATE;
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '@') {
@@ -829,11 +827,11 @@ long r;
 				tprintf ("Can't open tempfile: %s\n", tmpname);
 				continue;
 			}
-			while (fgets (buf, DIRBUF, filel) != NULLCHAR)
+			while (fgets (buf, FTPDIRBUF, filel) != NULLCHAR)
 				fputs (buf, files);
 
-			fclose (files);
-			fclose (filel);
+			(void) fclose (files);
+			(void) fclose (filel);
 			if ((files = fopen (tmpname, "r")) == NULLFILE) {
 				tprintf ("Can't open tempfile: %s\n", tmpname);
 				continue;
@@ -854,7 +852,7 @@ long r;
 		/* The tmp file now contains a list of the remote files, so
 		 * go get 'em. Break out if the user signals an abort.
 		 */
-		while (fgets (buf, DIRBUF, files) != NULLCHAR) {
+		while (fgets (buf, FTPDIRBUF, files) != NULLCHAR) {
 			rip (buf);
 			local = strdup (buf);
 #ifdef	MSDOS
@@ -875,14 +873,14 @@ long r;
 			if (ftp->abort) {
 				/* User abort */
 				ftp->abort = 0;
-				fclose (files);
+				(void) fclose (files);
 				unlink (tmpname);
 				free (buf);
 				ftp->state = COMMAND_STATE;
 				return 1;
 			}
 		}
-		fclose (files);
+		(void) fclose (files);
 		unlink (tmpname);
 	}
 	free (buf);
@@ -1049,7 +1047,7 @@ char tmpname[80];
 		return 1;
 	}
 	(void) tmpnam (tmpname);
-	buf = mallocw (DIRBUF);
+	buf = mallocw (FTPDIRBUF);
 	ftp->state = RECEIVING_STATE;
 	for (i = 1; i < argc; i++) {
 		r = getsub (ftp, "NLST", argv[i], tmpname);
@@ -1063,7 +1061,7 @@ char tmpname[80];
 		/* The tmp file now contains a list of the remote files, so
 		 * go get 'em. Break out if the user signals an abort.
 		 */
-		while (fgets (buf, DIRBUF, files) != NULLCHAR) {
+		while (fgets (buf, FTPDIRBUF, files) != NULLCHAR) {
 			rip (buf);
 			if (compsub (ftp, buf, buf) == 0)
 				tprintf ("%s - Same\n", buf);
@@ -1073,14 +1071,14 @@ char tmpname[80];
 			if (ftp->abort) {
 				/* User abort */
 				ftp->abort = 0;
-				fclose (files);
+				(void) fclose (files);
 				unlink (tmpname);
 				free (buf);
 				ftp->state = COMMAND_STATE;
 				return 1;
 			}
 		}
-		fclose (files);
+		(void) fclose (files);
 		unlink (tmpname);
 	}
 	free (buf);
@@ -1145,8 +1143,8 @@ char lochash[16];
 	}
 	usprintf (control, "XMD5 %s\n", remotename);
 	/* Try to overlap the two MD5 operations */
-	(void) md5hash (fp, lochash, ftp->type == ASCII_TYPE);	/*lint !e730 */
-	fclose (fp);
+	(void) md5hash (fp, lochash, ftp->type == ASCII_TYPE);
+	(void) fclose (fp);
 	if (typewait && (resp = getresp (ftp, 200)) > 299)
 		goto failure;
 	if ((resp = getresp (ftp, 200)) > 299) {
@@ -1222,7 +1220,7 @@ unsigned long starting;
 	if (localname == NULLCHAR) {
 		fp = NULLFILE;
 	} else if ((fp = fopen (make_fname (ftp->curdirs->dir, localname), mode)) == NULLFILE) {
-		tprintf ("Can't write %s: %s\n", localname, sys_errlist[errno]);
+		tprintf ("Can't write %s: %s\n", localname, SYS_ERRLIST(errno));
 		return -1;
 	}
 	/* Open the data connection */
@@ -1263,8 +1261,12 @@ unsigned long starting;
 	 */
 	i = SOCKSIZE;
 	(void) getsockname (ftp->data, (char *) &lsocket, &i);	/* Get port number */
+	if (!i)
+		goto failure;
 	i = SOCKSIZE;
 	(void) getsockname (ftp->control, (char *) &lcsocket, &i);
+	if (!i)
+		goto failure;
 	lsocket.sin_addr.s_addr = lcsocket.sin_addr.s_addr;
 	sendport (control, &lsocket);
 	if (!ftp->batch) {
@@ -1335,7 +1337,7 @@ unsigned long starting;
 		putc (CTLZ, fp);
 #endif
 	if (fp != NULLFILE && fp != stdout)
-		fclose (fp);
+		(void) fclose (fp);
 	if (remotename == NULLCHAR)
 		remotename = "";
 	if (total == (unsigned long) -1) {
@@ -1363,7 +1365,7 @@ unsigned long starting;
 failure:
 	/* Error, quit */
 	if (fp != NULLFILE && fp != stdout)
-		fclose (fp);
+		(void) fclose (fp);
 	close_s (ftp->data);
 	ftp->data = -1;
 	ftp->state = (char) prevstate;
@@ -1433,9 +1435,9 @@ char *buf, *file;
 	}
 
 	rewind (files);
-	buf = mallocw (DIRBUF);
+	buf = mallocw (FTPDIRBUF);
 	ftp->state = SENDING_STATE;
-	while (fgets (buf, DIRBUF, files) != NULLCHAR) {
+	while (fgets (buf, FTPDIRBUF, files) != NULLCHAR) {
 		rip (buf);
 		if (!ftp->update || compsub (ftp, buf, buf) != 0)
 			(void) putsub (ftp, buf, buf, 0);
@@ -1443,7 +1445,7 @@ char *buf, *file;
 			break;	/* User abort */
 		usflush (ftp->control);
 	}
-	fclose (files);
+	(void) fclose (files);
 	unlink (tmpname);
 	free (buf);
 	ftp->state = COMMAND_STATE;
@@ -1504,7 +1506,7 @@ unsigned long check, local_check;
 
 	/* Open the file */
 	if ((fp = fopen (make_fname (ftp->curdirs->dir, localname), mode)) == NULLFILE) {
-		tprintf ("Can't read %s: %s\n", localname, sys_errlist[errno]);
+		tprintf ("Can't read %s: %s\n", localname, SYS_ERRLIST(errno));
 		return -1;
 	}
 	if (ftp->type == ASCII_TYPE && isbinary (fp)) {
@@ -1547,8 +1549,12 @@ unsigned long check, local_check;
 	 */
 	i = SOCKSIZE;
 	(void) getsockname (ftp->data, (char *) &lsocket, &i);
+	if (!i)
+		goto failure;
 	i = SOCKSIZE;
 	(void) getsockname (ftp->control, (char *) &lcsocket, &i);
+	if (!i)
+		goto failure;
 	lsocket.sin_addr.s_addr = lcsocket.sin_addr.s_addr;
 	sendport (control, &lsocket);
 	if (!ftp->batch) {
@@ -1631,7 +1637,7 @@ unsigned long check, local_check;
 	total = (unsigned long) sendfile (fp, ftp->data, ftp->type, (ftp->verbose >= V_HASH) ? ftp->verbose : 0);
 	close_s (ftp->data);
 	ftp->data = -1;
-	fclose (fp);
+	(void) fclose (fp);
 
 	/* Wait for control channel ack before calculating transfer time;
 	 * this accounts for transmitted data in the pipe.
@@ -1657,7 +1663,7 @@ unsigned long check, local_check;
 
 failure:
 	/* Error, quit */
-	fclose (fp);
+	(void) fclose (fp);
 	close_s (ftp->data);
 	ftp->data = -1;
 	ftp->state = (char) prevstate;
@@ -1797,10 +1803,10 @@ FILE *fp;
 	}
 	if (feof (fp)) {
 		/* User name not found in file */
-		fclose (fp);
+		(void) fclose (fp);
 		return NULLCHAR;
 	}
-	fclose (fp);
+	(void) fclose (fp);
 	/* Look for space after user field in file */
 	cp = skipwhite (cp);
 	if ((cp1 = strpbrk (cp, " \t")) == NULLCHAR)
@@ -1878,7 +1884,7 @@ char *buf;
 	/*	undosify(argv[1]);	done in make_fname	*/
 	buf = strdup (make_fname (ftp->curdirs->dir, argv[1]));
 	if (mkdir (buf, 0777) == -1)
-		tprintf ("Can't make %s: %s\n", buf, sys_errlist[errno]);
+		tprintf ("Can't make %s: %s\n", buf, SYS_ERRLIST(errno));
 	else
 		tprintf ("Directory %s Created\n", buf);
 	free (buf);
@@ -1899,7 +1905,7 @@ char *buf;
 	}
 	buf = strdup (make_fname (ftp->curdirs->dir, argv[1]));
 	if (rmdir (buf) == -1)
-		tprintf ("Can't remove %s: %s\n", buf, sys_errlist[errno]);
+		tprintf ("Can't remove %s: %s\n", buf, SYS_ERRLIST(errno));
 	else
 		tprintf ("Directory %s Removed\n", buf);
 	free (buf);
