@@ -31,24 +31,23 @@ extern char *CDROM;		/* buckbook.c: defines CDROM drive letter e.g. "s:"  */
 
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: ftpserv.c,v 1.28 1997/05/24 12:53:20 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: ftpserv.c,v 1.31 1997/09/14 14:37:46 root Exp root $";
 #endif
 
 #ifdef UNIX
-extern int chown (const char *path, uid_t owner, gid_t group);
-extern int setreuid (uid_t ruid, uid_t euid);
-extern int setregid (gid_t rgid, gid_t egid);
-extern uid_t getuid (void);
-extern gid_t getgid (void);
 int ftpsecuritycheck (char *filename, int perms, int mode);
 extern long ACCESSgid, ACCESSuid;
 extern long CREATEgid, CREATEuid;
 extern short CREATEmask;
 extern int CREATEsecure;
 
+#ifndef R_OK
 #define R_OK 4
+#endif
+#ifndef W_OK
 #define W_OK 2
 #endif
+#endif
 
 static int isanonymous (char *name);
 static void ftpserv (int s, void *unused, void *p);
@@ -283,7 +282,7 @@ char buf[128], *cp;
 		for ( ; ; ) {
 			if (fgets (buf, 128, fp) == NULLCHAR) {
 				if (fpsave) {
-					fclose (fp);
+					(void) fclose (fp);
 					fp = fpsave;
 					fpsave = NULLFILE;
 					continue;
@@ -311,8 +310,8 @@ char buf[128], *cp;
 			}
 		}
 		if (fpsave)
-			fclose (fpsave);
-		fclose (fp);
+			(void) fclose (fpsave);
+		(void) fclose (fp);
 	}
 	return (retval);
 }
@@ -349,9 +348,16 @@ char *cp1;
 	ftp.control = s;
 	/* Set default data port */
 	i = SOCKSIZE;
-	(void) getpeername (s, (char *) &thesocket, &i);
-	thesocket.sin_port = IPPORT_FTPD;
-	ASSIGN (ftp.port, thesocket);
+	if (getpeername (s, (char *) &thesocket, &i) != -1)	{
+		thesocket.sin_port = IPPORT_FTPD;
+		ASSIGN (ftp.port, thesocket);
+	}
+
+	if (++FtpUsers > FtpMaxUsers) {
+		usprintf (s, "\n200- Sorry, too many FTP users at %s this time. Try again later!\n", Hostname);
+		FtpUsers--;
+		return;
+	}
 
 #ifdef FTPTDISC
 	/* Set the timeout timer - WG7J */
@@ -361,12 +367,6 @@ char *cp1;
 	start_timer (&ftp.tdisc);
 #endif
 
-	if (++FtpUsers > FtpMaxUsers) {
-		usprintf (s, "\n200- Sorry, too many FTP users at %s this time. Try again later!\n", Hostname);
-		FtpUsers--;
-		return;
-	}
-
 #ifdef STATS_USE
 	STATS_adduse (1);
 #endif
@@ -377,7 +377,7 @@ char *cp1;
 	strcpy (buf, ETCdir);
 	if ((fpm = fopen (strcat (buf, "/banner.ftp"), "r")) != NULL) {
 		sendmsgfile (s, 220, buf, sizeof (buf), fpm);
-		fclose (fpm);
+		(void) fclose (fpm);
 	}
 	usprintf (s, banner, Hostname, Version);
 	(void) init_dirs (&dirs);
@@ -490,7 +490,7 @@ loop:
 			if (!permcheck (ftp.path, ftp.perms, RNFR_CMD, file))
 				usprintf (ftp.control, noperm);
 			else if (access (file, 6))
-				usprintf (ftp.control, cantopen, file, sys_errlist[errno]);
+				usprintf (ftp.control, cantopen, file, SYS_ERRLIST(errno));
 			else {
 				usprintf (ftp.control, pendingto);
 				rnfrom = strdup (file);
@@ -506,7 +506,7 @@ loop:
 					usprintf (ftp.control, noperm);
 				else {
 					if (rename (rnfrom, file) == -1)
-						usprintf (ftp.control, norename, sys_errlist[errno]);
+						usprintf (ftp.control, norename, SYS_ERRLIST(errno));
 					else
 						usprintf (ftp.control, okay);
 				}
@@ -535,7 +535,7 @@ loop:
 #else
 			else if ((ftp.fp = fopen (file, mode)) == NULLFILE)
 #endif
-				usprintf (ftp.control, cantopen, file, sys_errlist[errno]);
+				usprintf (ftp.control, cantopen, file, SYS_ERRLIST(errno));
 			else {
 				if ((cmdp - commands) == RSME_CMD) {
 					log (ftp.control, "RSME %s", file);
@@ -559,7 +559,7 @@ loop:
 				usputs (ftp.control, noperm);
 			else if ((ftp.fp = fopen (file, READ_BINARY)) != NULLFILE) {
 				usprintf (ftp.control, filesize, filelength (fileno (ftp.fp)));
-				fclose (ftp.fp);
+				(void) fclose (ftp.fp);
 			} else if (!access (file, 0))
 				usprintf (ftp.control, notaplain, file);
 			else
@@ -600,7 +600,7 @@ store2:
 #else
 			else if ((ftp.fp = fopen (file, mode)) == NULLFILE)
 #endif
-				usprintf (ftp.control, cantmake, file, sys_errlist[errno]);
+				usprintf (ftp.control, cantmake, file, SYS_ERRLIST(errno));
 			else {
 				log (ftp.control, "%s %s", cp, file);
 				(void) recvit (&ftp, cp, file);
@@ -634,7 +634,7 @@ store2:
 					cp = Command->curdirs->dir;
 					Command->curdirs->dir = ftp.curdirs->dir;
 					if ((ftp.fp = dir (file, ((cmdp - commands) == LIST_CMD) ? 2 : 0)) == NULLFILE)
-						usprintf (ftp.control, nodir, file, sys_errlist[errno]);
+						usprintf (ftp.control, nodir, file, SYS_ERRLIST(errno));
 					else
 						(void) sendit (&ftp, ((cmdp - commands) == LIST_CMD) ? "LIST" : "NLST", file);
 					Command->curdirs->dir = cp;
@@ -672,7 +672,7 @@ store2:
 					usprintf (ftp.control, pwdmsg, file);
 				} else {
 					/* Failed, don't change anything */
-					usprintf (ftp.control, nodir, file, sys_errlist[errno]);
+					usprintf (ftp.control, nodir, file, SYS_ERRLIST(errno));
 					free (file);
 				}
 				break;
@@ -696,30 +696,39 @@ store2:
 				/* If exists, send the contents of 'desc.ftp' in the new
 				 * directory...
 				 */
-				strcpy (buf, file);
+				strncpy (buf, file, 512);
 				if ((fpm = fopen (strcat (buf, "/desc.ftp"), "r")) != NULL) {
 					sendmsgfile (ftp.control, 257, buf, sizeof (buf), fpm);
-					fclose (fpm);
+					(void) fclose (fpm);
 				}
 #ifndef MSDOS
 				/* If exists, send the contents of '.message' in the new
 				 * directory...
 				 */
-				strcpy (buf, file);
+				strncpy (buf, file, 512);
 				if ((fpm = fopen (strcat (buf, "/.message"), "r")) != NULL) {
 					sendmsgfile (ftp.control, 257, buf, sizeof (buf), fpm);
-					fclose (fpm);
+					(void) fclose (fpm);
 				}
 #endif
-				usprintf (ftp.control, pwdmsg, (strlen (ftp.root) == strlen (ftp.curdirs->dir)) ? "/" : &ftp.curdirs->dir[strlen (ftp.root)]);
+
+				usprintf (ftp.control, pwdmsg,
+					(!strcmp (ftp.root, ftp.curdirs->dir)) ? "/"
+					: (!strncmp (ftp.root, ftp.curdirs->dir, strlen (ftp.root)))
+					? &ftp.curdirs->dir[strlen (ftp.root)]
+					: ftp.curdirs->dir);
 			} else
 				/* Failed, nothing changed */
-				usprintf (ftp.control, nodir, arg, sys_errlist[errno]);
+				usprintf (ftp.control, nodir, arg, SYS_ERRLIST(errno));
 			free (file);
 			break;
 		case XPWD_CMD:
 		case PWD_CMD:
-			usprintf (ftp.control, pwdmsg, (strlen (ftp.root) == strlen (ftp.curdirs->dir)) ? "/" : &ftp.curdirs->dir[strlen (ftp.root)]);
+			usprintf (ftp.control, pwdmsg,
+				(!strcmp (ftp.root, ftp.curdirs->dir)) ? "/"
+				: (!strncmp (ftp.root, ftp.curdirs->dir, strlen (ftp.root)))
+				? &ftp.curdirs->dir[strlen (ftp.root)]
+				: ftp.curdirs->dir);
 			break;
 #else
 		case LIST_CMD:
@@ -761,7 +770,7 @@ store2:
 				log (ftp.control, "DELE %s", file);
 				usprintf (ftp.control, deleok);
 			} else
-				usprintf (ftp.control, delefail, sys_errlist[errno]);
+				usprintf (ftp.control, delefail, SYS_ERRLIST(errno));
 
 			free (file);
 			break;
@@ -781,7 +790,7 @@ store2:
 				log (ftp.control, "MKD %s", file);
 				usprintf (ftp.control, mkdok);
 			} else
-				usprintf (ftp.control, cantmake, file, sys_errlist[errno]);
+				usprintf (ftp.control, cantmake, file, SYS_ERRLIST(errno));
 
 			free (file);
 			break;
@@ -794,7 +803,7 @@ store2:
 				log (ftp.control, "RMD %s", file);
 				usprintf (ftp.control, deleok);
 			} else
-				usprintf (ftp.control, delefail, sys_errlist[errno]);
+				usprintf (ftp.control, delefail, SYS_ERRLIST(errno));
 
 			free (file);
 			break;
@@ -823,8 +832,12 @@ store2:
 			(void) listen (ftp.data, 0);
 			i = SOCKSIZE;
 			(void) getsockname (ftp.data, (char *) &lsocket, &i);
+			if (!i)
+				break;
 			i = SOCKSIZE;
 			(void) getsockname (ftp.control, (char *) &lcsocket, &i);
+			if (!i)
+				break;
 			lsocket.sin_addr.s_addr = lcsocket.sin_addr.s_addr;
 			/* send the address to the client.  */
 			SendPasv (ftp.control, &lsocket);
@@ -848,7 +861,7 @@ store2:
 #else
 			else if ((ftp.fp = fopen (file, mode)) == NULLFILE)
 #endif
-				usprintf (ftp.control, cantopen, file, sys_errlist[errno]);
+				usprintf (ftp.control, cantopen, file, SYS_ERRLIST(errno));
 			else {
 				char hash[16];
 
@@ -856,8 +869,8 @@ store2:
 				if (ftp.type == ASCII_TYPE && isbinary (ftp.fp))
 					usprintf (ftp.control, binwarn, file);
 
-				(void) md5hash (ftp.fp, hash, ftp.type == ASCII_TYPE);	/*lint !e730 */
-				fclose (ftp.fp);
+				(void) md5hash (ftp.fp, hash, ftp.type == ASCII_TYPE);
+				(void) fclose (ftp.fp);
 				ftp.fp = NULLFILE;
 				usprintf (ftp.control, "200 ");
 				for (i = 0; i < 16; i++)
@@ -899,7 +912,7 @@ finish:
 	if (ftp.data != -1)
 		close_s (ftp.data);
 	if (ftp.fp != NULLFILE)
-		fclose (ftp.fp);
+		(void) fclose (ftp.fp);
 	free (ftp.username);
 	free (ftp.path);
 	free_dirs (&dirs);
@@ -969,7 +982,14 @@ int anony = 0;
 		*cp1 = '\0';
 	if ((cp1 = strchr (cp, '=')) != NULLCHAR)
 		*cp1 = '\0';
-	ftp->root = strdup (cp);
+#ifdef MSDOS
+	if (*cp == '/' || *cp == '\\')	{
+		sprintf (buf, "%c:%s", ftp->curdirs->drv + '`', cp);
+		ftp->root = strdup (buf);
+	} else
+#endif
+		ftp->root = strdup (cp);	/*lint !e539 */
+	free (cp);
 
 	{
 	FILE *out;
@@ -980,7 +1000,7 @@ int anony = 0;
 
 			(void) time (&tt);
 			fprintf (out, "FTP from %s (%s) on %s", ftp->username, pass, ptime (&tt));
-			fclose (out);
+			(void) fclose (out);
 		}
 	}
 
@@ -992,13 +1012,13 @@ int anony = 0;
 	/* everyone gets the ftpmotd file, if it exists */
 	if ((fp = fopen (Ftpmotd, "r")) != NULL) {
 		sendmsgfile (ftp->control, 230, buf, sizeof (buf), fp);
-		fclose (fp);
+		(void) fclose (fp);
 	}
 	/* if there is a message.ftp file in the login dir, send it too */
 	sprintf (buf, "%s%s%s", path, (path[strlen (path) - 1] == '/') ? "" : "/", "message.ftp");
 	if ((fp = fopen (buf, "r")) != NULL) {
 		sendmsgfile (ftp->control, 230, buf, sizeof (buf), fp);
-		fclose (fp);
+		(void) fclose (fp);
 	}
 	path = strdup (ftp->path);
 	if ((p = strpbrk (path, "=;")) != NULLCHAR)
@@ -1009,7 +1029,7 @@ int anony = 0;
 		sprintf (buf, "%s/desc.ftp", path);
 		if ((fp = fopen (buf, "r")) != NULL) {
 			sendmsgfile (ftp->control, 230, buf, sizeof (buf), fp);
-			fclose (fp);
+			(void) fclose (fp);
 		}
 	}
 	free (path);
@@ -1174,7 +1194,7 @@ int pasv = 0;
 	usprintf (ftp->control, sending, command, (strlen (file) == strlen (ftp->path)) ? "/" : &file[strlen (ftp->path)], fsizetext);	/* N1BEE */
 	if (!pasv) {
 		if (connect (ftp->data, (char *) &ftp->port, SOCKSIZE) == -1) {
-			fclose (ftp->fp);
+			(void) fclose (ftp->fp);
 			ftp->fp = NULLFILE;
 			close_s (ftp->data);
 			ftp->data = -1;
@@ -1242,7 +1262,7 @@ int pasv = 0;
 		usprintf (ftp->control, txok);
 
 send_err:
-	fclose (ftp->fp);
+	(void) fclose (ftp->fp);
 	ftp->fp = NULLFILE;
 	close_s (ftp->data);
 	ftp->data = -1;
@@ -1278,7 +1298,7 @@ int pasv = 0;
 
 	if (!pasv) {
 		if (connect (ftp->data, (char *) &ftp->port, SOCKSIZE) == -1) {
-			fclose (ftp->fp);
+			(void) fclose (ftp->fp);
 			ftp->fp = NULLFILE;
 			close_s (ftp->data);
 			ftp->data = -1;
@@ -1324,13 +1344,13 @@ int pasv = 0;
 #endif
 	if (total == -1) {
 		/* An error occurred while writing the file */
-		usprintf (ftp->control, writerr, sys_errlist[errno]);
+		usprintf (ftp->control, writerr, SYS_ERRLIST(errno));
 		(void) shutdown (ftp->data, 2);	/* Blow it away */
 	} else
 		usprintf (ftp->control, rxok);
 	close_s (ftp->data);
 	ftp->data = -1;
-	fclose (ftp->fp);
+	(void) fclose (ftp->fp);
 	ftp->fp = NULLFILE;
 #ifdef LZW
 	ftp->lzw = 0;
