@@ -30,12 +30,19 @@
 #include "bm.h"
 #endif
 #endif
+#ifdef STAT_HTTP
+#include "stats.h"
+#endif
 
 #ifdef HTTP
 
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: http.c,v 1.32 1997/06/13 13:23:22 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: http.c,v 1.35 1997/09/07 00:31:16 root Exp root $";
+#endif
+
+#ifdef NO_SETENV
+char *setenv (char const *, char const *, int);
 #endif
 
 #define DEFAULT_ERROR_MSG "[an error occurred while processing this directive]"
@@ -46,7 +53,7 @@ static char rcsid[] OPTIONAL = "$Id: http.c,v 1.32 1997/06/13 13:23:22 root Exp
 #define SECS_IN_6MONTHS 15768000 /* seconds in 1/2 year */
 
 static char entry[] = "<IMG SRC=\"/icons/%s.xbm\" ALT=\"%s\"> ";
-static char entry1[] = "<A HREF=\"/%s%s\">%-s</a>";
+static char entry1[] = "<A HREF=\"/%s%s%s\">%-s</a>";
 static char entry2[] = "%2d-%s-%02d %02d:%02d %6dK\n";
 int HttpUsers = 0;
 static int Httpmax = 10;
@@ -60,6 +67,7 @@ static int HTTPusecookies = 0;
 static int HTTPmisc = 1;
 static int HTTPpost = 1;
 static int HTTPexeccmd = 0;
+static int HTTPstatusURL = 1;
 #ifdef HTTPCGI
 static int HTTPexeccgi = 0;
 #endif
@@ -74,11 +82,13 @@ extern int32 MbNonAmprPerms;
 extern int32 MbHttpPerms;
 extern uint32 NonSecureAmpr;
 #endif
+extern time_t StartTime;
 #if 0
 static int HTTPscript = 0;
 #endif
 static FILE * HTTPsavedfp;
 static char *HTTPHostname;
+static char *HTTPFtpdir = NULLCHAR;
 
 #ifdef MSDOS
 #define HTMLEXT	"htm"
@@ -114,6 +124,7 @@ struct reqInfo {
 	char		*agent;
 	char		*passwd;
 	char		*dirname;
+	char		*ftpdirname;
 	char 		*usetime;
 	char		*useerror;
 	char		*realm;
@@ -147,6 +158,7 @@ static void setup_cgi_variables (struct reqInfo *rq, char *name);
 #endif
 static int dohttpagent (int argc,char *argv[],void *p);
 static int dohttpexeccmd (int argc,char *argv[],void *p);
+static int dohttpstatusurl (int argc,char *argv[],void *p);
 static int dohttphostname (int argc,char *argv[],void *p);
 #ifdef HTTPCGI
 static int dohttpexeccgi (int argc,char *argv[],void *p);
@@ -168,6 +180,7 @@ static int dohttpscript (int argc,char *argv[],void *p);
 static int dohttpsim (int argc,char *argv[],void *p);
 static int dohttpstatus (int argc,char *argv[],void *p);
 static int dohttptdisc (int argc,char *argv[],void *p);
+static int dohttpftpdir (int argc,char *argv[],void *p);
 
 #ifdef STRICT_HTTPCALL
 static int dohttpstrict (int argc,char *argv[],void *p);
@@ -175,7 +188,7 @@ static int dohttpstrict (int argc,char *argv[],void *p);
 
 static char *assigncookie (struct reqInfo *rq, time_t thetime);
 static void httpserv (int s,void *unused,void *p);
-static int mkwelcome (const char *Hdir,char *file);
+static int mkwelcome (const char *Hdir,char *file, int ftpfile, const char *Ftpdir);
 static char * decode (char *string);
 static void httpHeaders (int type, const char *str, struct reqInfo *rq, int more, const char *realm);
 static void httpFileinfo (FILE *fp, struct reqInfo *rq, int thetype, char *vers);
@@ -331,7 +344,14 @@ struct portsused {
 	int	sock;
 	uint32	address;
 	char	*dirname;
+	char	*ftpdirname;
 	char	*hostname;
+	long	requests;
+	long	homehits;
+	long	stathits;
+#ifdef HTTPPBBS
+	long	bbshits;
+#endif
 };
 static struct portsused ports[MAXHTTPPORTS];
 
@@ -359,6 +379,7 @@ static struct cmds Httpcmds[] = {
 	{ "execcgi",		dohttpexeccgi,	0, 0, NULLCHAR },
 #endif
 	{ "execcmd",		dohttpexeccmd,	0, 0, NULLCHAR },
+	{ "ftpdir",		dohttpftpdir,	0, 0, NULLCHAR },
 	{ "hostname",		dohttphostname, 0, 0, NULLCHAR },
 	{ "maxcli",		dohttpmax,	0, 0, NULLCHAR },
 	{ "misclogging",	dohttpmisc,	0, 0, NULLCHAR },
@@ -370,6 +391,7 @@ static struct cmds Httpcmds[] = {
 	{ "secure",		dohttpsecure,	0, 0, NULLCHAR },
 	{ "simult",		dohttpsim,	0, 0, NULLCHAR },
 	{ "status",		dohttpstatus,	0, 0, NULLCHAR },
+	{ "statusurl",		dohttpstatusurl,0, 0, NULLCHAR },
 #ifdef STRICT_HTTPCALL
 	{ "strictcall",		dohttpstrict,	0, 0, NULLCHAR },
 #endif
@@ -418,6 +440,7 @@ int found = 0, k;
 	(void) dohttpexeccgi (0, argv, p);
 #endif
 	(void) dohttpexeccmd (0, argv, p);
+	(void) dohttpftpdir (0, argv, p);
 	(void) dohttphostname (0, argv, p);
 	(void) dohttpmax (0, argv, p);
 	(void) dohttpmisc (0, argv, p);
@@ -444,6 +467,14 @@ dohttpexeccmd (int argc, char *argv[], void *p OPTIONAL)
 
 
 
+static int
+dohttpstatusurl (int argc, char *argv[], void *p OPTIONAL)
+{
+	return setbool (&HTTPstatusURL, "Enable status URL (/status)", argc, argv);
+}
+
+
+
 #ifdef HTTPCGI
 static int
 dohttpexeccgi (int argc, char *argv[], void *p OPTIONAL)
@@ -629,6 +660,21 @@ dohttphostname (int argc, char *argv[], void *p OPTIONAL)
 
 
 
+static int
+dohttpftpdir (int argc, char *argv[], void *p OPTIONAL)
+{
+	if (argc < 2)	{
+		tprintf ("Default HTTP FTP directory: %s\n", (HTTPFtpdir) ? HTTPFtpdir : "none - disabled");
+		return 0;
+	}
+	if (HTTPFtpdir)
+		free (HTTPFtpdir);
+	HTTPFtpdir = strdup (argv[1]);
+	return 0;
+}
+
+
+
 /* Start up http service */
 int
 httpstart (int argc, char *argv[], void *p OPTIONAL)
@@ -636,15 +682,19 @@ httpstart (int argc, char *argv[], void *p OPTIONAL)
 int port, k;
 uint32 address = INADDR_ANY;
 
-	if (HTTPHostname == NULLCHAR && Hostname)
-		HTTPHostname = strdup (Hostname);
+	if (HTTPHostname == NULLCHAR) 	{
+		if (Hostname)
+			HTTPHostname = strdup (Hostname);
+		else
+			HTTPHostname = strdup ("localhost");
+	}
 
 	if(argc < 2 || argv[1][0] == '-')
 		port = IPPORT_HTTP;
 	else	{
 		if (argv[1][0] == '?')	{
 			/* they don't really want to start it, they want usage info */
-			tprintf ("usage: start http [<port> [<rootdir> [<hostname> [<ipaddress>]]]]\n");
+			tprintf ("usage: start http [<port> [<rootdir> [<hostname> [<ipaddress>] [ftpdir]]]]\n");
 			return 0;
 		}
 		port = atoi(argv[1]);
@@ -685,6 +735,11 @@ uint32 address = INADDR_ANY;
 	else
 		ports[k].hostname = strdup (argv[3]);
 
+	if (argc < 5 || argv[4][0] == '-')
+		ports[k].ftpdirname = (HTTPFtpdir) ? strdup (HTTPFtpdir) : NULLCHAR;
+	else
+		ports[k].ftpdirname = strdup (argv[4]);
+
 	ports[k].sock = -1;
 	ports[k].port = port;
 	ports[k].address = address;
@@ -755,7 +810,7 @@ static void
 doPBBScmd (struct reqInfo *rq, const char *cmdstr)
 {
 	doPBBScmd_init (rq);
-	strcpy (rq->m->line, cmdstr);
+	strncpy (rq->m->line, cmdstr, MBXLINE);
 	doPBBScmd_term (rq);
 }
 
@@ -770,7 +825,7 @@ doPBBScmd_init (struct reqInfo *rq)
 	rq->m->user = Curproc->output;
 	rq->m->path = rq->bbspath;
 	if (!rq->username)
-		strcpy (rq->username, "anonymous");
+		rq->username = strdup ("anonymous");
 	strncpy (rq->m->name, rq->username, 19);
 	if (rq->areaname)
 		changearea (rq->m, rq->areaname, 0);
@@ -820,7 +875,7 @@ FILE *fp;
 
 	if ((fp = fopen (fname, READ_BINARY)) != NULLFILE) {
 		sendhtml (fp, 1, rq);
-		fclose (fp);
+		(void) fclose (fp);
 	}
 }
 
@@ -851,7 +906,7 @@ FILE *fp;
 
 	if ((fp = fopen (fname, READ_BINARY)) != NULLFILE) {
 		(void) sendfile (fp, s, IMAGE_TYPE, 0);
-		fclose (fp);
+		(void) fclose (fp);
 		return 1;
 	}
 	return 0;
@@ -873,12 +928,14 @@ int posted_message = 0;
 struct sockaddr fsocket;
 uint32 addr;
 register struct usock *up;
-	
+int isValidFtpRequest = 0;
+time_t nowtime;
+
 	(void) sockowner (s, Curproc);
 	(void) sockmode (s, SOCK_ASCII);
 
 	chname (Curproc, "HTTP Server - Init");
-	rq = (struct reqInfo *)calloc (1, sizeof (struct reqInfo));
+	rq = (struct reqInfo *)callocw (1, sizeof (struct reqInfo));
 	if (rq == (struct reqInfo *)0)
 		return;
 		
@@ -888,17 +945,25 @@ register struct usock *up;
 	rq->index = (int) v1;
 	rq->port = ports[rq->index].port;
 	rq->hostname = ports[rq->index].hostname;
+	ports[rq->index].requests++;
+#ifdef STATS_HTTP
+	STATS_addhttp (0);
+#endif
 
 	if (getpeername (s, (char *) &fsocket, &i) != -1)	{
+		port = DTranslate;
+		DTranslate = 0;
 		rq->rmtaddr = strdup (psocket (&fsocket));
 		if ((cp = strchr (rq->rmtaddr, ':')) != NULLCHAR)
 			*cp = 0;
+		addr = aton (rq->rmtaddr);
+		DTranslate = 1;
+		rq->rmthost = strdup (inet_ntoa (addr));
+		DTranslate = port;
+	} else	{
+		rq->rmthost = strdup ("unknown");
+		rq->rmtaddr = strdup ("unknown");
 	}
-	addr = aton (rq->rmtaddr);
-	port = DTranslate;
-	DTranslate = 1;
-	rq->rmthost = strdup (inet_ntoa (addr));
-	DTranslate = port;
 
 	close_s(Curproc->output);
 	close_s(Curproc->input);
@@ -939,6 +1004,10 @@ register struct usock *up;
 
 	port = ports[(int)v1].port;
 	rq->dirname = ports[rq->index].dirname;
+	if (ports[rq->index].ftpdirname == NULLCHAR && HTTPFtpdir)
+		/* newly defined default - allow it to set after the fact */
+		ports[rq->index].ftpdirname = strdup (HTTPFtpdir);
+	rq->ftpdirname = ports[rq->index].ftpdirname;
 	
 	rip (command);
 	if (!*command)	{
@@ -963,6 +1032,9 @@ register struct usock *up;
 	/* at this point, tmp=command, newcommand=ulr, and version=httpversion info */
 	if (!strcmp (newcommand, "/"))
 		newcommand++;
+	if (!*newcommand)
+		ports[rq->index].homehits++;
+	
 	rq->url = strdup (newcommand);
 	rq->version = strdup (version);
 	rq->method = strdup (tmp);
@@ -1039,7 +1111,7 @@ register struct usock *up;
 			}
 
 			/* if this is a posted message, this will be our data line - save it */
-			strcpy (poststr, rq->buf);
+			strncpy (poststr, rq->buf, PLINELEN + 16);
 			break;
 		}
 		kalarm (0L);
@@ -1122,9 +1194,58 @@ post_response:
 		goto quit0;
 	}
 
+	if (HTTPstatusURL && (!strcmp (newcommand, "status") || !strcmp (newcommand, "status/")))	{
+		ports[rq->index].stathits++;
+		rq->isvolatile = 1;
+	  	httpHeaders (HTTPhtml, "200 Document follows", rq, 0, NULLCHAR);
+		usprintf (s, "<HEAD><TITLE>Server Status at %s</TITLE></HEAD>\n", ports[rq->index].hostname);
+		usputs (s, "<HTML><BODY><A HREF=\"http://www.lantz.com/tnos/\"><IMG SRC=\"http://www.lantz.com/tnos/tnos.gif\" align=left></A>");
+		usprintf (s, "<H1>%s:%d</H1>\n<H2>Server Info</H2>\n", ports[rq->index].hostname, ports[rq->index].port);
+
+		cp = ctime (&StartTime);
+		rip (cp);
+		usprintf (s, "<TABLE border=0><tr><td>Start Time</td><td>%s</td></tr>\n", cp);
+		nowtime = time (&nowtime);
+		cp = ctime (&nowtime);
+		rip (cp);
+		usprintf (s, "<tr><td>Current Time&nbsp;&nbsp;</td><td>%s</td></tr>\n", cp);
+
+		usputs (s, "<tr><td>Server</td><td>TNOS/"VERSION" "OS"</td></tr></table><pre>\n\n\n</pre><H3>Current Counters</H3>\n");
+		usprintf (s, "<table border><tr><td>URL Requests</td><td>&nbsp;%ld&nbsp;</td></tr>\n", ports[rq->index].requests);
+		usprintf (s, "<tr><td>Home Page Hits</td><td>&nbsp;%ld&nbsp;</td></tr>\n", ports[rq->index].homehits);
+		usprintf (s, "<tr><td>Status Page Hits&nbsp;</td><td>&nbsp;%ld&nbsp;</td></tr>\n", ports[rq->index].stathits);
+#ifdef HTTPPBBS
+		usprintf (s, "<tr><td>PBBS Hits</td><td>&nbsp;%ld&nbsp;</td></tr>\n", ports[rq->index].bbshits);
+#endif
+		usprintf (s, "<tr><td>Active Requests</td><td>&nbsp;%d&nbsp;</td></tr>\n</table>\n", HttpUsers);
+
+#ifdef STATS_HTTP
+		usputs (s, "<pre>\n\n\n</pre><HR><H3>General Statistics</H3><PRE>\n");
+		doPBBScmd (rq, "stats h g");
+
+		usputs (s, "<pre>\n\n\n</pre><HR><H3>Daily Statistics</H3><PRE>\n");
+		doPBBScmd (rq, "stats h d");
+
+		usputs (s, "<pre>\n\n\n</pre><HR><H3>Weekly Statistics</H3><PRE>\n");
+		doPBBScmd (rq, "stats h w");
+
+		usputs (s, "<pre>\n\n\n</pre><HR><H3>Monthly Statistics</H3><PRE>\n");
+		doPBBScmd (rq, "stats h m");
+
+		usputs (s, "<pre>\n\n\n</pre><HR><H3>Yearly Statistics</H3><PRE>\n");
+		doPBBScmd (rq, "stats h y");
+#endif
+
+		usputs (s, "<pre>\n\n\n</pre><HR><H5>For further information on TNOS can be found at <A HREF=\"http://www.lantz.com/tnos\">TNOS Central</A></H5>\n</HTML>\n");
+		goto quit0;
+	}
 #ifdef HTTPPBBS
 http_bbs_entry:
 	if (!strncmp (newcommand, "bbs", 3))	{
+		ports[rq->index].bbshits++;
+#ifdef STATS_HTTP
+		STATS_addhttp (1);
+#endif
 		/* if no authentication, it fails */
 		if (rq->passwd == NULLCHAR)	{
 			goto failauthorize;
@@ -1159,13 +1280,15 @@ failauthorize:			retval = 401;
 				ii = SOCKSIZE;
 				trans = DTranslate; /* Save IP address translation state */
 				DTranslate = 0;     /* Force output to be numeric IP addr*/
-				(void) getpeername(s,(char *)&fsocket,&ii);
-				cptr = psocket(&fsocket);
-				if ((cptr2 = strchr(cptr, ':')) != NULLCHAR)
-					*cptr2 = 0;
-				if (strcmp(inet_ntoa(NonSecureAmpr), cptr) && (!*cptr || !strnicmp (cptr, "44.", 3) || !strnicmp (cptr, "127.0.0.1", 9)))	{
-					if (MbAmprPerms)
-						rq->privs = MbAmprPerms;
+				if (getpeername(s,(char *)&fsocket,&ii) != -1)	{
+					cptr = psocket(&fsocket);
+					if ((cptr2 = strchr(cptr, ':')) != NULLCHAR)
+						*cptr2 = 0;
+					if (strcmp(inet_ntoa(NonSecureAmpr), cptr) && (!*cptr || !strnicmp (cptr, "44.", 3) || !strnicmp (cptr, "127.0.0.1", 9)))	{
+						if (MbAmprPerms)
+							rq->privs = MbAmprPerms;
+					} else if (MbNonAmprPerms)
+						rq->privs = MbNonAmprPerms;
 				} else if (MbNonAmprPerms)
 					rq->privs = MbNonAmprPerms;
 				DTranslate = trans;             /* Restore original state */
@@ -1212,7 +1335,6 @@ failauthorize:			retval = 401;
 		if (!strncmp (newcommand, "bbs/message/", 12) && !strstr (newcommand, ".htm"))	{
 			cp = &newcommand[12];
 			sprintf (rq->buf, "bbs/area/%s.%s", cp, HTMLEXT);
-			strcpy (rq->buf, rq->buf);
 			free (rq->url);
 			rq->url = strdup (rq->buf);
 			retval = 302;
@@ -1238,7 +1360,7 @@ failauthorize:			retval = 401;
 		if (!strncmp (newcommand, "bbs/area/", 9) && !strchr(&newcommand[9], '/') && strstr (newcommand, ".htm"))	{
 			char *cp2;
 			/* special URL for area listing */
-		  	strcpy (rq->buf, &newcommand[9]);
+		  	strncpy (rq->buf, &newcommand[9], PLINELEN + 16);
 		  	cp2 = strstr (rq->buf, ".htm");
 		  	if (cp2)
 				*cp2 = 0;
@@ -1263,7 +1385,7 @@ failauthorize:			retval = 401;
 		if (!strncmp (newcommand, "bbs/area/", 9))	{
 			char *cp2, *cp3;
 			/* special URL for area listing */
-		  	strcpy (rq->buf, &newcommand[9]);
+		  	strncpy (rq->buf, &newcommand[9], PLINELEN + 16);
 		  	if ((cp2 = strchr (rq->buf, '/')) == NULLCHAR)
 		  		goto failauthorize;
 		  	*cp2++ = 0;
@@ -1301,7 +1423,7 @@ failauthorize:			retval = 401;
 		if (!strncmp (newcommand, "bbs/message/", 12))	{
 			char *cp2, *cp3;
 			/* special URL for reading messages */
-		  	strcpy (rq->buf, &newcommand[12]);
+			strncpy (rq->buf, &newcommand[12], PLINELEN + 16);
 		  	if ((cp2 = strchr (rq->buf, '/')) == NULLCHAR)
 		  		goto failauthorize;
 		  	*cp2++ = 0;
@@ -1368,7 +1490,7 @@ failauthorize:			retval = 401;
 				sprintf (fromaddr, "%s/bbs/sendfail.%s", HTTPdir, HTMLEXT);
 				if ((postfp = fopen (fromaddr, "r")) != NULLFILE)	{
 					sendhtml (postfp, 1, rq);
-					fclose (postfp);
+					(void) fclose (postfp);
 				} else
 					usputs (s, "</BODY>\n");
 
@@ -1431,7 +1553,7 @@ failauthorize:			retval = 401;
 				usputs (s, "<HEAD><TITLE>400 Bad Request</TITLE></HEAD>\n<BODY><H1>400 Bad Request</H1>\nYour client sent a query that this server could not understand.<P>\nReason: No email address given.<P></BODY>\n");
 			}
 
-			fclose (postfp);
+			(void) fclose (postfp);
 			free (toaddr);
 			free (subject);
 			goto quit0;
@@ -1451,8 +1573,18 @@ failauthorize:			retval = 401;
 
 	if (length == 0)
 		sprintf(rq->buf,"%s/index.%s", rq->dirname, HTMLEXT);
-	else
-		sprintf(rq->buf,"%s/%s", rq->dirname, newcommand);
+	else	{
+		if (rq->ftpdirname && !strncmp (newcommand, "ftp", 3))	{
+			char *newercommand = newcommand;
+			/* viewing/retrieving files from defined FTP directory */
+			newercommand += 3;
+			if (*newercommand && *newercommand == '/')
+				newercommand++;
+			sprintf(rq->buf, "%s/%s", rq->ftpdirname, newercommand);
+			isValidFtpRequest = 1;
+		} else
+			sprintf(rq->buf,"%s/%s", rq->dirname, newcommand);
+	}
 
 	/* delete any .. to avoid */
 	/* anyone telnetting to http and cd'ing to a directory */
@@ -1469,16 +1601,16 @@ failauthorize:			retval = 401;
 		isdir = sb.st_mode & S_IFDIR;
 	if (rq->buf[strlen(rq->buf)-1] == '/') {
 		strcat (rq->buf, "welcome." HTMLEXT);
-		if (!access (rq->buf, 0))	{
+		if (isValidFtpRequest || !access (rq->buf, 0))	{
 			retval = 200;
 		  	httpHeaders (HTTPhtml, "200 Document follows", rq, 0, NULLCHAR);
 		  	if (!head)
-				err = mkwelcome (rq->dirname, rq->buf);
+				err = mkwelcome (rq->dirname, rq->buf, isValidFtpRequest, rq->ftpdirname);
 			goto quit0;
 		}
 	} else if (isdir) {
 		retval = 302;
-		strcpy (rq->buf, newcommand);
+		strncpy (rq->buf, newcommand, PLINELEN + 15);
 		strcat (rq->buf, "/");
 		free (rq->url);
 		rq->url = strdup (rq->buf);
@@ -1523,7 +1655,7 @@ failauthorize:			retval = 401;
 			} else				/* otherwise, scan for server-side includes */
 				sendhtml (fp, 1, rq);
 		}
-		fclose (fp);
+		(void) fclose (fp);
 	} else if (err) {
 		retval = 404;
 		httpHeaders (HTTPhtml, "404 Not Found", rq, 0, NULLCHAR);
@@ -1623,10 +1755,10 @@ struct sockaddr_in fsocket;
 FILE *fp;
 long nextCookie = 0;
 int assignone = 1;
-unsigned long address;
+unsigned long address = INADDR_ANY;
 
-	(void) getpeername (rq->sock, (char *) &fsocket, &i);
-	address = fsocket.sin_addr.s_addr;
+	if (getpeername (rq->sock, (char *) &fsocket, &i) != -1)
+		address = fsocket.sin_addr.s_addr;
 
 	/* build ipaddress portion of cookie, which we'll also use to
 	   lookup this site, and see if there is already an assigned cookie */
@@ -1640,7 +1772,7 @@ unsigned long address;
 		cookie[0] = 0;
 		(void) fgets (cookie, 30, fp);
 		nextCookie = atol (cookie);
-		fclose (fp);
+		(void) fclose (fp);
 	}
 	sprintf (cookie, "%8.8s_%08ld_%012ld", ipaddr, ++nextCookie, thetime);
 
@@ -1650,20 +1782,20 @@ unsigned long address;
 	if ((fp = fopen (Cookiedata, READ_TEXT)) != NULLFILE)	{
 		while (fgets (buf, 32, fp) != NULLCHAR)	{
 			if (!strncmp (buf, ipaddr, 8))	{
-				strcpy (cookie, buf);
+				strncpy (cookie, buf, 32);
 				rip (cookie);
 				assignone = 0;
 				break;
 			}
 		}
-		fclose (fp);
+		(void) fclose (fp);
 	}
 	
 	
 	if (assignone)	{	/* if we didn't find an old one, save this info */
 		if ((fp = fopen (Cookiefile, WRITE_TEXT)) != NULLFILE)	{
 			fprintf (fp, "%ld\n", nextCookie);
-			fclose (fp);
+			(void) fclose (fp);
 		} else	{
 			/* if we can't save them, log an error, and disable creating more */
 			log (-1, "Cannot write data to Cookiefile: '%s'", Cookiefile);
@@ -1672,7 +1804,7 @@ unsigned long address;
 
 		if ((fp = fopen (Cookiedata, APPEND_TEXT)) != NULLFILE)	{
 			fprintf (fp, "%s\n", cookie);
-			fclose (fp);
+			(void) fclose (fp);
 		}
 	}
 
@@ -1712,7 +1844,7 @@ time_t thetime;
 
 
 static int
-mkwelcome (const char *Hdir, char *file)
+mkwelcome (const char *Hdir, char *file, int ftpfile, const char *Ftpdir)
 {
 char *dirstring,*p,*cp;
 const char *ccp, *bitmap;
@@ -1720,6 +1852,7 @@ struct ffblk ffblk;
 int done,pre = 0, k;
 char *updir = NULLCHAR;
 char *endstr;
+int isadir;
 
 	if ((dirstring = (char *)mallocw (strlen(file) + 16)) == NULL)
 		return 0;
@@ -1730,8 +1863,18 @@ char *endstr;
 	cp = strrchr (dirstring, '/');
 	if (cp)
 		*++cp = '\0';
-	if (strlen(Hdir) != strlen(dirstring))	{
-		updir = strdup(&dirstring[strlen(Hdir) + 1]);
+	if (!ftpfile && strlen(Hdir) != strlen(dirstring))	{
+		updir = strdup(&dirstring[strlen(Hdir)]);
+		p = strrchr (updir, '/');
+		if (p)
+			*p = 0;
+		p = strrchr (updir, '/');
+		if (p == NULLCHAR)
+			updir[0] = 0;
+		else
+			*++p = 0;
+	} else if (ftpfile && strlen(Ftpdir) != strlen(dirstring))	{
+		updir = strdup(&dirstring[strlen(Ftpdir)]);
 		p = strrchr (updir, '/');
 		if (p)
 			*p = 0;
@@ -1744,11 +1887,11 @@ char *endstr;
 	
 	endstr = &dirstring[strlen(dirstring)];
 
-	strcat (dirstring, "wwwpre.cat");
+	strcat (dirstring, HTTPheadfile);
 	pre = addFile (dirstring, Curproc->output);
 
 	strcpy (endstr, WILDCARD);
-	done = findfirst (dirstring, &ffblk, FA_DIREC);
+	done = findfirst (dirstring, &ffblk, (FA_HIDDEN|FA_SYSTEM|FA_DIREC));
 	if (done)	{
 		free (dirstring);
 		return -1;
@@ -1757,23 +1900,36 @@ char *endstr;
 	
 	(void) sockmode (Curproc->output, SOCK_ASCII);
 	(void) setflush (Curproc->output, -1); /* we will do our own flushing */
-	p = dirstring + strlen(Hdir) + 1;
+	p = dirstring + 1 + ((ftpfile) ? strlen(Ftpdir) : strlen(Hdir));
+	if (ftpfile)	{
+		p = (char *) mallocw (strlen (p) + 5);
+		sprintf (p, "ftp/%s", dirstring + 1 + ((ftpfile) ? strlen(Ftpdir) : strlen(Hdir)));
+	}
 	undosify (p);
 	if (!pre)
 		tprintf ("<HEAD><TITLE>Index of /%s</TITLE></HEAD></BODY><H1>Index of /%s</H1>\n", p, p);
 	tputs ("<PRE>\n");
 
-	tputs ("<IMG SRC=\"/icons/blank.xbm\" ALT=\"    \"> Name                   Last modified     Size\n<HR>\n");
+	/* include the 'include' file, if it exists in this directory */
+	strcpy (endstr, HTTPinclude);
+	if (addFile (dirstring, Curproc->output))
+		tputs ("<P>\n");
+	*endstr = 0;
+
+	tputs ("<IMG SRC=\"/icons/blank.xbm\" ALT=\"      \"> Name                   Last modified     Size\n<HR>\n");
 	if (updir != NULLCHAR)	{
-		tprintf ("<IMG SRC=\"/icons/menu.xbm\" ALT=\"[DIR]\"> <A HREF=\"/%s\">Parent Directory</a>\n", updir);
+		if (*updir)
+			tprintf ("<IMG SRC=\"/icons/menu.xbm\" ALT=\"[DIR ]\"> <A HREF=\"%s%s\">Parent Directory</a>\n", (ftpfile) ? "/ftp" : "", updir);
 		free (updir);
 	}
 	while (!done) {
-		(void) strlwr (ffblk.ff_name);
-		if (ffblk.ff_name[0] != '.') {
-			if (ffblk.ff_attrib == FA_DIREC)	{
-				ccp = "[DIR]";
+		isadir = 0;
+
+		if (ffblk.ff_name[0] != '.')	{
+			if (ffblk.ff_attrib & FA_DIREC)	{
+				ccp = "[DIR ]";
 				bitmap = "menu";
+				isadir = 1;
 			} else {
 				ccp = "[FILE]";
 				bitmap = "text";
@@ -1782,7 +1938,7 @@ char *endstr;
 					bitmap = "image";
 			}
 			tprintf (entry, bitmap, ccp);
-			tprintf (entry1, p, ffblk.ff_name, ffblk.ff_name);
+			tprintf (entry1, p, ffblk.ff_name, (isadir) ? "/" : "", ffblk.ff_name);
 			for (k = (int) strlen (ffblk.ff_name); k < 22; k++)
 				tputc (' ');
 			tprintf (entry2,
@@ -1807,10 +1963,12 @@ char *endstr;
 	tputs ("</PRE>\n");
 	usflush (Curproc->output);
 
-	strcat (dirstring, "wwwpost.cat");
+	strcat (dirstring, HTTPtailfile);
 	if (!addFile (dirstring, Curproc->output))
 		tputs ("</BODY>\n");
 	free (dirstring);
+	if (ftpfile)
+		free (p);
 	return 0;
 }
 
@@ -1835,7 +1993,7 @@ static void
 closeHTTPlog (void)
 {
 	if (Logfp != NULLFILE)
-		fclose (Logfp);
+		(void) fclose (Logfp);
 	Logfp = HTTPsavedfp;
 }
 
@@ -1866,14 +2024,14 @@ long count = 0;
 	sprintf (name,"%s/counts/%s", basedir, file);
 	if ((fp = fopen (name, READ_TEXT)) != NULLFILE) {
 		(void) fgets (buf, sizeof(buf), fp);
-		fclose (fp);
+		(void) fclose (fp);
 		count = atol (buf);
 	}
 	if (increase)	{
 		count++;
 		if ((fp = fopen (name, WRITE_TEXT)) != NULLFILE) {
 			fprintf (fp, "%ld", count);
-			fclose (fp);
+			(void) fclose (fp);
 		}
 	}
 	if (display)
@@ -1928,15 +2086,15 @@ time_t offset;
 	}
 	if (t.tm_year > 1900)
 		t.tm_year -= 1900;
-#if !defined(sun) && !defined(__bsdi__)
+#ifdef HAVE_TM_ISDST
 	t.tm_isdst = -1;
 #endif
 	new = mktime (&t);
 	ltm = localtime(&new);		/* to get timezone info */
-#if defined(sun) || defined(__bsdi__) || defined(MSDOS) || defined(__FreeBSD__)
-	offset = (ltm->tm_gmtoff * -1);
-#else
+#if defined(HAVE_TM_ISDST) && !defined(MSDOS)
 	offset = ((timezone - (ltm->tm_isdst * 3600L)) * -1);
+#else
+	offset = (ltm->tm_gmtoff * -1);
 #endif
 	new += offset;
 	return (new != thetime);
@@ -2124,7 +2282,7 @@ int val;
 							sprintf (rq->buf, "%s/bbs/plbutton.cat", rq->dirname);
 							if ((tfp = fopen (rq->buf, "r")) != NULLFILE) {
 								sendhtml (tfp, 0, rq);
-								fclose (tfp);
+								(void) fclose (tfp);
 							}
 						}
 					} else if (!stricmp (cp1, "PREV_MSG_BUTTON"))	{
@@ -2132,7 +2290,7 @@ int val;
 							sprintf (rq->buf, "%s/bbs/pmbutton.cat", rq->dirname);
 							if ((tfp = fopen (rq->buf, "r")) != NULLFILE) {
 								sendhtml (tfp, 0, rq);
-								fclose (tfp);
+								(void) fclose (tfp);
 							}
 						}
 					} else if (!stricmp (cp1, "NEXT_MSG_BUTTON"))	{
@@ -2140,7 +2298,7 @@ int val;
 							sprintf (rq->buf, "%s/bbs/nmbutton.cat", rq->dirname);
 							if ((tfp = fopen (rq->buf, "r")) != NULLFILE) {
 								sendhtml (tfp, 0, rq);
-								fclose (tfp);
+								(void) fclose (tfp);
 							}
 						}
 					} else if (!stricmp (cp1, "NEXT_LIST_BUTTON"))	{
@@ -2148,7 +2306,7 @@ int val;
 							sprintf (rq->buf, "%s/bbs/nlbutton.cat", rq->dirname);
 							if ((tfp = fopen (rq->buf, "r")) != NULLFILE) {
 								sendhtml (tfp, 0, rq);
-								fclose (tfp);
+								(void) fclose (tfp);
 							}
 						}
 					} else if (!stricmp (cp1, "AREA_LIST") || !stricmp (cp1, "AREA_MSG"))
@@ -2201,7 +2359,7 @@ int val;
 									cp2 = strrchr (++cp1, '\"');
 									if (cp2)
 										*cp2 = 0;
-									(void) countusage (cp1, rq->dirname, (c != 's'), (c != 'c'));	/*lint !e730 */
+									(void) countusage (cp1, rq->dirname, (c != 's'), (c != 'c'));
 								}
 								break;
 						default:	tputs (rq->useerror);
@@ -2232,7 +2390,7 @@ int val;
 				file_or_virtual (cp1, newname, rq);
 				if (*newname && (tfp = fopen (newname, "r")) != NULLFILE) {
 					sendhtml (tfp, 0, rq);
-					fclose (tfp);
+					(void) fclose (tfp);
 				} else
 					tputs (rq->useerror);
 			} else if (!strnicmp (cmdname, "fsize", 5)) {
@@ -2401,7 +2559,7 @@ int retval = FALSE;
 				break;
 			}
 		}
-		fclose(fp);
+		(void) fclose(fp);
 	}
 	return retval;
 }
@@ -2540,6 +2698,19 @@ struct options *opt;
 	return NULLCHAR;	
 }
 
+#ifdef NO_SETENV
+char *
+setenv (char const *label, char const *value, int unused OPTIONAL)
+{
+char *temp;
+
+	temp = (char *) mallocw (strlen (label) + strlen (value) + 2);
+	sprintf (temp, "%s=%s", label, value);
+	(void) putenv (temp);
+	free (temp);
+	return NULLCHAR;
+}
+#endif
 
 
 #endif
