@@ -15,11 +15,12 @@
 #include "pool.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: ipcmd.c,v 1.19 1997/01/18 16:13:20 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: ipcmd.c,v 1.21 1997/09/07 00:31:16 root Exp root $";
 #endif
 
 uint32 Ip_addr;
 static int Route_Sort = 1;
+int Route_Same = 0;
 
 
 static int doadd (int argc, char *argv[], void *p);
@@ -39,6 +40,7 @@ static int doiphport (int argc, char *argv[], void *p);
 static int doiphsize (int argc, char *argv[], void *p);
 void dumproute (struct route * rp, char *p);
 static int doroutesort (int argc, char *argv[], void *p);
+static int doroutesame (int argc, char *argv[], void *p);
 
 /* IP heard logging - WG7J */
 static struct iph *iph_create (uint32 addr);
@@ -64,9 +66,10 @@ static int Maxipheard = MAXIPHEARD;
 #define heardhdr	__STR(8)
 #define sortstr		__STR(9)
 #define iphsizestr	__STR(10)
+#define samestr		__STR(11)
 
 #else /* CATALOG */
-static const char accesshdr[] = "Source Address  Len Dest Address    Len Interface    Proto Low   High  State\n";
+static const char accesshdr[] = "Source Address  Len Dest Address    Len Interface    Proto   Low  High  State\n";
 static const char accesssyntax[] = " Format: ip access <permit|deny|delete> <proto> <src addr>[/<bits>] <dest addr>[/<bits>] <if name> [low [high]]\n";
 static const char rtimerstr[] = "IP reasm timeout (sec)";
 static const char ttlstr[] = "IP Time-to-live";
@@ -77,6 +80,7 @@ static const char noaddroute[] = "Can't add route\n";
 static const char heardhdr[] = "Tcp/Ip systems heard:\nAddress                Port       Since       Pkts\n";
 static const char sortstr[] = "Route Sort flag";
 static const char iphsizestr[] = "Max ip-heard";
+static const char samestr[] = "Permit IP routing out same interface it was received";
 #endif /* CATALOG */
 
 
@@ -118,6 +122,7 @@ static struct cmds Ipcmds[] =
 	{ "heard",	doipheard,	0, 0, NULLCHAR },
 	{ "hport",	doiphport,	0, 0, NULLCHAR },
 	{ "hsize",	doiphsize,	0, 0, NULLCHAR },
+	{ "routesame",	doroutesame,	0, 0, NULLCHAR },
 	{ "rtimer",	dortimer,	0, 0, NULLCHAR },
 	{ "status",	doipstat,	0, 0, NULLCHAR },
 	{ "ttl",	dottl,		0, 0, NULLCHAR },
@@ -437,7 +442,7 @@ unsigned int a = 4;
 char *name;
 
 	if (rp->target != 0) {
-		if (DTranslate && (name = resolve_a (rp->target, !DVerbose)) != NULLCHAR) {	/*lint !e730 */
+		if (DTranslate && (name = resolve_a (rp->target, !DVerbose)) != NULLCHAR) {
 			strcpy (temp, name);
 			free (name);
 		} else {
@@ -479,6 +484,13 @@ doroutesort (int argc, char *argv[], void *p OPTIONAL)
 }
 
 
+static int
+doroutesame (int argc, char *argv[], void *p OPTIONAL)
+{
+	return setbool (&Route_Same, samestr, argc, argv);
+}
+
+
 /* Add an entry to the routing table
  * E.g., "add 1.2.3.4 ax0 5.6.7.8 3"
  */
