@@ -17,20 +17,23 @@
 
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: iproute.c,v 1.16 1997/01/18 16:13:20 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: iproute.c,v 1.20 1997/09/07 21:18:28 root Exp root $";
 #endif
 
 extern int Icmp_timeexceed;
+extern int Route_Same;
 
 struct route *Routes[32][HASHMOD];	/* Routing table */
-/*lint -save -e785 */
+
 struct route R_default =
 {				/* Default route entry */
 	NULLROUTE, NULLROUTE,
 	0, 0, 0,
-	RIP_INFINITY		/* Init metric to infinity */
+	RIP_INFINITY,		/* Init metric to infinity */
+	NULLIF, 0,
+	{ 0, NULLTIMER, 0, 0, (void (*) (void *)) 0, NULL, 0, NULL, 0},
+	0, 0
 };
-/*lint -restore */
 
 static struct rt_cache Rt_cache;
 
@@ -85,8 +88,10 @@ int i;
 struct mbuf *tbp;
 int ckgood = IP_CS_OLD;	/* Has good checksum without modification */
 #ifdef IPACCESS
-/*lint -esym(550,srcport) */
-int16 srcport, dstport = 0;	/* for use in access checking */
+#if 0
+int16 src_port;
+#endif
+int16 dstport = 0;	/* for use in access checking */
 struct rtaccess *tpacc;	/* temporary for access checking */
 #endif
 int pointer;		/* Relative pointer index for sroute/rroute */
@@ -307,6 +312,14 @@ no_opt:
 
 	/* Check for output forwarding and divert if necessary */
 	iface = rp->iface;
+
+	/* to prevent IP digipeating to same interface on non-RF media */
+	if (Route_Same && iface == i_iface && iface->iftype->type != CL_AX25 && iface->iftype->type != CL_NETROM)	{
+		free_p (bp);
+		ipForwDatagrams--;
+		return 0;
+	}
+	
 	if (iface->forw != NULLIF)
 		iface = iface->forw;
 
@@ -317,7 +330,9 @@ no_opt:
 	 */
 	if ((ip.protocol == TCP_PTCL) || (ip.protocol == UDP_PTCL)) {
 		/* pull up source & destination port */
-		srcport = get16 ((char *) &(bp->data[0]));	/*done for futures & debug*/
+#if 0
+		src_port = get16 ((char *) &(bp->data[0]));	/*done for futures & debug*/
+#endif
 		dstport = get16 ((char *) &(bp->data[2]));
 	}
 	/* for backwards compatibility, if there are *no* entries */
@@ -327,9 +342,10 @@ no_opt:
 	for (tpacc = IPaccess; tpacc != NULLACCESS; tpacc = tpacc->nxtiface) {
 		/*find a matching iface*/
 		if (tpacc->iface == iface) {	/* Needs to be authorized */
-			if (!ip_check (tpacc, (int16) (int) ip.protocol, ip.source, ip.dest, dstport))
+			if (ip_check (tpacc, (int16) (int) ip.protocol, ip.source, ip.dest, dstport) <= 0)
 				break;
-			/* not found, throw it away. */
+
+			/* denied, throw it away. */
 			/* if NO access, then drop & increment counter */
 			/* see new(er) RFC for proper ICMP codes */
 			/* maybe counter is in MIB? else create.*/
@@ -535,8 +551,7 @@ uint32 gwtmp;
 	rp->timer.func = rt_timeout;	/* Set the timer field */
 	rp->timer.arg = (void *) rp;
 	set_timer (&rp->timer, ttl * 1000L);
-	stop_timer (&rp->timer);
-	start_timer (&rp->timer);	/* start the timer if appropriate */
+	start_detached_timer (&rp->timer);	/* start the timer if appropriate */
 
 #ifdef ENCAP
 	/* Check to see if this created an encapsulation loop */
@@ -824,6 +839,7 @@ uint32 src, dest;
 {
 uint32 smask, tmask;
 unsigned short stmp, ttmp;
+int retval = -1;
 
 	for (; accptr != NULLACCESS; accptr = accptr->nxtbits) {
 		if ((accptr->protocol == 0) ||
@@ -843,11 +859,19 @@ unsigned short stmp, ttmp;
 			    ((accptr->lowport == 0) ||
 			     ((port >= accptr->lowport) &&
 			      (port <= accptr->highport)))) {
-				return (accptr->status);
+			   	/* if this is a default entry, save the status,
+		   		   but look in case there is a more specific entry
+			   	   later in the list. If not, upon exit, we will
+			   	   return the proper status
+			   	 */
+			      	if (!accptr->lowport && !accptr->highport)
+		   			retval = accptr->status;
+		   		else
+					return (accptr->status);
 			}
 		}
 	}
-	return -1;		/* fall through to here if not found */
+	return retval;		/* fall through to here if specific entry not found */
 }
 
 
@@ -879,8 +903,12 @@ struct rtaccess *holder;/*for the new record*/
 	holder->lowport = low;
 	holder->highport = high;
 	holder->status = permit;
+
 	for (tpacc = IPaccess; tpacc != NULLACCESS; tpacc = tpacc->nxtiface)
 		if (tpacc->iface == ifp) {	/* get to end */
+			if ((low || high) && ip_check (tpacc, protocol, source, target, low) == -1)
+				addaccess (protocol, source, sbits, target, tbits, ifp, 0, 0, ~permit);
+
 			while (tpacc->nxtbits != NULLACCESS)
 				tpacc = tpacc->nxtbits;
 			tpacc->nxtbits = holder;
@@ -889,6 +917,8 @@ struct rtaccess *holder;/*for the new record*/
 	/* iface wasn't found, so just add at head of list */
 	holder->nxtiface = IPaccess;
 	IPaccess = holder;
+
+	addaccess (protocol, source, sbits, target, tbits, ifp, 0, 0, ~permit);
 }
 
 #endif
