@@ -8,7 +8,7 @@
 #include "kisspoll.h"
   
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: kisspoll.c,v 1.9 1996/12/29 02:47:22 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: kisspoll.c,v 1.10 1997/08/19 01:19:22 root Exp root $";
 #endif
 
 #ifdef POLLEDKISS
@@ -30,7 +30,7 @@ struct slip *sp = &Slip[xdev];
 	/* The actual physical interface */
 	master = sp->kiss[0];
   
-	while (1) {		/*lint !e716 !e774 */
+	for ( ; ; ) {
 		for (i = 0; i < 16; i++) {
 			if ((iface=sp->kiss[i]) == NULL)
 				continue;
