@@ -11,15 +11,9 @@
 #include "netrom.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: lapb.c,v 1.14 1996/12/29 02:47:22 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: lapb.c,v 1.17 1997/09/14 14:37:46 root Exp root $";
 #endif
 
-#ifdef AXNEGOTIATION
-static void ax_negotiate (struct ax25_cb * axp, struct mbuf * bp);
-#endif
-
-/*lint -esym(534, sendctl, ackours) */
-
 /* This forces data sent by jumpstarting a mailbox connect to
  * resend the header etc. when the first UA reply is missed, and
  * a second SABM is received for the same connection
@@ -46,7 +40,6 @@ extern struct no_js *Exclude;
 #endif
 
 
-
 /*If we have an AX.25 AUTO route record, drop it.*/
 static void
 drop_axr (struct ax25_cb *axp)
@@ -81,7 +74,7 @@ int recovery = 0;
 struct no_js *ep;	/*scan the 'exclude-calls' list */
 #endif
 
-	if (bp == NULLBUF || axp == NULLAX25) {
+	if (bp == NULLBUF || axp == NULLAX25 || axp->iface == NULLIF) {
 		free_p (bp);
 		return -1;
 	}
@@ -90,6 +83,10 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 		free_p (bp);	/* Probably not necessary */
 		return -1;
 	}
+
+	/* update the counters for messages in */
+	axp->iface->axcnt.msgin++;
+
 	type = ftype (control);
 	class = type & 0x3;
 	pf = control & PF;
@@ -122,7 +119,7 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 			switch (type) {
 				case SABM:	/* Initialize or reset link */
 					/* This a new incoming connection. */
-					sendctl (axp, LAPB_RESPONSE, UA | pf);	/* Always accept */
+					(void) sendctl (axp, LAPB_RESPONSE, UA | pf);	/* Always accept */
 					clr_ex (axp);
 					axp->unack = axp->vr = axp->vs = 0;
 					lapbstate (axp, LAPB_CONNECTED);	/* Resets state counters */
@@ -170,13 +167,13 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 #endif /* MAILBOX */
 					break;
 				case DISC:	/* Always answer a DISC with a DM */
-					sendctl (axp, LAPB_RESPONSE, DM | pf);
+					(void) sendctl (axp, LAPB_RESPONSE, DM | pf);
 					break;
 				case DM:	/* Ignore to avoid infinite loops */
 					break;
 				default:	/* All others get DM */
 					if (poll)
-						sendctl (axp, LAPB_RESPONSE, DM | pf);
+						(void) sendctl (axp, LAPB_RESPONSE, DM | pf);
 					break;
 			}
 			if (axp->state == LAPB_DISCONNECTED) {	/* we can close connection */
@@ -195,10 +192,10 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 		case LAPB_SETUP:
 			switch (type) {
 				case SABM:	/* Simultaneous open */
-					sendctl (axp, LAPB_RESPONSE, UA | pf);
+					(void) sendctl (axp, LAPB_RESPONSE, UA | pf);
 					break;
 				case DISC:
-					sendctl (axp, LAPB_RESPONSE, DM | pf);
+					(void) sendctl (axp, LAPB_RESPONSE, DM | pf);
 #ifdef NETROM
 					nr_derate (axp);
 #endif
@@ -228,18 +225,18 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 
 				default:	/* Respond with DM only to command polls */
 					if (poll)
-						sendctl (axp, LAPB_RESPONSE, DM | pf);
+						(void) sendctl (axp, LAPB_RESPONSE, DM | pf);
 					break;
 			}
 			break;
 		case LAPB_DISCPENDING:
 			switch (type) {
 				case SABM:
-					sendctl (axp, LAPB_RESPONSE, DM | pf);
+					(void) sendctl (axp, LAPB_RESPONSE, DM | pf);
 					drop_axr (axp);	/* drop ax25 route */
 					break;
 				case DISC:
-					sendctl (axp, LAPB_RESPONSE, UA | pf);
+					(void) sendctl (axp, LAPB_RESPONSE, UA | pf);
 					drop_axr (axp);	/* drop ax25 route */
 					break;
 				case UA:
@@ -251,7 +248,7 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 
 				default:	/* Respond with DM only to command polls */
 					if (poll)
-						sendctl (axp, LAPB_RESPONSE, DM | pf);
+						(void) sendctl (axp, LAPB_RESPONSE, DM | pf);
 					drop_axr (axp);	/* drop ax25 route */
 					break;
 			}
@@ -261,7 +258,7 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 		case LAPB_CONNECTED:
 			switch (type) {
 				case SABM:
-					sendctl (axp, LAPB_RESPONSE, UA | pf);
+					(void) sendctl (axp, LAPB_RESPONSE, UA | pf);
 #ifdef SABM_HOLDOFF
 					if (axp->flags.rxd_I_frame) {
 						/* only reset if we've had a */
@@ -283,7 +280,7 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 					break;
 				case DISC:
 					free_q (&axp->txq);
-					sendctl (axp, LAPB_RESPONSE, UA | pf);
+					(void) sendctl (axp, LAPB_RESPONSE, UA | pf);
 					stop_timer (&axp->t1);
 					stop_timer (&axp->t3);
 					axp->reason = LB_NORMAL;
@@ -306,15 +303,24 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 					lapbstate (axp, LAPB_SETUP);	/* Re-establish */
 					break;
 				case FRMR:
+					/* update the counters for FRMR in */
+					if (axp && axp->iface)
+						axp->iface->axcnt.frmerr++;
+						
 					est_link (axp);
 					lapbstate (axp, LAPB_SETUP);	/* Re-establish link */
 					break;
 				case RR:
 				case RNR:
 					axp->flags.remotebusy = (type == RNR) ? YES : NO;
+
+					if (type == RNR)
+						/* update the counters for RNR in */
+						axp->iface->axcnt.rnrin++;
+
 					if (recovery && (axp->proto == V1 || final)) {
 						stop_timer (&axp->t1);
-						ackours (axp, nr);
+						(void) ackours (axp, nr);
 						if (axp->unack != 0) {
 							if (control != RNR || axp->pthresh != 65535)
 								inv_rex (axp);
@@ -331,7 +337,7 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 					} else {
 						if (poll)
 							enq_resp (axp);
-						ackours (axp, nr);
+						(void) ackours (axp, nr);
 						/* Keep timer running even if all frames
 						 * were acked, since we must see a Final
 						 */
@@ -348,11 +354,15 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 					break;
 				case REJ:
 					axp->flags.remotebusy = NO;
+
+					/* update the counters for rej in */
+					axp->iface->axcnt.rejin++;
+
 					if (recovery) {
 						/* Don't insist on a Final response from the old proto */
 						if (axp->proto == V1 || final) {
 							stop_timer (&axp->t1);
-							ackours (axp, nr);
+							(void) ackours (axp, nr);
 							if (axp->unack != 0) {
 								inv_rex (axp);
 							} else {
@@ -364,7 +374,7 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 						} else {
 							if (poll)
 								enq_resp (axp);
-							ackours (axp, nr);
+							(void) ackours (axp, nr);
 							if (axp->unack != 0) {
 								/* This is certain to trigger output */
 								inv_rex (axp);
@@ -379,7 +389,7 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 					} else {
 						if (poll)
 							enq_resp (axp);
-						ackours (axp, nr);
+						(void) ackours (axp, nr);
 						stop_timer (&axp->t1);
 						start_timer (&axp->t3);
 						/* This may or may not actually invoke transmission,
@@ -390,7 +400,10 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 					}
 					break;
 				case I:
-					ackours (axp, nr);	/** == -1) */
+					(void) ackours (axp, nr);	/** == -1) */
+
+					/* update the counters for data in */
+					axp->iface->axcnt.datain++;
 #ifdef SABM_HOLDOFF
 					axp->flags.rxd_I_frame = 1;	/* we got something */
 #endif
@@ -408,8 +421,12 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 						 * drastic action is necessary to avoid
 						 * deadlock.
 						 */
-						if (recovery || poll)
-							sendctl (axp, LAPB_RESPONSE, RNR | pf);
+						if (recovery || poll)	{
+							(void) sendctl (axp, LAPB_RESPONSE, RNR | pf);
+							
+							/* update the counters for rnr out */
+							axp->iface->axcnt.rnrout++;
+						}
 						free_p (bp);
 						bp = NULLBUF;
 						break;
@@ -418,7 +435,10 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 					if ((char) ns != axp->vr) {
 						if (axp->proto == V1 || !axp->flags.rejsent) {
 							axp->flags.rejsent = YES;
-							sendctl (axp, LAPB_RESPONSE, REJ | pf);
+							(void) sendctl (axp, LAPB_RESPONSE, REJ | pf);
+
+							/* update the counters for rej out */
+							axp->iface->axcnt.rejout++;
 						} else if (poll)
 							enq_resp (axp);
 						axp->response = 0;
@@ -427,9 +447,13 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 					axp->flags.rejsent = NO;
 					axp->vr = (axp->vr + 1) & MMASK;
 					tmp = len_p (axp->rxq) >= axp->window ? RNR : RR;
-					if (poll)
-						sendctl (axp, LAPB_RESPONSE, tmp | PF);
-					else
+					if (poll)	{
+						(void) sendctl (axp, LAPB_RESPONSE, tmp | PF);
+
+						if (tmp == RNR)
+							/* update the counters for rnr out */
+							axp->iface->axcnt.rnrout++;
+					} else
 						axp->response = (char) tmp;
 
 					procdata (axp, bp);
@@ -449,8 +473,13 @@ struct no_js *ep;	/*scan the 'exclude-calls' list */
 	 */
 	(void) lapb_output (axp);
 
-	if (axp->response != 0) {
-		sendctl (axp, LAPB_RESPONSE, axp->response);
+	if (axp && axp->response != 0) {
+		(void) sendctl (axp, LAPB_RESPONSE, axp->response);
+
+		if (axp->response == RNR)
+			/* update the counters for rnr out */
+			axp->iface->axcnt.rnrout++;
+
 		axp->response = 0;
 	}
 	return 0;
@@ -530,7 +559,7 @@ est_link (struct ax25_cb *axp)
 {
 	clr_ex (axp);
 	axp->retries = 0;
-	sendctl (axp, LAPB_COMMAND, SABM | PF);
+	(void) sendctl (axp, LAPB_COMMAND, SABM | PF);
 	stop_timer (&axp->t3);
 	start_timer (&axp->t1);
 }
@@ -556,7 +585,12 @@ enq_resp (struct ax25_cb *axp)
 char ctl;
 
 	ctl = len_p (axp->rxq) >= axp->window ? RNR | PF : RR | PF;
-	sendctl (axp, LAPB_RESPONSE, ctl);
+	(void) sendctl (axp, LAPB_RESPONSE, ctl);
+
+	if (ctl == (RNR | PF))
+		/* update the counters for rnr out */
+		axp->iface->axcnt.rnrout++;
+
 	axp->response = 0;
 	stop_timer (&axp->t3);
 }
@@ -580,6 +614,7 @@ sendctl (struct ax25_cb *axp, int cmdrsp, int cmd)
 {
 	if ((ftype ((char) cmd) & 0x3) == S)	/* Insert V(R) if S frame */
 		cmd |= (axp->vr << 5);		/*lint !e701 */
+
 	return sendframe (axp, cmdrsp, cmd, NULLBUF);
 }
 
@@ -597,7 +632,7 @@ char control;
 int sent = 0;
 int i;
 
-	if (axp == NULLAX25
+	if (axp == NULLAX25 || axp->iface == NULLIF
 	    || (axp->state != LAPB_RECOVERY && axp->state != LAPB_CONNECTED)
 	    || axp->flags.remotebusy)
 		return 0;
@@ -631,6 +666,10 @@ int i;
 			start_timer (&axp->t1);
 		}
 		sent++;
+
+		/* update the counters for data out */
+		axp->iface->axcnt.dataout++;
+
 		bp = bp->anext;
 		if (!axp->flags.rtt_run) {
 			/* Start round trip timer */
@@ -683,7 +722,15 @@ int seq;
 		if (pid == PID_SEGMENT && (seq & SEG_REM) == axp->segremain - 1) {
 			/* Correct, in-order segment */
 			append (&axp->rxasm, bp);
+
+			/* update the counters for segments in */
+			axp->iface->axcnt.segin++;
+
 			if ((axp->segremain = (seq & SEG_REM)) == 0) {
+
+				/* update the counters for segment errors */
+				axp->iface->axcnt.segerr++;
+
 				/* Done; kick it upstairs */
 				bp = axp->rxasm;
 				axp->rxasm = NULLBUF;
@@ -700,6 +747,9 @@ int seq;
 	} else {
 		/* No reassembly in progress */
 		if (pid == PID_SEGMENT) {
+			/* update the counters for segments in */
+			axp->iface->axcnt.segin++;
+
 			/* Start reassembly */
 			seq = PULLCHAR (&bp);		/*lint !e506 */
 			if (!(seq & SEG_FIRST))
@@ -720,9 +770,10 @@ int seq;
  * original packet if small enough
  */
 struct mbuf *
-segmenter (bp, ssize)
+segmenter (bp, ssize, axp)
 struct mbuf *bp;		/* Complete packet */
 int16 ssize;			/* Max size of frame segments */
+struct ax25_cb *axp;
 {
 struct mbuf *result = NULLBUF;
 struct mbuf *bptmp, *bp1;
@@ -740,6 +791,10 @@ int segments;
 	segments = 1 + (len - 1) / ssize;	/* # segments  */
 	offset = 0;
 
+	/* update the counters for seqments out */
+	if (axp && axp->iface)
+		axp->iface->axcnt.segout += segments;
+
 	while (segments != 0) {
 		offset += dup_p (&bptmp, bp, offset, ssize);
 		if (bptmp == NULLBUF) {
@@ -773,43 +828,12 @@ struct axlink *ipp;
 		if (ipp->pid == pid)
 			break;
 	}
-#ifdef AXNEGOTIATION
-	if (AXNegotiation && pid == PID_NO_L3 && !(axp->remote[AXALEN - 1] & (AXN | Q)))
-		ax_negotiate (axp, bp);
+	if (ipp->funct != NULL)
+		(*ipp->funct) (axp->iface, axp, NULLCHAR, NULLCHAR, bp, 0);
 	else
-#endif
-	{
-		if (ipp->funct != NULL)
-			(*ipp->funct) (axp->iface, axp, NULLCHAR, NULLCHAR, bp, 0);
-		else
-			free_p (bp);
-	}
+		free_p (bp);
 }
 
 
 
-#ifdef AXNEGOTIATION
-void
-ax_negotiate (struct ax25_cb *axp, struct mbuf *bp)
-{
-unsigned char type, negdata[256];
-int size;
-
-	if (!(pullup (&bp, &type, 1)))
-		return;
-	if ((size = pullup (&bp, negdata, sizeof (negdata))) == 0)
-		return;
-	switch (type) {
-		case QBIT_NREQ:
-			AXNegotiation_request (axp, negdata, size);
-			break;
-		case QBIT_NREPLY:
-			AXNegotiation_reply (axp, negdata, size);
-			break;
-		default:
-			break;	/* ignore */
-	}
-}
-#endif /* AXNEGOTIATION */
-
 #endif /* AX25 */
