@@ -54,7 +54,7 @@
 #include "help.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: mailbox2.c,v 1.30 1997/06/28 16:46:13 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: mailbox2.c,v 1.32 1997/08/19 01:19:22 root Exp root $";
 #endif
 
 #ifdef DELEGATE
@@ -63,15 +63,18 @@ extern int Mbdelegate;
 #endif
 
 #ifdef UNIX
-extern int chown (const char *path, uid_t owner, gid_t group);
 extern int ftpsecuritycheck (char *filename, int perms, int mode);
 extern long CREATEgid, CREATEuid;
 extern short CREATEmask;
 extern int CREATEsecure;
 
+#ifndef R_OK
 #define R_OK 4
+#endif
+#ifndef W_OK
 #define W_OK 2
 #endif
+#endif
 
 extern int AMPRonly;
 extern char Compiled[];
@@ -117,7 +120,8 @@ static int dombgetall (struct mbx *m);
 static void expertstat (struct mbx *m);
 
 #ifdef FILECMDS
-static int uuencode (FILE * infile, int s, char *infilename);
+static int uuencode (FILE *infile, int s, char *infilename);
+static int uudecode (FILE *infile, FILE *outfile);
 #endif
 
 int dombencode (int argc, char *argv[], void *p);
@@ -130,7 +134,6 @@ int dombnews (int argc, char *argv[], void *p);
 int dombtutor (int argc, char *argv[], void *p);
 int dombgroup (int argc, char *argv[], void *p);
 int dombquote (int argc, char *argv[], void *p);
-void quickscan (struct mbx *m, int olduser);
 int dombset (int argc, char *argv[], void *p);
 int dombrmail (int argc, char *argv[], void *p);
 int dombroute (int argc, char *argv[], void *p);
@@ -311,9 +314,9 @@ static const char delegationdisabled[] = "Forwarding delegation disabled by SYSO
 static const char expandsto[] = "Address '%s' expands to: ";
 static const char noaddress[] = "No address specified.\n";
 static const char addressroutes[] = "Address '%s' routes to:";
-static const char unreadmessages[] = "You have unread messages, in the following message areas:\n";
+static const char unreadmessages[] = "%s ha%s unread messages, in the following message areas:\n";
 static const char newmessages[] = "As a new user, you will find messages in the following message areas:\n";
-static const char nounreadmessages[] = "No unread messages in the public areas! You are up-to-date!\n";
+static const char nounreadmessages[] = "No unread messages in the public areas! %s %s up-to-date!\n";
 
 #if 0
 static const char systemtime[] = "The system time at '%s' is now: %s\n";
@@ -396,7 +399,7 @@ unsigned attr;
 		return 0;
 	}
 #ifdef FILECMDS
-	if ((file = permtest (m->path, m->privs, (argc > 1) ? argv[1] : ".", RETR_CMD, NULLCHAR, (argc < 2))) != NULLCHAR) {	/*lint !e730 */
+	if ((file = permtest (m->path, m->privs, (argc > 1) ? argv[1] : ".", RETR_CMD, NULLCHAR, (argc < 2))) != NULLCHAR) {
 		m->state = MBX_WHAT;
 
 		/* check for index.dat file in that directory */
@@ -423,7 +426,7 @@ unsigned attr;
 			}
 		}
 		if ((fp = dir (file, (in == NULLFILE) ? 1 : 2)) == NULLFILE)
-			tprintf (noread, file, sys_errlist[errno]);
+			tprintf (noread, file, SYS_ERRLIST(errno));
 		else {
 #ifdef MBXTDISC
 			stop_timer (&m->tdisc);
@@ -459,9 +462,9 @@ unsigned attr;
 						}
 					}
 				}
-				fclose (in);
+				(void) fclose (in);
 			}
-			fclose (fp);
+			(void) fclose (fp);
 		}
 		free (file);
 	} else {
@@ -484,7 +487,7 @@ char *file;
 	m = (struct mbx *) p;
 	if ((file = permtest (m->path, m->privs, argv[1], DELE_CMD, NULLCHAR, 0)) != NULLCHAR) {
 		if (unlink (file))
-			tprintf (zapfailed, sys_errlist[errno]);
+			tprintf (zapfailed, SYS_ERRLIST(errno));
 		else
 			log (m->user, logzap, file);
 		free (file);
@@ -575,7 +578,7 @@ struct mbx *m;
 		newargv[0] = buf2;
 		newargv[1] = Hostname;
 		if (MAttendedAt != NULLCHAR) {
-			strcpy (doitat, MAttendedAt);
+			strncpy (doitat, MAttendedAt, 50);
 			newargv[1] = doitat;
 			rip (newargv[1]);
 		}
@@ -659,7 +662,7 @@ char *cp;
 			continue;
 
 		cp = _variable_expansion (strdup (m->line));
-		strcpy (m->line, cp);
+		strncpy (m->line, cp, MBXLINE);
 		free (cp);
 		/* Check for special commands. Execute these mailbox cmds. KF5MG. */
 #if defined AX25 || defined NETROM
@@ -726,7 +729,7 @@ char *cp;
 #ifdef GWTRACE
 		log (m->user, loglink, m->name, argv[3]);
 #endif
-		strcpy (m->name, argv[3]);
+		strncpy (m->name, argv[3], MBXLINE);
 		oldprivs = m->privs;	/*Save this !*/
 		/* Try to find the privileges of this user from the userfile */
 		if ((m->privs = userlogin (m->name, NULLCHAR, &m->path, MBXLINE,
@@ -1034,7 +1037,7 @@ struct mbx *m;
 also:
 		if ((fp = fopen (buf, READ_TEXT)) != NULLFILE) {
 			(void) sendfile (fp, Curproc->output, ASCII_TYPE, 0);
-			fclose (fp);
+			(void) fclose (fp);
 		} else {
 			if (argc > 1 && *argv[0] == 'h')
 				tputs (nohelp);
@@ -1132,7 +1135,7 @@ int msg, maxmsg;
 		pp = permtest (m->path, m->privs, to, STOR_CMD, NULLCHAR, 0);
 		if (!pp)
 			return 0;
-		strcpy (buf, pp);
+		strncpy (buf, pp, MBXLINE);
 		free (pp);
 	} else
 		sprintf (buf, "%s/%s.txt", Mailspool, to);
@@ -1185,7 +1188,7 @@ int msg, maxmsg;
 		}
 		tprintf (message, thisone, (m->stype == 'M') ? "moved" : (m->stype == 'W') ? "written" : "copied");
 	}
-	fclose (Mfile);
+	(void) fclose (Mfile);
 	if (m->stype != 'W') {
 #ifdef ALERTMONITOR
 		alertarea (to);
@@ -1445,7 +1448,7 @@ int loopindex;
 			free (newaddr);
 			newaddr = NULLCHAR;
 		} else {
-			strcpy (buf, newaddr);
+			strncpy (buf, newaddr, LINELEN);
 			arg = buf;
 		}
 	list = NULLLIST;
@@ -1586,15 +1589,15 @@ static int
 quickstat (register struct mbx *m, char *area)
 {
 long last, newlast, lastone;
-char oldarea[20], buf[256];
+char oldarea[64], buf[256];
 FILE *fp;
 register int retval = 0;
 
 	last = m->lastread;
 	newlast = m->newlastread;
 	m->lastread = 0;
-	strcpy (oldarea, m->area);
-	strcpy (m->area, area);
+	strncpy (oldarea, m->area, 64);
+	strncpy (m->area, area, 64);
 	(void) strlwr (m->area);
 	(void) nntp_name_expansion (m->area);
 	/* only read last read message-id if current area is a public area
@@ -1617,11 +1620,11 @@ register int retval = 0;
 		fseek (fp, size, SEEK_SET);
 		(void) fread (&lt, sizeof (struct let), 1, fp);
 
-		fclose (fp);
+		(void) fclose (fp);
 		if (lt.bid > lastone)
 			retval = 1;
 	}
-	strcpy (m->area, oldarea);
+	strncpy (m->area, oldarea, 64);
 	kwait (NULL);
 	return retval;
 }
@@ -1629,7 +1632,7 @@ register int retval = 0;
 
 
 void
-quickscan (struct mbx *m, int olduser)
+quickscan (struct mbx *m, int olduser, int usename)
 {
 char const *cp;
 char *cp2;
@@ -1652,13 +1655,13 @@ int found = 0, length = 3;
 	while (fgets (buf, MBXLINE, fp) != NULLCHAR) {
 		kwait (NULL);
 		if (isalnum (buf[0])) {	/* skip comments */
-			if ((cp2 = strpbrk (buf, " \t")) != NULLCHAR)
+			if ((cp2 = strpbrk (buf, " \t\n\r")) != NULLCHAR)
 				*cp2 = '\0';
 			/*			statarea (m, buf, &total, &new, &hold, 0);
 			if (new)	{		*/
 			if (quickstat (m, buf)) {
 				if (!found) {
-					tputs (initmsg);
+					tprintf (initmsg, (usename) ? m->name : "You", (usename) ? "s" : "ve");
 					found++;
 				}
 				length += (int) (strlen (buf) + 2);
@@ -1677,9 +1680,9 @@ int found = 0, length = 3;
 		tputs (out);
 		tputc ('\n');
 	}
-	fclose (fp);
+	(void) fclose (fp);
 	if (!found)
-		tputs (nounreadmessages);
+		tprintf (nounreadmessages, (usename) ? m->name : "You", (usename) ? "is" : "are");
 }
 #endif
 
@@ -1758,7 +1761,7 @@ char buf[80];
 			break;
 		}
 	}
-	fclose (old);
+	(void) fclose (old);
 	return retval;
 }
 
@@ -1785,7 +1788,7 @@ unsigned len;
 	newname = (char *) mallocw (strlen (Group) + 5);
 	sprintf (newname, "%s.new", Group);
 	if ((new = fopen (newname, CREATE_TEXT)) == NULLFILE) {
-		fclose (old);
+		(void) fclose (old);
 		free (newname);
 		return 0;
 	}
@@ -1873,16 +1876,16 @@ unsigned len;
 	}
 	while (fgets (buf, 50, old) != NULLCHAR)
 		fputs (buf, new);
-	fclose (new);
-	fclose (old);
+	(void) fclose (new);
+	(void) fclose (old);
 	unlink (Group);
 	(void) rename (newname, Group);
 	free (newname);
 	return 1;
 
 confuzed:
-	fclose (new);
-	fclose (old);
+	(void) fclose (new);
+	(void) fclose (old);
 	unlink (newname);
 	free (newname);
 	return retval;
@@ -2136,7 +2139,7 @@ long offset;
 
 	/* write the old or new entry, with the new timestamp and close file */
 	fprintf (fp, "%-*s %-14ld\n", thesize, entry, now);
-	fclose (fp);
+	(void) fclose (fp);
 	kwait (NULL);
 
 	/* If we were updating the entry and the entry DID change..... */
@@ -2160,7 +2163,7 @@ long offset;
 				} else
 					fp = fopentmp (WhitePages, APPEND_TEXT);
 				fprintf (fp, "%-*s %-14ld\n", thesize, buf, now);
-				fclose (fp);
+				(void) fclose (fp);
 				kwait (NULL);
 			}
 			/* record it in the daily WP Update temp file */
@@ -2211,7 +2214,7 @@ int skipupdate = 0;
 					break;
 				}
 			}
-			fclose (fp);
+			(void) fclose (fp);
 		}
 	}
 	if (result != -1) {	/* found it */
@@ -2224,7 +2227,7 @@ int skipupdate = 0;
 			if (fp != NULLFILE)	{
 				fseek (fp, (long) (long) result * (long) entrysize, 0);
 				wpageUpdate (fp, string, buf, &buf[(bbs) ? 33 : 14], bbs, updateit, (which != 'G') ? which : 'X');
-				/* wpageUpdate does the fclose(fp); */
+				/* wpageUpdate does the (void) fclose(fp); */
 			}
 		}
 	}
@@ -2257,7 +2260,7 @@ char *last, buf[14];
 		if ((fp = fopennew ((bbs) ? WhitePagesBBS : WhitePages, APPEND_TEXT)) != NULLFILE) {
 			fprintf (fp, "%-*s %-14ld\n", (bbs) ? 32 : 13, entry, now);	/* Save h_addr in whitepages file */
 			kwait (NULL);
-			fclose (fp);
+			(void) fclose (fp);
 		}
 		if (bbs)
 			add_WPUpdate (entry, entry, NULLCHAR, which);
@@ -2347,12 +2350,12 @@ char name[14];
 	tputc ('\n');
 	while (mbxrecvline (m) != -1) {
 		if (!strnicmp (m->line, "/ex", 3)) {
-			fclose (fp);
+			(void) fclose (fp);
 			return (0);
 		}
 		fprintf (fp, " %-37.37s\n", skipwhite (m->line));
 	}
-	fclose (fp);
+	(void) fclose (fp);
 	return (-2);		/* socket closed here! */
 }
 
@@ -2403,21 +2406,19 @@ char *file;
 	}
 #endif
 #endif
-	if ((fp = fopen (file, READ_TEXT)) == NULLFILE)
-		tprintf (cannotopen, file, sys_errlist[errno]);
+	if ((fp = fopen (file, (m->stype == 'U') ? READ_BINARY : READ_TEXT)) == NULLFILE)
+		tprintf (cannotopen, file, SYS_ERRLIST(errno));
 	else {
 #ifdef MBXTDISC
 		/* disable the mbox inactivity timeout timer - WG7J */
 		stop_timer (&m->tdisc);
 #endif
-		if (m->stype == 'U') {	/* uuencode ? */
-			fclose (fp);
-			fp = fopen (file, READ_BINARY);	/* assume non-ascii */
+		if (m->stype == 'U') 	/* uuencode ? */
 			(void) uuencode (fp, m->user, file);
-		} else
+		else
 			(void) sendfile (fp, m->user, ASCII_TYPE, 0);
 		if (fp != NULLFILE)
-			fclose (fp);
+			(void) fclose (fp);
 	}
 	free (file);
 	return 0;
@@ -2430,8 +2431,11 @@ int
 dombupload (int argc, char *argv[], void *p)
 {
 struct mbx *m;
-FILE *fp;
+FILE *fp, *uufp = NULLFILE;
 char *file;
+#ifdef UNIX
+int mode = 0x7fff;
+#endif
 
 	m = (struct mbx *) p;
 	if (m->stype == 'S') {
@@ -2479,11 +2483,17 @@ char *file;
 	}
 #endif
 
-	if ((fp = fopen (file, WRITE_TEXT)) == NULLFILE) {
-		tprintf (cannotcreate, file, sys_errlist[errno]);
+	if ((fp = fopen (file, (m->stype == 'U') ? WRITE_BINARY : WRITE_TEXT)) == NULLFILE) {
+		tprintf (cannotcreate, file, SYS_ERRLIST(errno));
 		free (file);
 		return 0;
 	}
+
+	if (m->stype == 'U')	{
+		uufp = fp;
+		fp = tmpfile ();
+	}
+
 	log (m->user, logupload, file);
 	m->state = MBX_UPLOAD;
 	tprintf ("Send %s,  %s", (m->stype == 'F') ? "personal information" : (m->stype == 'S') ? "signature information" : (m->stype == 'M') ? "personal message" : "file", Howtoend);
@@ -2511,7 +2521,17 @@ char *file;
 		if (putc ('\n', fp) == EOF)
 			break;
 	}
-	fclose (fp);
+	if (m->stype == 'U')	{
+#ifdef UNIX
+		mode =
+#else
+		(void)
+#endif
+			uudecode (fp, uufp);
+		(void) fclose (uufp);
+	}
+
+	(void) fclose (fp);
 
 #if defined(FILECMDS) && defined(TIPMAIL) && defined(XMODEM)
 secureit:
@@ -2520,8 +2540,9 @@ secureit:
 #ifdef UNIX
 	if (CREATEsecure) {
 		(void) chown (file, (uid_t) CREATEuid, (gid_t) CREATEgid);
-		(void) chmod (file, (mode_t) CREATEmask);
-	}
+		(void) chmod (file, (mode_t) (mode & CREATEmask));
+	} else if (m->stype == 'U')
+		(void) chmod (file, (mode_t) mode);	
 #endif
 	free (file);
 	return 0;
@@ -2591,11 +2612,8 @@ struct mbx *m;
 */
 
 static int
-uuencode (
-FILE *infile,
-int s,				/* output socket */
-char *infilename
-) {
+uuencode (FILE *infile, int s, char *infilename)
+{
 int n_read_so_far = 0, n_written_so_far = 0, in_chars, n, mode = 0755;
 unsigned long cnt = 0;
 unsigned char in[3], out[4], line[100];
@@ -2641,5 +2659,54 @@ struct stat stb;
 	return 0;
 }
 
+
+int
+uudecode (FILE *infile, FILE *outfile)
+{
+char line[100];
+int header = 0;
+int n_to_write, n_to_read, n_reads, n, m;
+unsigned char b[4], out[3];
+int mode = 0x7fff;
+
+	rewind (infile);
+	
+	/* skip till the (begin) line is eaten */
+	while (!header)	{
+		(void) fgets(line, 100, infile);
+		if (strstr (line, "begin"))	{
+			header = 1;
+			sscanf (line, "begin %03o", &mode);
+		}
+	}			
+
+	/* decode the remainder of the file */
+	while ((void) fgets (line, 100, infile), strncmp (line, "end", 3))	{
+		n_to_write = (int)line[0] - 32;
+		n_to_read = ((n_to_write + 2) / 3) * 4;
+		if (strlen ((char *)line) - 2 < (unsigned int) n_to_read)	/* EOF apparently */
+			n_to_read = 0;
+		n_reads = n_to_read / 4;
+		for (n = 0; n < n_reads; n++)	{
+			for (m = 0; m < 4; m++)
+				b[m] = (unsigned char) line[1 + 4 * n + m] - 32;
+
+			/* now do the actual decode */
+			out[0] = (unsigned char) (b[0] << 2) | ((b[1] & 0x30) >> 4);
+			out[1] = ((b[1] & 0x0f) << 4) | ((b[2] & 0x3c) >> 2);
+			out[2] = ((b[2] & 0x03) << 6) | (b[3]);
+			if (n_to_write > 3)
+				for (m = 0; m < 3; m++)
+					fputc(out[m], outfile);
+			else
+				for (m = 0; m < n_to_write; m++)
+					fputc (out[m], outfile);
+			n_to_write -= m;
+		}
+	}
+	return (mode);
+}
+
+
 #endif
 #endif /* MAILBOX */
