@@ -48,7 +48,7 @@
 #ifdef SQL
 #include "sql.h"
 #endif
-#if defined(CATALOG) && !defined(BETA)
+#ifdef CATALOG
 #include "catalog.h"
 #endif
 #include "asy.h"
@@ -60,9 +60,8 @@
 #include "slip.h"
 #include "nrs.h"
 
-
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: main.c,v 1.51 1997/06/28 16:46:13 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: main.c,v 1.55 1997/09/14 14:37:46 root Exp root $";
 #endif
 
 
@@ -79,9 +78,11 @@ extern int32 LastIO;
 void reset_all_ax25 (void);
 #endif
 
-static char symname[] = "$Name: Release2_22 $";
+static char symname[] = "$Name: Release2_30 $";
 
 extern int Warnings;
+static int CrashProtect = 0;
+
 
 #ifdef LOG_GMT_TZ
 #define TZTIME(x) asctime(gmtime(x))
@@ -133,7 +134,6 @@ static char MPRUNNINGerr[] = "\007*** Cannot resolve hostnames in the startup fi
 
 #ifdef UNIX
 static char LockName[] = "tnos.lock";
-extern unsigned int sleep (unsigned int seconds);
 extern char const *LOCKDIR;
 #endif
 
@@ -248,6 +248,7 @@ extern void statLineToggle (int clearmarquee);
 extern void statlog (const char *buf);
 static void logcmd (char *);
 static void freehistory (void);
+static int _dowriteall (char *str);
 
 void variable_expansion (struct mbuf ** bp);
 
@@ -657,7 +658,13 @@ int32 cmdcounter = 0;
 #ifdef RLINE
 	ReadFwdBbs ();
 #endif
-#if defined(CATALOG) && !defined(BETA)
+#ifdef CATALOG
+	/* this serves two purposes. First, it will error if file not found
+	   NOW rather than later. Second, it makes the OWNER of the open file
+	   be the Command Session. If this wasn't done (and a user process
+	   opened the file later) then the file is closed when the process
+	   dies, which confuses TNOS totally!
+	 */
 	switch (catalog_check ()) {
 		case -1:
 			tputs ("\nImproperly formatted etc/catalog.cat file!!\nIt looks corrupted. Unless you fix this, a crash is probable!\n\n\007");
@@ -751,7 +758,7 @@ int32 cmdcounter = 0;
 		intmp = mallocw (BUFSIZ);
 		while (fgets (inbuff, BUFSIZ, fp) != NULLCHAR) {
 			kwait (NULL);
-			strcpy (intmp, inbuff);
+			strncpy (intmp, inbuff, BUFSIZ);
 			if (Verbose == 1) {
 				tprintf ("%s", intmp);
 				tflush ();
@@ -777,7 +784,7 @@ int32 cmdcounter = 0;
 			if (Mprunning_err)
 				tprintf (MPRUNNINGerr, intmp);
 		}
-		fclose (fp);
+		(void) fclose (fp);
 		free (inbuff);
 		free (intmp);
 	}
@@ -875,8 +882,10 @@ int got = 0;
 			Current = Command;
 			swapscreen (Lastcurr, Current);
 
+#if 0
 			/* set 'more' paging on command screen */
 			Command->flowmode = 1;
+#endif
 		}
 		if (c < -2) {	/* F1 to F9 (or INSERT) pressed */
 			switch (c) {
@@ -1113,7 +1122,7 @@ unsigned attr;
 #endif
 
 	for (i = 1; i < argc; i++) {
-		strcpy (fname, make_fname (Command->curdirs->dir, argv[i]));
+		strncpy (fname, make_fname (Command->curdirs->dir, argv[i]), 128);
 #ifdef DOS_GETFILEATTR
 		if (!_dos_getfileattr (fname, &attr) && (attr & FA_DIREC)) {
 			strcat (fname, "/*.*");
@@ -1148,8 +1157,8 @@ dorename (int argc OPTIONAL, char *argv[], void *p OPTIONAL)
 char fname1[128];
 char fname2[128];
 
-	strcpy (fname1, make_fname (Command->curdirs->dir, argv[1]));
-	strcpy (fname2, make_fname (Command->curdirs->dir, argv[2]));
+	strncpy (fname1, make_fname (Command->curdirs->dir, argv[1]), 128);
+	strncpy (fname2, make_fname (Command->curdirs->dir, argv[2]), 128);
 	if (rename (fname1, fname2) == -1)
 		tprintf ("Can't rename: %s\n", sys_errlist[errno]);
 	return 0;
@@ -1171,7 +1180,7 @@ unsigned attr;
 #endif
 
 	if (argc > 2) {
-		strcpy (fname2, make_fname (Command->curdirs->dir, argv[2]));
+		strncpy (fname2, make_fname (Command->curdirs->dir, argv[2]), 128);
 #ifdef DOS_GETFILEATTR
 		(void) _dos_getfileattr (fname2, &attr);
 		if (attr & FA_DIREC) {
@@ -1181,7 +1190,7 @@ unsigned attr;
 #endif
 	} else
 		usedir = 1;
-	strcpy (fname, make_fname (Command->curdirs->dir, argv[1]));
+	strncpy (fname, make_fname (Command->curdirs->dir, argv[1]), 128);
 #ifdef DOS_GETFILEATTR
 	if (!_dos_getfileattr (fname, &attr) && (attr & FA_DIREC)) {
 		strcat (fname, "/*.*");
@@ -1202,7 +1211,7 @@ unsigned attr;
 		sprintf (fullname, "%s%s%s", (dirnam) ? dirnam : "", (dirnam) ? "/" : "", (usedir) ? filenm : fname2);
 		if ((new = fopen (fullname, WRITE_BINARY)) == NULL) {
 			tprintf ("Can't open %s: %s\n", fullname, sys_errlist[errno]);
-			fclose (old);
+			(void) fclose (old);
 			continue;
 		}
 		(void) strlwr (fullname);
@@ -1217,8 +1226,8 @@ unsigned attr;
 			if (!(++count))	/* be polite to other users */
 				kwait (NULL);
 		}
-		fclose (old);
-		fclose (new);
+		(void) fclose (old);
+		(void) fclose (new);
 		if (!usedir)
 			break;
 	}
@@ -1232,7 +1241,7 @@ unsigned attr;
 /*this is also called from the remote-server for the 'exit' command - WG7J*/
 void
 where_outta_here (resetme, where)
-int resetme;			/* resetme is 0=exit, 1=low memory, 2=reset (if possible), 3=free lockup error */
+int resetme;			/* resetme is 0=exit, 1=low memory, 2=reset (if possible), 3=free lockup error, 5=proc loop error */
 char const *where;
 {
 time_t StopTime;
@@ -1240,6 +1249,7 @@ FILE *fp;
 char *inbuff, *intmp;
 char *bptr;
 
+	tcmdprintf ("*** Exiting TNOS...\n");
 	if (NextCmd)
 		NextCmd = 1;
 #if 0
@@ -1255,7 +1265,7 @@ char *bptr;
 		inbuff = mallocw (BUFSIZ);
 		intmp = mallocw (BUFSIZ);
 		while (fgets (inbuff, BUFSIZ, fp) != NULLCHAR) {
-			strcpy (intmp, inbuff);
+			strncpy (intmp, inbuff, BUFSIZ);
 			if (Verbose) {
 				tprintf ("%s", intmp);
 				rflush ();
@@ -1267,7 +1277,7 @@ char *bptr;
 			}
 			free (bptr);
 		}
-		fclose (fp);
+		(void) fclose (fp);
 		free (inbuff);
 		free (intmp);
 	}
@@ -1277,7 +1287,7 @@ char *bptr;
 #ifdef MBFWD
 	FWDmode = 0;		/* disable future forwarding sessions */
 #endif
-	if (resetme != 3)	{
+	if (resetme != 3 && resetme != 5)	{
 		reset_all_ax25 ();
 		kpause (2000);
 	}
@@ -1293,17 +1303,13 @@ char *bptr;
 #if defined(MSDOS)
 	DOSterm ();
 #endif
-#if 0
-	log (-1, "TNOS was stopped");
-#else
 	log (-1, "TNOS was stopped from '%s' with value %d", where, resetme);
-#endif
 #ifdef STATS
 	if (!resetme)
 		(void) doSTATsave (0, (char **) 0, (void *) 0);
 #endif
 	if (Logfp) {
-		fclose (Logfp);
+		(void) fclose (Logfp);
 		Logfp = NULLFILE;
 	}
 #ifdef UNIX
@@ -1352,7 +1358,8 @@ struct mbuf *bp;
 time_t nowtime, elapsedtime;
 unsigned int days, hrs, mins, secs;
 
-	if (strnicmp (Curproc->name, "at ", 3) == 0)
+	if (strnicmp (Curproc->name, "at ", 3) == 0 ||
+	    strnicmp (Curproc->name, "Cron ", 5) == 0)
 		where_outta_here (0, "doexit");
 
 	if (Curproc->input != Command->input)
@@ -1453,25 +1460,25 @@ char *cp;
 int
 dolog (int argc, char *argv[], void *p OPTIONAL)
 {
-static char *logname;
+static char *logname = NULLCHAR;
 char fname[128];
 
 	if (argc < 2) {
-		if (Logfp)
-			tprintf ("Logging to %s\n", logname);	/*lint !e727 */
+		if (Logfp && logname != NULLCHAR)
+			tprintf ("Logging to %s\n", logname);	/* lint !e727 */
 		else
 			tputs ("Logging off\n");
 		return 0;
 	}
 	if (Logfp) {
 		log (-1, "TNOS log closed");
-		fclose (Logfp);
+		(void) fclose (Logfp);
 		Logfp = NULLFILE;
 		free (logname);
 		logname = NULLCHAR;
 	}
 	if (strcmp (argv[1], "stop") && strcmp (argv[1], "off")) {
-		strcpy (fname, make_fname (Command->curdirs->dir, argv[1]));
+		strncpy (fname, make_fname (Command->curdirs->dir, argv[1]), 128);
 		logname = strdup (fname);
 		Logfp = fopen (logname, APPEND_TEXT);
 		log (-1, "TNOS v%s was started at %s", Version, TZTIME (&StartTime));
@@ -1588,7 +1595,11 @@ doescape (int argc, char *argv[], void *p OPTIONAL)
 
 
 /* Generate system command packet. Synopsis:
- * remote [-p port#] [-k key] [-a hostname] <hostname> reset|exit|kickme
+ * remote [-p <port#>] [-a <kickaddress>] <hostname> kickme
+ * remote [-p <port#>] -k <key> <hostname> reset|exit
+ * remote [-p <port#>] -k <key> -r <destIPaddr>[/<bits>] <hostname> add|drop
+ * remote -s <password>
+ * remote -g <gatewaypassword>
  */
 int
 doremote (int argc, char *argv[], void *p OPTIONAL)
@@ -1601,10 +1612,11 @@ char *key = NULLCHAR;
 int klen = 0;
 uint32 addr = 0;
 char *cmd, *host;
+char *route = NULLCHAR;
 
 	port = IPPORT_REMOTE;	/* Set default */
 	optind = 0;		/* reinit getopt() */
-	while ((c = getopt (argc, argv, "a:p:k:s:")) != EOF) {
+	while ((c = getopt (argc, argv, "a:r:g:p:k:s:x:")) != EOF) {
 		switch (c) {
 			case 'a':
 				if ((addr = resolve (optarg)) == 0) {
@@ -1622,6 +1634,19 @@ char *cmd, *host;
 			case 's':
 				Rempass = strdup (optarg);
 				return 0;	/* Only set local password */
+#ifdef ENCAP
+			case 'r':
+				route = optarg;
+				break;
+			case 'g':
+				RemRTpass = strdup (optarg);
+				return 0;	/* Only set local password for adding routes */
+#else
+			case 'r':
+			case 'g':
+				tputs ("This option is not available to you. TNOS was NOT compiled\nwith support for the ENCAP features!\n");
+				/* and fall through */
+#endif
 			default:
 				return 0;
 		}
@@ -1641,9 +1666,12 @@ char *cmd, *host;
 	if (addr != 0 && cmd[0] == 'k')
 		len += sizeof (int32);
 
-	if (key != NULLCHAR && (cmd[0] == 'r' || cmd[0] == 'e'))
+	if (key != NULLCHAR && cmd[0] != 'k')
 		len += (int16) klen;
 
+	if (route != NULLCHAR)
+		len += (int16) (strlen (route) + 1);
+
 	if (len == 1)
 		data = &x;
 	else
@@ -1658,12 +1686,8 @@ char *cmd, *host;
 
 	switch (cmd[0]) {
 		case 'r':
-			data[0] = SYS_RESET;
-			if (key != NULLCHAR)
-				strncpy (&data[1], key, (unsigned) klen);	/*lint !e661 */
-			break;
 		case 'e':
-			data[0] = SYS_EXIT;
+			data[0] = (cmd[0] == 'r') ? SYS_RESET : SYS_EXIT;
 			if (key != NULLCHAR)
 				strncpy (&data[1], key, (unsigned) klen);	/*lint !e661 */
 			break;
@@ -1672,16 +1696,28 @@ char *cmd, *host;
 			if (addr != 0)
 				(void) put32 ((unsigned char *) &data[1], addr);	/*lint !e661 */
 			break;
+		case 'a':	/* add a route */
+		case 'd':	/* drop a route */
+			if (key == NULLCHAR || route == NULLCHAR)	{
+				tprintf ("Command '%s' requires a password and a route entry!\n", cmd);
+				goto cleanup;
+			}
+			data[0] = (cmd[0] == 'a') ? ROUTE_ADD : ROUTE_DROP;
+			data[1] = (char) strlen(route);		/*lint !e661 */
+			strcpy (&data[2], route);		/*lint !e661 !e662 */
+			strncpy (&data[2 + strlen(route)], key, (unsigned) klen);	/*lint !e661 */
+			break;
 		default:
 			tprintf ("Unknown command %s\n", cmd);
 			goto cleanup;
 	}
+
 	/* Form the command packet and send it */
 	if (sendto (s, data, len, 0, (char *) &fsock, sizeof (fsock)) == -1) {
 		tprintf ("sendto failed: %s\n", sys_errlist[errno]);
 		goto cleanup;
 	}
-      cleanup:
+cleanup:
 	if (data != &x)
 		free (data);
 	close_s (s);
@@ -1715,7 +1751,7 @@ int usesession = 0;
 		row = SCREENlength;
 #endif
 	}
-	strcpy (fname, make_fname (Command->curdirs->dir, argv[1]));
+	strncpy (fname, make_fname (Command->curdirs->dir, argv[1]), 256);
 	if ((fp = fopen (fname, READ_TEXT)) == NULLFILE) {
 		tprintf (NoRead, fname, sys_errlist[errno]);
 		if (usesession) {
@@ -1808,8 +1844,8 @@ register int numlines = 18;
 	}
 	buffer = callocw (rdsize, sizeof (char));
 
-	strcpy (fname, make_fname (Command->curdirs->dir, argv[1]));
-	if ((handle = open (fname, O_BINARY | O_RDONLY)) == -1) {	/*lint !e718 !e746 */
+	strncpy (fname, make_fname (Command->curdirs->dir, argv[1]), 128);
+	if ((handle = open (fname, O_BINARY | O_RDONLY)) == -1) {	/* lint !e718 !e746 */
 		tprintf (NoRead, fname, sys_errlist[errno]);
 		free (buffer);
 		return -1;
@@ -1946,6 +1982,16 @@ doerror (int argc, char *argv[], void *p OPTIONAL)
 
 
 
+#ifdef BETA
+int
+docrashprot (int argc, char *argv[], void *p OPTIONAL)
+{
+	return setbool (&CrashProtect, "Enable crash protection mode", argc, argv);
+}
+#endif
+
+
+
 /* Mail a system message to the sysop - WG7J */
 void
 mail_error (const char *fmt,...)
@@ -1975,7 +2021,7 @@ time_t t;
 	rewind (txt);
 	(void) rdaemon (txt, NULLCHAR, NULLCHAR, "syserror", SysMessage, 'P', 0);
 
-	fclose (txt);
+	(void) fclose (txt);
 
 	/* Now kick the smtp server */
 	smtptick (NULL);
@@ -2079,7 +2125,7 @@ FILE *fp;
 char fname[128];
 
 	/* Read command source file */
-	strcpy (fname, make_fname (Command->curdirs->dir, argv[1]));
+	strncpy (fname, make_fname (Command->curdirs->dir, argv[1]), 128);
 	if ((fp = fopen (fname, READ_TEXT)) == NULLFILE) {
 		tprintf (NoRead, fname, sys_errlist[errno]);
 		return 1;
@@ -2087,7 +2133,7 @@ char fname[128];
 	inbuff = (char *) mallocw (BUFSIZ);
 	intmp = (char *) mallocw (BUFSIZ);
 	while (fgets (inbuff, BUFSIZ, fp) != NULLCHAR) {
-		strcpy (intmp, inbuff);
+		strncpy (intmp, inbuff, BUFSIZ);
 		linenum++;
 		if (Verbose)
 			tprintf ("%s", intmp);
@@ -2108,7 +2154,7 @@ char fname[128];
 		Curproc->gotolabel = NULLCHAR;
 	}
 #endif
-	fclose (fp);
+	(void) fclose (fp);
 	free (inbuff);
 	free (intmp);
 	Mprunning_err = 0;
@@ -2249,28 +2295,71 @@ struct mbx *m = NULLMBX;
 }
 
 
+
+int
+doshutdown (int argc OPTIONAL, char *argv[], void *p OPTIONAL)
+{
+int32 delayval = 0;
+char *str = NULLCHAR;
+char buffer[1024];
+
+	str = argv[2];
+
+	if (strcasecmp (argv[1], "now"))
+		delayval = atol (argv[1]);
+
+	if (delayval)	{
+		sprintf (buffer, "System being shutdown in %ld seconds%s%s",
+			delayval, (str != NULLCHAR) ? "\n*** " : "",
+			(str != NULLCHAR) ? str : "");
+		tcmdprintf (FromSysop, buffer);
+		(void) _dowriteall (buffer);
+		kpause (delayval * 1000L);
+	}
+
+	sprintf (buffer, "System now being shutdown%s%s",
+		(str != NULLCHAR) ? "\n*** " : "",
+		(str != NULLCHAR) ? str : "");
+	tcmdprintf ("\n");
+	tcmdprintf (FromSysop, buffer);
+	(void) _dowriteall (buffer);
+	kpause (5000L);
+	where_outta_here (0, "doshutdown");
+	return 0;			/*lint !e527 */
+}
+
+
+
 #ifdef MAILBOX
 extern void conversWriteall (char *str);
 
 
+static int
+_dowriteall (char *str)
+{
+register struct mbx *m;
+int i;
+
+	for (i = 0; i < NUMMBX; i++)
+		if ((m = Mbox[i]) != NULLMBX) {
+			usprintf (m->user, FromSysop, str);
+			usflush (m->user);
+		}
+#ifdef CONVERS
+	conversWriteall (str);
+#endif
+	return 0;
+}
+
+
+
 /* write a message to all nodeshell users
  * argv[1] is the message.
  */
 int
 dowriteall (int argc OPTIONAL, char *argv[], void *p OPTIONAL)
 {
-register struct mbx *m;
-int i;
-
-	for (i = 0; i < NUMMBX; i++)
-		if ((m = Mbox[i]) != NULLMBX) {
-			usprintf (m->user, FromSysop, argv[1]);
-			usflush (m->user);
-		}
-#ifdef CONVERS
-	conversWriteall (argv[1]);
-#endif
-	return 0;
+	return _dowriteall (argv[1]);
 }
 
 #endif
@@ -2290,7 +2379,7 @@ char buf[80];
 	tprintf (Nosversion, Version, Version2);
 	tputs (Compiled);
 	if (strlen (symname) > 9) {
-		strcpy (buf, &symname[7]);
+		strncpy (buf, &symname[7], 80);
 		buf[strlen (buf) - 2] = 0;
 		tprintf ("Symbolic RCS Name for this Executable: '%s'\n", buf);
 	}
@@ -2361,7 +2450,7 @@ domotd (int argc, char *argv[], void *p OPTIONAL)
 		if (!strlen (argv[1]))
 			return 0;	/* clearing the buffer */
 
-		Motd = mallocw (strlen (argv[1]) + 5);	/* allow for the EOL char etc */
+		Motd = mallocw (strlen (argv[1]) + 2);	/* allow for the EOL char etc */
 		strcpy (Motd, argv[1]);
 		strcat (Motd, "\n");	/* add the EOL char */
 	}
@@ -2533,9 +2622,9 @@ struct session *sp;
 	}
 	_setcursortype (_NOCURSOR);
 	while (sp == Current) {
-		/*  clrscr(); */
-		/* gotoxy seems to work better - turn cursor off?? */
-		gotoxy (1, 1);
+		clrscr();
+		/* gotoxy seems to work better (but doen't clear screen - turn cursor off?? */
+		/* gotoxy (1, 1); */
 		ret = subcmd (Cmds, argc, argv, p);
 		if (ret != 0 || kpause (interval) == -1)
 			break;
@@ -2556,7 +2645,7 @@ int size = 0;
 	if ((fp = fopen (fname, READ_TEXT)) != (FILE *) 0) {
 		(void) sendfile (fp, thesocket, ASCII_TYPE, 0);
 		size = ftell (fp);
-		fclose (fp);
+		(void) fclose (fp);
 	}
 	return (size);
 }
@@ -2591,3 +2680,17 @@ register int i;
 
 
 
+void
+crash_it_already (char const *where)
+{
+	/* TRY a log */
+	log (-1, "A 'crashprotect'-preventable restart occurred from a %s", where);
+	exit (1);
+}
+
+
+int
+shall_we_crash (void)
+{
+	return (CrashProtect == 0);
+}
