@@ -7,7 +7,7 @@
 #endif
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: merge.c,v 1.10 1996/12/23 22:44:37 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: merge.c,v 1.11 1997/07/31 00:44:20 root Exp root $";
 #endif
 
 FILE *
@@ -43,7 +43,7 @@ char buf[128], name[128];
 	if ((fpnew = fopen(name, "rt")) == (FILE *)NULL)
 		return 0;
 	if ((fp = fopen(fname, "r+t")) == (FILE *)NULL)		{
-		fclose (fpnew);
+		(void) fclose (fpnew);
 		(void) rename(name, fname);
 		return 1;
 	}
@@ -53,8 +53,8 @@ char buf[128], name[128];
 		kwait (NULL);
 		fputs (buf, fp);
 	}
-	fclose (fp);
-	fclose (fpnew);
+	(void) fclose (fp);
+	(void) fclose (fpnew);
 	(void) remove (name);
 	return 1;
 }
