@@ -46,7 +46,7 @@
 #include "hardware.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: nntpcli.c,v 1.19 1997/05/31 14:41:31 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: nntpcli.c,v 1.21 1997/09/07 21:18:28 root Exp root $";
 #endif
 
 #define NNTPMAXLEN	512
@@ -183,7 +183,7 @@ struct nntpservers *np;
 	}
 	/* set timer duration */
 	set_timer (&np->nntpcli_t, atol (argv[2]) * 1000L);
-	start_timer (&np->nntpcli_t);	/* and fire it up */
+	start_detached_timer (&np->nntpcli_t);	/* and fire it up */
 	return 0;
 }
 
@@ -394,14 +394,14 @@ char tbuf[NNTPMAXLEN], buf[NNTPMAXLEN], *cp, *lastdate = NULLCHAR;
 		if (now < np->lowtime || now >= np->hightime) {
 			if (nntptrace >= 3)
 				tcmdprintf ("NNTP window to '%s' not open\n", np->name);
-			start_timer (&np->nntpcli_t);
+			start_detached_timer (&np->nntpcli_t);
 			return;
 		}
 	} else {
 		if (now < np->lowtime && now >= np->hightime) {
 			if (nntptrace >= 3)
 				tcmdprintf ("NNTP window to '%s' not open\n", np->name);
-			start_timer (&np->nntpcli_t);
+			start_detached_timer (&np->nntpcli_t);
 			return;
 		}
 	}
@@ -411,7 +411,7 @@ char tbuf[NNTPMAXLEN], buf[NNTPMAXLEN], *cp, *lastdate = NULLCHAR;
 		if (nntptrace >= 2)
 			tcmdprintf ("NNTP can't resolve host '%s'\n", np->name);
 		/* Try again later */
-		start_timer (&np->nntpcli_t);
+		start_detached_timer (&np->nntpcli_t);
 		return;
 	}
 	fsocket.sin_family = AF_INET;
@@ -591,7 +591,7 @@ quit:
 		tcmdprintf ("NNTP daemon exiting\n");
 	close_s (s);
 	/* Restart timer */
-	start_timer (&np->nntpcli_t);
+	start_detached_timer (&np->nntpcli_t);
 	return;
 }
 
@@ -686,7 +686,7 @@ time_t t;
 				break;
 		}
 		if (strncmp (buf, Hdrs[NEWSGROUPS], 12) == 0) {
-			strcpy (newgl, &buf[12]);
+			strncpy (newgl, &buf[12], NNTPMAXLEN);
 			if (froml[0] != '\0')
 				break;
 		}
@@ -707,10 +707,10 @@ time_t t;
 		if (*cp == ',' || *cp == '\n') {
 			char tempdir[80], prefix[20], *p;
 
-			strcpy (tempdir, buf);
+			strncpy (tempdir, buf, 80);
 			if ((p = strrchr (tempdir, '/')) != NULLCHAR) {
 				*p++ = '\0';
-				strcpy (prefix, p);
+				strncpy (prefix, p, 20);
 			}
 			if (mlock (tempdir, prefix)) {
 				if (nntptrace >= 2)
@@ -754,9 +754,9 @@ time_t t;
 		buf[strlen (buf)] = strchr (validchars, tolower (*cp)) ? *cp : '_';
 	}
 	fclose (tmpf);
-	strcpy (buf, msgid);	/* Get a copy we can munge */
-	rip (buf);		/* remove trailing new-line */
-	rip (newgl);		/* ditto */
+	strncpy (buf, msgid, NNTPMAXLEN);	/* Get a copy we can munge */
+	rip (buf);				/* remove trailing new-line */
+	rip (newgl);				/* ditto */
 	if (!nntpquiet)
 		tprintf ("New news arrived: %s, article %s%c\n", newgl, buf, Smtpquiet ? ' ' : '\007');
 	return 0;
