@@ -15,7 +15,7 @@
 #include "trace.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: nr3.c,v 1.19 1996/12/23 20:37:36 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: nr3.c,v 1.21 1997/09/07 21:18:28 root Exp root $";
 #endif
 
 /* IF the following is defined,
@@ -326,7 +326,7 @@ struct iface *iface;
 	}
 	bp->data[0] = PID_NETROM;
 
-	if ((pbp = segmenter (bp, axp->paclen)) == NULLBUF) {
+	if ((pbp = segmenter (bp, axp->paclen, axp)) == NULLBUF) {
 		free_p (bp);
 		return;
 	}
@@ -506,13 +506,13 @@ void *p OPTIONAL;
 	Nodetimer.func = (void (*)(void *)) donodetick;	/* what to call on timeout */
 	Nodetimer.arg = NULLCHAR;	/* dummy value */
 	set_timer (&Nodetimer, 1800000L);	/* 'standard netrom' 30 minutes*/
-	start_timer (&Nodetimer);	/* and fire it up */
+	start_detached_timer (&Nodetimer);	/* and fire it up */
 
 	stop_timer (&Obsotimer);
 	Obsotimer.func = (void (*)(void *)) doobsotick;	/* what to call on timeout */
 	Obsotimer.arg = NULLCHAR;	/* dummy value */
 	set_timer (&Obsotimer, 2100000L);	/* 35 minutes */
-	start_timer (&Obsotimer);	/* and fire it up */
+	start_detached_timer (&Obsotimer);	/* and fire it up */
 
 	return 0;
 }
