@@ -34,7 +34,7 @@
 #define	DEL	0x7f
 
 #ifndef MSDOS
-static char rcsid[] OPTIONAL = "$Id: osk.c,v 1.14 1997/06/01 22:10:46 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: osk.c,v 1.15 1997/07/31 00:44:20 root Exp root $";
 #endif
 
 static int kbchar (void);
@@ -969,7 +969,7 @@ FILE *fp;
 }
 
 void
-getdate (datep)
+_getdate (datep)
 struct date *datep;
 {
 int time, date, tick;
