@@ -22,7 +22,7 @@
 #include "smtp.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: pc.c,v 1.24 1997/06/01 22:10:46 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: pc.c,v 1.25 1997/07/31 00:44:20 root Exp root $";
 #endif
 
 #define FP_OFF( fp )( (unsigned )( fp ))
@@ -717,7 +717,7 @@ unsigned char attr;
 				tr.winbottom = SCREENlength;
 			}
 			stowit ((unsigned long) old->index, 1, 0);
-			//save it
+			/* save it */
 		}
 	}
 	if (new != NULLSESSION) {
@@ -730,11 +730,11 @@ unsigned char attr;
 
 			setscreens ((new->screen->attr >> 4) & 0x07, new->screen->attr & 0x0f, 0);
 			stowit ((unsigned long) new->index, 0, new->screen->attr);
-			//retrieve it
+			/* retrieve it */
 				new->screen->attr &= 0x7f;
-			//remove color change bit
+			/* remove color change bit */
 
-				// if (STATLINE && (Current == Command))
+				/* if (STATLINE && (Current == Command)) */
 				window (1, 1 + new->screen->statline, SCREENwidth, SCREENlength - (new->split * 2));
 			gotoxy (new->screen->col, new->screen->row);
 			textattr (new->screen->attr);
@@ -771,7 +771,9 @@ display (int i, void *v1, void *v2)
 {
 int c;
 struct session *sp;
+int statsize;
 
+	server_disconnect_io ();
 	/* This is very tricky code. Because the value of "Current" can
 	 * change any time we do a kwait, we have to be careful to detect
 	 * any change and go back and start again.
@@ -815,13 +817,15 @@ struct session *sp;
 		if (sp->record != NULLFILE)	/* Don't save CR if ascii mode */
 			if (c != '\r' || sockmode (sp->output, -1) != SOCK_ASCII)
 				putc (c, sp->record);
-#ifdef notdef
-		if (sp->record != NULLFILE)
-			putc (c, sp->record);
-#endif
-		if (sp->flowmode && c == '\n' && --sp->row <= 0) {
+
+		statsize = 0;
+		if (sp->screen->statline)
+			statsize = ((sp == Command) ? getStatlines() : 1);
+
+		if (sp->flowmode && c == '\n' && sp->row > 0 && --sp->row == statsize) {
 			cprintf (somemore);
 			sp->morewait = 1;
+			sp->row = 0;
 		}
 	}
 }
@@ -1217,17 +1221,17 @@ register char *cp;
 			l = (int) (scrsize / 2);
 			split = Sessions[theindex].split;
 			if (split)
-				l -= 160;	// last 2 lines reversed
+				l -= 160;	/* last 2 lines reversed */
 			cp = (nerf + 1);
 			for (k = 0; k < l; k++)	{
-				*cp &= 8;	// keep high video bit
+				*cp &= 8;	/* keep high video bit */
 				*cp |= attr;
 				cp += 2;
 				}
 			attr = ((attr & 0x0f) << 4) + ((attr & 0x70) >> 4);
 			if (split)
 				for (k = 0; k < 160; k++)	{
-					*cp &= 8;	// keep high video bit
+					*cp &= 8;	/* keep high video bit */
 					*cp |= attr;
 					cp += 2;
 				}
