@@ -28,7 +28,7 @@
 #include "stats.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: pop2serv.c,v 1.17 1997/05/24 12:55:33 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: pop2serv.c,v 1.19 1997/09/07 21:18:28 root Exp root $";
 #endif
 
 
@@ -77,7 +77,6 @@ static char	count_rsp[]    = "#%d messages in this folder\n",
 		no_more_rsp[]  = "=%d No more messages in this folder\n",
 		signoff_msg[]  = "+ Bye, thanks for calling\n";
 
-void rrip (register char *s);
 static struct pop_scb *create_scb (void);
 static void delete_scb (struct pop_scb *scb);
 static void popserv (int s,void *unused,void *p);
@@ -154,7 +153,7 @@ loop:	if ((scb->count = recvline(s,scb->buf,TLINELEN)) == -1){
 		goto quit;
 	}
 
-	rrip(scb->buf); 	/* Was "rip()" not "rrip()" -- N1BEE */
+	rip(scb->buf);
 	if (strlen(scb->buf) == 0)	/* Ignore blank cmd lines */
 		goto loop;
 
@@ -226,25 +225,6 @@ register struct pop_scb *scb;
 
 
 
-/* replace terminating end of line marker(s) (\r and \n) with null */
-void
-rrip(s)
-register char *s;
-{
-register char *cp;
-
-	if((cp = strchr(s,'\r')) != NULLCHAR)
-		*cp = '\0';
-#ifndef TNOS_68K
-	if((cp = strchr(s,'\n')) != NULLCHAR)
-#else
-	if((cp = strchr(s,'\l')) != NULLCHAR)
-#endif
-		*cp = '\0';
-}
-
-
-
 /* --------------------- start of POP server code ------------------------ */
 
 #define BITS_PER_WORD	16
@@ -437,7 +417,7 @@ struct pop_scb	*scb;
 static void
 close_folder (struct pop_scb *scb)
 {
-char folder_pathname[64];
+char folder_pathname[128];
 char line[TLINELEN+1];
 FILE *fd;
 int deleted = FALSE;
@@ -686,7 +666,7 @@ long cnt;
 	cnt  = scb->msg_len;
 	while(!feof(scb->wf) && (cnt > 0)) {
 		fgets(line,TLINELEN,scb->wf);
-		rrip(line);
+		rip(line);
 
 		usprintf(scb->socket,msg_line,line);
 		cnt -= (strlen(line)+2);	/* Compensate for CRLF */
@@ -720,7 +700,7 @@ int isdeleted (struct pop_scb *, int);
 				scb->curpos = ftell(scb->wf);
 		        
 			fgets(line,TLINELEN,scb->wf);
-			rrip(line);
+			rip(line);
 
 			if (isSOM(line))
 				msg_no--;
