@@ -23,7 +23,7 @@
 
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: ppplcp.c,v 1.13 1997/05/24 12:47:53 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: ppplcp.c,v 1.14 1997/07/31 00:44:20 root Exp root $";
 #endif
 
 /* These defaults are defined in the PPP RFCs, and must not be changed */
@@ -278,7 +278,7 @@ dolcp_magic (int argc, char *argv[], void *p)
 {
 struct lcp_side_s *side_p = p;
 int result = 0;
-int32 clock;
+int32 theclock;
 
 	if (argc < 2)
 		tprintf ("%ld\n", side_p->want.magic_number);
@@ -295,8 +295,8 @@ int32 clock;
 			result = setbool (&test, "Magic Number", argc, argv);
 
 			if (test) 	{	/* Make a non-zero random number */
-				clock = rdclock();
-				x = clock << ((clock & 0xf) + 8);	/*lint !e703 !e564 */
+				theclock = rdclock();
+				x = theclock << ((theclock & 0xf) + 8);	/*lint !e703 !e564 */
 			}
 		}
 		if (x)
