@@ -4,10 +4,14 @@
 /* Remote reset/restart server definitions */
 
 extern char const *Rempass;
+extern char const *RemRTpass;
 
 /* Commands */
 #define	SYS_RESET	1
 #define	SYS_EXIT	2
 #define	KICK_ME		3
+#define ROUTE_ADD	4
+#define ROUTE_DROP	5
+
 
 #endif	/* _REMOTE_H */
