@@ -38,7 +38,7 @@
 #include "rip.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: rip.c,v 1.22 1997/06/28 16:46:13 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: rip.c,v 1.25 1997/09/07 21:18:28 root Exp root $";
 #endif
 
 struct rip_stat Rip_stat = {{{0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}}, 0, 0, 0, 0, 0, 0};
@@ -87,7 +87,7 @@ register struct rip_list *rl;
 	stop_timer (&rl->rip_time);
 	send_routes (rl->dest, RIP_PORT, 0, rl->flags, (unsigned char) rl->rip_version, rl);
 	set_timer (&rl->rip_time, rl->interval * 1000L);
-	start_timer (&rl->rip_time);
+	start_detached_timer (&rl->rip_time);
 }
 
 
@@ -339,7 +339,7 @@ struct route *rp;
 	set_timer (&rl->rip_time, interval * 1000L);
 	rl->rip_time.func = rip_shout;
 	rl->rip_time.arg = rl;
-	start_timer (&rl->rip_time);
+	start_detached_timer (&rl->rip_time);
 	return 1;
 }
 
@@ -396,12 +396,11 @@ int x;
 	Rip_auth = ra;
 
 	/* fill in the data */
-	ra->ifc_name = mallocw (strlen (ifcname) + 1);
-	strcpy (ra->ifc_name, ifcname);
+	ra->ifc_name = strdup (ifcname);
 	ra->domain = domain;
 	for (x = 0; x < RIP_AUTH_SIZE + 1; x++)
 		ra->rip_auth_code[x] = '\0';
-	strcpy (ra->rip_auth_code, password);
+	strncpy (ra->rip_auth_code, password, RIP_AUTH_SIZE);
 	return 0;
 }
 
@@ -684,7 +683,7 @@ struct rip_list *rl;
 			 */
 			if ((rp = rt_blookup (fsock.address, 32)) != NULLROUTE)
 				/* Host-specific route already exists, refresh it */
-				start_timer (&rp->timer);
+				start_detached_timer (&rp->timer);
 			else if ((rp = rt_lookup (fsock.address)) == NULLROUTE
 				 || rp->iface != iface) {
 				entry.rip_family = RIP_AF_INET;
@@ -892,7 +891,7 @@ int trigger = 0;
 			   inet_ntoa (target), bits, ep->rip_metric);
 	} else if (rp->gateway == gateway && rp->iface == iface) {
 		/* This is the gateway for the entry we already have, restart the timer */
-		start_timer (&rp->timer);
+		start_detached_timer (&rp->timer);
 		if (rp->metric != ep->rip_metric) {
 			/* Metric has changed. Update it and trigger an
 			 * update. If route has become unavailable, start
@@ -1162,7 +1161,7 @@ register struct route *rp = (struct route *) s;
 		set_timer (&rp->timer, dur_timer (&rp->timer) * 2 / 3);
 		rp->timer.func = rt_timeout;
 		rp->timer.arg = (void *) rp;
-		start_timer (&rp->timer);
+		start_detached_timer (&rp->timer);
 		/* Route changed; mark it for triggered update */
 		rp->flags |= RTTRIG;
 		rip_trigger ();
@@ -1204,7 +1203,7 @@ rip_trace (short level, char const *errstr,...)
 			(void) vfprintf (Rip_trace_file, errstr, argptr);
 			va_end (argptr);
 			fprintf (Rip_trace_file, "\n");
-			fclose (Rip_trace_file);
+			(void) fclose (Rip_trace_file);
 		} else {
 			va_start (argptr, errstr);
 			(void) usvprintf (Curproc->output, errstr, argptr);
