@@ -17,7 +17,7 @@
 #include "rspf.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: rspf.c,v 1.21 1997/05/31 14:42:01 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: rspf.c,v 1.23 1997/09/07 21:18:28 root Exp root $";
 #endif
 
 #ifdef RSPF
@@ -332,7 +332,7 @@ int dif, origdif;
 					 */
 					if (dur_timer (&adj->timer) == 0L)
 						set_timer (&adj->timer, dur_timer (&Susptimer));
-					start_timer (&adj->timer);
+					start_detached_timer (&adj->timer);
 					set_timer (&adj->timer, dur_timer (&oldadj->timer));
 				}
 				/* We must now try to figure out from which AX.25 callsign this
@@ -429,7 +429,7 @@ struct rspfadj *prev;
 				adj->state = RSPF_SUSPECT;	/* The adjacency is now Suspect */
 			} else {/* Try later */
 				set_timer (&adj->timer, dur_timer (&Susptimer));
-				start_timer (&adj->timer);
+				start_detached_timer (&adj->timer);
 			}
 			break;
 		case RSPF_BAD:
@@ -470,7 +470,7 @@ struct mbuf *bp;
 	if ((s = socket (AF_INET, SOCK_RAW, ICMP_PTCL)) == -1) {
 		--Nrspfproc;
 		adj->state = (char) oldstate;
-		start_timer (&adj->timer);
+		start_detached_timer (&adj->timer);
 		return;
 	}
 	fromlen = sizeof (from);
@@ -487,7 +487,7 @@ struct mbuf *bp;
 				adj->state = (char) oldstate;
 				close_s (s);
 				--Nrspfproc;
-				start_timer (&adj->timer);
+				start_detached_timer (&adj->timer);
 				return;
 			}
 			(void) ntohicmp (&icmp, &bp);
@@ -501,7 +501,7 @@ struct mbuf *bp;
 				goodbadnews (adj);	/* Good news */
 			close_s (s);
 			--Nrspfproc;
-			start_timer (&adj->timer);
+			start_detached_timer (&adj->timer);
 			adj->state = RSPF_OK;	/* Finally change state */
 			return;
 		}
@@ -510,7 +510,7 @@ struct mbuf *bp;
 	adj->state = RSPF_BAD;
 	close_s (s);
 	--Nrspfproc;
-	start_timer (&adj->timer);
+	start_detached_timer (&adj->timer);
 }
 
 
@@ -553,7 +553,7 @@ struct rspfiface *riface;
 		default:
 			return;
 	}
-	if ((adj = (struct rspfadj *) calloc (1, sizeof (struct rspfadj))) == NULLADJ)
+	if ((adj = (struct rspfadj *) callocw (1, sizeof (struct rspfadj))) == NULLADJ)
 		        return;
 
 	adj->addr = addr;
@@ -565,7 +565,7 @@ struct rspfiface *riface;
 		adj->timer.func = rspfevent;	/* Fill in timer values */
 		adj->timer.arg = (void *) &adj->timer;
 		set_timer (&adj->timer, dur_timer (&Susptimer));
-		start_timer (&adj->timer);
+		start_detached_timer (&adj->timer);
 	}
 	if ((bp = alloc_mbuf (1 + sizeof (int32) + sizeof (iface))) == NULLBUF) {
 		stop_timer (&adj->timer);
@@ -615,7 +615,7 @@ struct route *rp;
 struct arp_tab *ap;
 int i;
 
-	start_timer (&Susptimer);	/* restart the timer */
+	start_detached_timer (&Susptimer);	/* restart the timer */
 	for (i = 0; i < HASHMOD; i++)	/* Check the routing table */
 		for (rp = Routes[31][i]; rp != NULLROUTE; rp = rp->next) {
 			if ((rp->flags & RTPRIVATE) || dur_timer (&rp->timer) != 0)
@@ -1314,7 +1314,7 @@ timstart:		/* Start the timer if needed */
 		Rspfreasmt.func = reasmtimeout;
 		Rspfreasmt.arg = NULL;
 		set_timer (&Rspfreasmt, RSPF_RTIME * 1000L);	/* The reassembly timeout */
-		start_timer (&Rspfreasmt);
+		start_detached_timer (&Rspfreasmt);
 	}
 	return NULLBUF;
 }
@@ -1389,7 +1389,7 @@ int32 low;
 		/* KF8NH 1993.01.17: clock() on below line changed to secclock() */
 		set_timer (&Rspfreasmt, (RSPF_RTIME - (secclock () - low)) * 1000L);
 		if (dur_timer (&Rspfreasmt) > 0)	/* just to be safe */
-			start_timer (&Rspfreasmt);
+			start_detached_timer (&Rspfreasmt);
 	}
 }
 
