@@ -32,7 +32,7 @@ void mail2news (FILE *fp, const char *from, const char *to, const char *area);
 #include "stats.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: smtpserv.c,v 1.40 1997/06/13 13:20:42 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: smtpserv.c,v 1.42 1997/08/19 01:19:22 root Exp root $";
 #endif
 
 const char *Days[7] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
@@ -46,10 +46,6 @@ extern char *HOLDsound;
 extern int HOLDmode, HOLDindicator;
 #endif
 
-#if defined(__bsdi__) || defined(__FreeBSD__)
-extern char *tzname[2];
-#endif
-
 #ifdef __FreeBSD__
 /*  timezone is a function that does something else in FreeBSD	*/
 #define timezone (-ltm->tm_gmtoff)
@@ -77,6 +73,7 @@ static struct smtpsv *mail_create (void);
 static void mail_clean (struct smtpsv * mp);
 static int mailit (FILE * data, const char *from, struct list * tolist, char *sender, int enablehold, char **destarea);
 static int router_queue (FILE * data, const char *from, struct list * to);
+static int __queuejob (FILE *dfile, char *host, struct list *to, const char *from, const char *qdir);
 static void smtplog (const char *fmt,...);
 static void smtpserv (int s, void *unused, void *p);
 int mailuser (FILE * data, const char *from, const char *to, const char *origto);
@@ -376,7 +373,7 @@ loop:
 					free (newaddr);
 					break;
 				}
-				strcpy (buf, newaddr);
+				strncpy (buf, newaddr, LINELEN);
 				cp = buf;
 				free (newaddr);
 			}
@@ -438,7 +435,7 @@ loop:
 					free (newaddr);
 					newaddr = NULLCHAR;
 				} else {
-					strcpy (buf, newaddr);
+					strncpy (buf, newaddr, LINELEN);
 					arg = buf;
 				}
 			list = NULLLIST;
@@ -608,7 +605,7 @@ int firstfrom = 0;
 					(void) usputs (mp->s, Ioerr);
 				else
 					usprintf (mp->s, Sent);
-				fclose (mp->data);
+				(void) fclose (mp->data);
 				mp->data = NULLFILE;
 				del_list (mp->to);
 				mp->to = NULLLIST;
@@ -723,7 +720,7 @@ mail_clean (register struct smtpsv *mp)
 		free (mp->sender);
 	free (mp->bid);
 	if (mp->data != NULLFILE)
-		fclose (mp->data);
+		(void) fclose (mp->data);
 	del_list (mp->to);
 	free ((char *) mp);
 }
@@ -754,7 +751,7 @@ int oldstatus;
 
 			info->status = (int16) oldstatus;
 		}
-		fclose (fp);
+		(void) fclose (fp);
 	}
 }
 
@@ -773,7 +770,7 @@ char mailbox[100];
 	if ((fp = subdir_fopen (mailbox, APPEND_BINARY)) != NULLFILE) {
 		fwrite (info, sizeof (struct let), 1, fp);
 
-		fclose (fp);
+		(void) fclose (fp);
 	}
 }
 
@@ -797,7 +794,7 @@ time_t now;
 	(void) time (&now);
 	if ((fp = fopen (mailbox, APPEND_TEXT)) != NULLFILE) {
 		fprintf (fp, " %s %-ld %-ld %-ld\n", area, bid, now, size);
-		fclose (fp);
+		(void) fclose (fp);
 	}
 #ifdef MBFWD
 	fwdunlockit (who);
@@ -909,7 +906,7 @@ char rejto[256];
 				if (theindex == 99)
 					break;
 			}
-			fclose (fpp);
+			(void) fclose (fpp);
 		}
 		badwords[theindex] = NULLCHAR;
 		totalindex = theindex;
@@ -1230,7 +1227,7 @@ char rejto[256];
 #if defined(REJECT) || defined(NNTPS)
 							cp = getaddress (buf, 0);
 							if (cp)
-								strcpy (rejto, cp);
+								strncpy (rejto, cp, 256);
 #endif
 							break;
 #ifdef EDITHEADERS
@@ -1241,7 +1238,7 @@ char rejto[256];
 #ifdef REJECT
 							cp = getaddress (buf, 0);
 							if (cp)
-								strcpy (rejfrom, cp);
+								strncpy (rejfrom, cp, 256);
 							if ((cp = strchr (rejfrom, '%')) != NULLCHAR)
 								*cp = 0;
 #endif
@@ -1298,7 +1295,7 @@ char rejto[256];
 				(void) fflush (fp);
 				endedat = ftell (fp);
 #endif
-				fclose (fp);
+				(void) fclose (fp);
 
 #ifdef ALERTMONITOR
 				alertarea (realarea);
@@ -1312,14 +1309,14 @@ char rejto[256];
 							while (fgets (buf, sizeof (buf), fpp) != NULLCHAR) {
 								rip (buf);
 								(void) strlwr (buf);
-								strcpy (badbuf, from);
+								strncpy (badbuf, from, LINELEN);
 								(void) strlwr (badbuf);
 								if (strstr (badbuf, buf)) {
 									Holdit = 1;
 									break;
 								}
 								if (sender) {
-									strcpy (badbuf, sender);
+									strncpy (badbuf, sender, LINELEN);
 									(void) strlwr (badbuf);
 									if (strstr (badbuf, buf)) {
 										Holdit = 1;
@@ -1327,7 +1324,7 @@ char rejto[256];
 									}
 								}
 							}
-							fclose (fpp);
+							(void) fclose (fpp);
 							if (Holdit)
 								log (-1, "UserHold New mail for %s from <%s>\n", realarea, from);
 						}
@@ -1349,7 +1346,7 @@ char rejto[256];
 						int rej;
 
 						sprintf (rejbf, "S%c %s <%s %s", bbstype, rejto, rejfrom, rejmid);
-						rej = reject (rejbf, 1);
+						rej = reject (rejbf, 1, 0);
 						if (rej == REJ_REJECT || rej == REJ_DEFER) {
 							lt.status = BM_DELETE;
 							log (-1, "REJECTED New mail for %s from <%s>\n", realarea, from);
@@ -1489,10 +1486,10 @@ ptime (time_t *t)
 #ifndef USE_TZOFFSET
 	sprintf (str, "%s, %.2d %s %02d %02d:%02d:%02d %s\n",
 #else
-#if defined(sun) || defined(__bsdi__) || defined(MSDOS)
-	offset = ltm->tm_gmtoff / -36;
-#else
+#if defined(HAVE_TM_ISDST) && !defined(MSDOS)
 	offset = (timezone - (ltm->tm_isdst * 3600L)) / (-36L);
+#else
+	offset = ltm->tm_gmtoff / -36;
 #endif
 	if (offset % 100L)
 		offset = (100L * (offset / 100L)) + (60L * (offset % 100L)) / 100L;
@@ -1505,18 +1502,18 @@ ptime (time_t *t)
 		 ltm->tm_hour,
 		 ltm->tm_min,
 		 ltm->tm_sec,
-#ifndef USE_TZOFFSET
-#ifdef sun
-		 ltm->tm_zone);
+#ifdef USE_TZOFFSET
+		 offset);
 #else
 #ifdef MSDOS
 		 ltm->__tm_zone);
 #else
+#ifdef HAVE_TM_ISDST
 		 ltm->tm_isdst ? tzname[1] : tzname[0]);
-#endif
-#endif
 #else
-		 offset);
+		 ltm->tm_zone);
+#endif
+#endif
 #endif
 	return (str);
 }
@@ -1548,14 +1545,14 @@ FILE *sfile;
 			if (sequence < 0L || sequence > 99999998L)
 				sequence = 0;
 		}
-		fclose (sfile);
+		(void) fclose (sfile);
 	}
 	/* increment sequence number, and write to sequence file */
 	sfile = fopen (sfilename, WRITE_TEXT);
 	if (sfile != NULLFILE)	{
 		setbuf (sfile, NULL);
 		fprintf (sfile, "%ld", ++sequence);
-		fclose (sfile);
+		(void) fclose (sfile);
 	}
 	return sequence;
 }
@@ -1630,27 +1627,30 @@ uint32 addr;
 
 
 
-/* place a mail job in the outbound queue */
-int
-queuejob (
+/* place a mail job in one of the queues */
+static int
+__queuejob (
 register FILE *dfile,
-char *host,
+char *host,		/* equal to NULLCHAR, if placing in rqueue dir */
 struct list *to,
-const char *from
+const char *from,
+const char *qdir
 ) {
 register FILE *fp;
-struct list *ap;
+register struct list *ap;
 char tmpstring[50], prefix[9], buf[LINELEN];
 register int cnt;
 long totalcnt = 0;
 
 	sprintf (prefix, "%ld", get_msgid (0));
-	(void) mlock (Mailqdir, prefix);
-	sprintf (tmpstring, "%s/%s.txt", Mailqdir, prefix);
+	(void) mlock (qdir, prefix);
+	sprintf (tmpstring, "%s/%s.txt", qdir, prefix);
 	if ((fp = fopen (tmpstring, WRITE_TEXT)) == NULLFILE) {
-		rmlock (Mailqdir, prefix);
+		rmlock (qdir, prefix);
 		return 1;
 	}
+	
+	rewind (dfile);
 	while ((cnt = (int) fread (buf, 1, LINELEN, dfile)) > 0) {
 		totalcnt += cnt;
 		kwait (NULL);
@@ -1658,24 +1658,36 @@ long totalcnt = 0;
 			break;
 	}
 	if (ferror (fp)) {
-		fclose (fp);
-		rmlock (Mailqdir, prefix);
+		(void) fclose (fp);
+		rmlock (qdir, prefix);
 		return 1;
 	}
-	fclose (fp);
-	sprintf (tmpstring, "%s/%s.wrk", Mailqdir, prefix);
+	(void) fclose (fp);
+
+	sprintf (tmpstring, "%s/%s.wrk", qdir, prefix);
 	if ((fp = fopen (tmpstring, WRITE_TEXT)) == NULLFILE) {
-		rmlock (Mailqdir, prefix);
+		rmlock (qdir, prefix);
 		return 1;
 	}
 	kwait (NULL);
-	fprintf (fp, "%s\n%s\n", host, from);
+
+	if (host != NULLCHAR)
+		fprintf (fp, "%s\n%s\n", host, from);
+	else
+		fprintf (fp, "From: %s\n", from);
+
 	for (ap = to; ap != NULLLIST; ap = ap->next) {
+		if (host == NULLCHAR)
+			fprintf (fp, "To: ");
+		else
+			smtplog ("queue job %s To: %s From: %s Size: %ld", prefix, ap->val, from, totalcnt);
 		fprintf (fp, "%s\n", ap->val);
-		smtplog ("queue job %s To: %s From: %s Size: %ld", prefix, ap->val, from, totalcnt);
 	}
-	fclose (fp);
-	rmlock (Mailqdir, prefix);
+	(void) fclose (fp);
+
+	rmlock (qdir, prefix);
+	if (host == NULLCHAR)
+		smtplog ("rqueue job %s From: %s Size: %ld", prefix, from, totalcnt);
 	return 0;
 }
 
@@ -1688,48 +1700,23 @@ FILE *data,
 const char *from,
 struct list *to
 ) {
-int c;
-register struct list *ap;
-FILE *fp;
-char tmpstring[50];
-char prefix[9];
-
-	sprintf (prefix, "%ld", get_msgid (0));
-	(void) mlock (Routeqdir, prefix);
-	sprintf (tmpstring, "%s/%s.txt", Routeqdir, prefix);
-	if ((fp = fopen (tmpstring, WRITE_TEXT)) == NULLFILE) {
-		rmlock (Routeqdir, prefix);
-		return 1;
-	}
-	rewind (data);
-	kwait (NULL);
-	while ((c = getc (data)) != EOF)
-		if (putc (c, fp) == EOF)
-			break;
-	kwait (NULL);
-	if (ferror (fp)) {
-		fclose (fp);
-		rmlock (Routeqdir, prefix);
-		return 1;
-	}
-	fclose (fp);
-	sprintf (tmpstring, "%s/%s.wrk", Routeqdir, prefix);
-	if ((fp = fopen (tmpstring, WRITE_TEXT)) == NULLFILE) {
-		rmlock (Routeqdir, prefix);
-		return 1;
-	}
-	fprintf (fp, "From: %s\n", from);
-	for (ap = to; ap != NULLLIST; ap = ap->next)
-		fprintf (fp, "To: %s\n", ap->val);
-
-	fclose (fp);
-	rmlock (Routeqdir, prefix);
-	smtplog ("rqueue job %s From: %s", prefix, from);
-	return 0;
+	return (__queuejob (data, NULLCHAR, to, from, Routeqdir));
 }
 
 
 
+/* place a mail job in the outbound queue */
+int
+queuejob (
+register FILE *dfile,
+char *host,
+struct list *to,
+const char *from
+) {
+	return (__queuejob (dfile, host, to, from, Mailqdir));
+}
+
+
 /* add an element to the front of the list pointed to by head
 ** return NULLLIST if out of memory.
 */
@@ -2062,7 +2049,7 @@ FILE *fp;
 	(void) vfprintf (fp, fmt, ap);
 	va_end (ap);
 	fprintf (fp, "\n");
-	fclose (fp);
+	(void) fclose (fp);
 }
 
 
@@ -2169,7 +2156,7 @@ int cnt;
 	fseek (tfile, 0L, 0);
 	/* A null From<> so no looping replys to MAIL-DAEMONS */
 	(void) mailuser (tfile, "", newto != NULLCHAR ? newto : to, newto != NULLCHAR ? newto : to);
-	fclose (tfile);
+	(void) fclose (tfile);
 	free (newto);
 	return 0;
 }
