@@ -16,7 +16,7 @@
 #include "lzw.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: socket.c,v 1.22 1997/06/01 22:10:46 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: socket.c,v 1.24 1997/08/19 01:19:22 root Exp root $";
 #endif
 
 /* In connect(), accept() and recv_mbuf()
@@ -693,7 +693,7 @@ struct nr3hdr n3hdr;
 			       && up->cb.local->peer != NULLUSOCK) {
 #ifdef useblock
 				if (up->noblock & SOCK_NORXBLOCK) {
-					errno = EWOULDBLOCK;
+					errno = (Curproc->retval == EALARM) ? EALARM : EWOULDBLOCK;
 					return -1;
 				} else
 #endif
@@ -724,7 +724,7 @@ struct nr3hdr n3hdr;
 			    && (cnt = recv_tcp (cb.tcb, &bp, (int16) 0)) == -1) {
 #ifdef useblock
 				if (up->noblock & SOCK_NORXBLOCK) {
-					errno = EWOULDBLOCK;
+					errno = (Curproc->retval == EALARM) ? EALARM : EWOULDBLOCK;
 					return -1;
 				} else
 #endif
@@ -745,7 +745,7 @@ struct nr3hdr n3hdr;
 			while ((cb.udp = up->cb.udp) != NULLUDP && (cnt = recv_udp (cb.udp, &fsocket, &bp)) == -1) {
 #ifdef useblock
 				if (up->noblock & SOCK_NORXBLOCK) {
-					errno = EWOULDBLOCK;
+					errno = (Curproc->retval == EALARM) ? EALARM : EWOULDBLOCK;
 					return -1;
 				} else
 #endif
@@ -769,7 +769,7 @@ struct nr3hdr n3hdr;
 			while ((cb.rip = up->cb.rip) != NULLRIP && cb.rip->rcvq == NULLBUF) {
 #ifdef useblock
 				if (up->noblock & SOCK_NORXBLOCK) {
-					errno = EWOULDBLOCK;
+					errno = (Curproc->retval == EALARM) ? EALARM : EWOULDBLOCK;
 					return -1;
 				} else
 #endif
@@ -798,7 +798,7 @@ struct nr3hdr n3hdr;
 			while ((cb.ax25 = up->cb.ax25) != NULLAX25 && (bp = recv_ax25 (cb.ax25, (int16) 0)) == NULLBUF) {
 #ifdef useblock
 				if (up->noblock & SOCK_NORXBLOCK) {
-					errno = EWOULDBLOCK;
+					errno = (Curproc->retval == EALARM) ? EALARM : EWOULDBLOCK;
 					return -1;
 				} else
 #endif
@@ -819,7 +819,7 @@ struct nr3hdr n3hdr;
 			while (s == Axui_sock && Bcq == NULLBUF) {
 #ifdef useblock
 				if (up->noblock & SOCK_NORXBLOCK) {
-					errno = EWOULDBLOCK;
+					errno = (Curproc->retval == EALARM) ? EALARM : EWOULDBLOCK;
 					return -1;
 				} else
 #endif
@@ -848,7 +848,7 @@ struct nr3hdr n3hdr;
 			while ((cb.rnr = up->cb.rnr) != NULLRNR && cb.rnr->rcvq == NULLBUF) {
 #ifdef useblock
 				if (up->noblock & SOCK_NORXBLOCK) {
-					errno = EWOULDBLOCK;
+					errno = (Curproc->retval == EALARM) ? EALARM : EWOULDBLOCK;
 					return -1;
 				} else
 #endif
@@ -877,7 +877,7 @@ struct nr3hdr n3hdr;
 			while ((cb.nr4 = up->cb.nr4) != NULLNR4CB && (bp = recv_nr4 (cb.nr4, (int16) 0)) == NULLBUF) {
 #ifdef useblock
 				if (up->noblock & SOCK_NORXBLOCK) {
-					errno = EWOULDBLOCK;
+					errno = (Curproc->retval == EALARM) ? EALARM : EWOULDBLOCK;
 					return -1;
 				} else
 #endif
@@ -1196,6 +1196,7 @@ register struct usock *up;
 	if (up->name == NULLCHAR) {
 		/* Not bound yet */
 		*namelen = 0;
+		*name = 0;
 		return 0;
 	}
 	if (up->name != NULLCHAR) {
@@ -1558,6 +1559,8 @@ int s				/* Socket index */
 ) {
 register struct usock *up;
 
+	if (s == -1)
+		return 0;
 	if ((up = itop (s)) == NULLUSOCK) {
 		errno = EBADF;
 		return -1;
@@ -2138,9 +2141,8 @@ getnextport (void)
 
 
 
-#ifdef UNIX	/* used by pipe.c */
 int
-socketalive (int s OPTIONAL)
+socketalive (int s)
 {
 register struct usock *up;
 
@@ -2158,7 +2160,6 @@ register struct usock *up;
 
 	return 1;
 }
-#endif
 
 
 /* this is called by tipmail only ! - WG7J */
