@@ -21,12 +21,16 @@
 #include "iface.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: tcpcmd.c,v 1.19 1997/06/28 16:46:13 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: tcpcmd.c,v 1.21 1997/09/07 00:31:16 root Exp root $";
 #endif
 
 #ifdef  TCPACCESS
 static int doaccess (int argc, char *argv[], void *p);
-static void addtaccess (uint32 target, unsigned int bits, int16 low, int16 high, int16 permit);
+#endif
+
+#if defined(TCPACCESS) || defined(UDPACCESS)
+int do_tcpudp_access (int argc, char *argv[], void *p);
+static void addtaccess (struct rtaccess **tpacc, uint32 target, unsigned int bits, int16 low, int16 high, int16 permit);
 #endif
 
 static int tcpirtt (int argc, char *argv[], void *p);
@@ -708,7 +712,7 @@ char *buf, *cp;
 			if (tcb->state == TCP_LISTEN)
 				continue;
 
-			strcpy (temp, pinet (&tcb->conn.remote));
+			strncpy (temp, pinet (&tcb->conn.remote), 80);
 			cp = strstr (pinet (&tcb->conn.local), ":");
 			if (cp)
 				strcat (temp, cp);
@@ -828,12 +832,13 @@ void *p OPTIONAL;
 }
 
 
-#ifdef  TCPACCESS
-static int
-doaccess (argc, argv, p)
+#if defined(TCPACCESS) || defined(UDPACCESS)
+
+int
+do_tcpudp_access (argc, argv, p)
 int argc;
 char *argv[];
-void *p OPTIONAL;
+void *p;
 {
 uint32 target;
 unsigned bits;
@@ -841,12 +846,13 @@ char *bitp;
 int16 lport, hport, state;
 char const *cp;		/* for printing the table */
 struct rtaccess *tpacc;
+struct rtaccess **headptr = (struct rtaccess **)p;
 struct rtaccess *head;
 struct rtaccess *prev;
 
 	if (argc == 1) {	/* print out the table */
 		tputs (accesshdr);
-		for (tpacc = TCPaccess; tpacc != NULLACCESS; tpacc = tpacc->nxtbits) {
+		for (tpacc = *headptr; tpacc != NULLACCESS; tpacc = tpacc->nxtbits) {
 			if (tpacc->target != 0)
 				cp = inet_ntoa (tpacc->target);
 			else
@@ -911,15 +917,15 @@ struct rtaccess *prev;
 
 	if (strcmp (argv[1], deletestr) == 0) {
 		prev = NULLACCESS;
-		head = tpacc = TCPaccess;
+		head = tpacc = *headptr;
 		while (tpacc != NULLACCESS) {
 			head = tpacc;
 			tpacc = tpacc->nxtbits;
 			if ((head->target == target) && (head->bits == bits) &&
 			    (head->lowport == lport) && (head->highport == hport)) {	/*match*/
 				/*now delete. watch for special cases*/
-				if (head == TCPaccess)	/* first in chain */
-					TCPaccess = head->nxtbits;
+				if (head == *headptr)	/* first in chain */
+					*headptr = head->nxtbits;
 				else	{
 					/* sanity check: we cant get here with prev == NULLACCESS !! */
 					if (prev)
@@ -934,16 +940,29 @@ struct rtaccess *prev;
 		return 1;
 	}
 	/* add the access */
-	addtaccess (target, bits, lport, hport, state);
+	addtaccess (headptr, target, bits, lport, hport, state);
 	return 0;
 }
 
 
+
+static int
+doaccess (argc, argv, p)
+int argc;
+char *argv[];
+void *p OPTIONAL;
+{
+	return (do_tcpudp_access (argc, argv, &TCPaccess));
+}
+
+
+
 /*lint -esym(18,addtaccess) */
 /* add an entry to the access control list */
 /* not a lot of error checking 8-) */
 static void
-addtaccess (target, bits, low, high, permit)
+addtaccess (headacc, target, bits, low, high, permit)
+struct rtaccess **headacc;
 uint32 target;			/* Target IP address prefix */
 unsigned int bits;		/* Size of target address prefix in bits (0-32) */
 int16 low;
@@ -962,8 +981,12 @@ struct rtaccess *holder;	/*for the new record*/
 	holder->lowport = low;
 	holder->highport = high;
 	holder->status = permit;
-	if ((tpacc = TCPaccess) == NULLACCESS)
-		TCPaccess = holder;
+
+	if ((low || high) && (*headacc == NULLACCESS || tcp_check (*headacc, target, low) == -1))
+		addtaccess (headacc, target, bits, 0, 0, ~permit);
+
+	if ((tpacc = *headacc) == NULLACCESS)
+		*headacc = holder;
 	else {
 		while (tpacc->nxtbits != NULLACCESS)
 			tpacc = tpacc->nxtbits;
@@ -972,6 +995,7 @@ struct rtaccess *holder;	/*for the new record*/
 }
 
 
+
 /*lint -esym(18,tcp_check) */
 /* check to see if port is "authorized".  Returns 0 if matching permit record
    is found or no access records exists, -1 if not found or deny record found */
@@ -983,6 +1007,7 @@ int16 port;
 {
 unsigned long mask;
 unsigned short tmp;
+int retval = -1;
 
 	if (accptr == NULLACCESS)
 		return 0;	/* no access control */
@@ -995,10 +1020,18 @@ unsigned short tmp;
 		if ((accptr->target == (uint32) (mask & src)) &&
 		  (((port >= accptr->lowport) && (port <= accptr->highport))
 		   || (!accptr->lowport))) {
-			return (accptr->status);
+		   	/* if this is a default entry, save the status,
+		   	   but look in case there is a more specific entry
+		   	   later in the list. If not, upon exit, we will
+		   	   return the proper status
+		   	 */
+		   	if (!accptr->lowport && !accptr->highport)
+		   		retval = accptr->status;
+		   	else
+				return (accptr->status);
 		}
 	}
-	return -1;		/* fall through to here if not found */
+	return retval;		/* fall through to here if specific entry not found */
 }
 
 #endif
