@@ -3,7 +3,7 @@
 #include "files.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: translat.c,v 1.10 1996/12/23 20:37:36 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: translat.c,v 1.11 1997/07/31 00:44:20 root Exp root $";
 #endif
 
 #ifdef TRANSLATE
@@ -46,7 +46,7 @@ char *retval, *trans;
 		*didwe = 1;
 		break;
 	}
-	fclose (fp);
+	(void) fclose (fp);
 	return (retval);
 }
 
