@@ -28,7 +28,7 @@
 
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: tutor.c,v 1.23 1996/12/29 02:47:22 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: tutor.c,v 1.25 1997/08/19 01:19:22 root Exp root $";
 #endif
 
 struct world {
@@ -68,6 +68,7 @@ struct world {
 
 #ifndef STANDALONE
 int Tutored = 0;
+static int inMbScriptHook = 0;
 extern int Stutor, Sinfo, Snews, Tutorlogins[];
 extern char Tcall[AXALEN], Icall[AXALEN], Ncall[AXALEN];
 #endif
@@ -864,7 +865,7 @@ long pos;
 					(void) rdaemon (out2, replyto, NULLCHAR, put, cp, 'P', 0);
 					lastresponse = 'y';
 					if (out2 != NULLFILE)
-						fclose (out2);
+						(void) fclose (out2);
 				}
 				free (put);
 				break;
@@ -911,13 +912,13 @@ long pos;
 #else
 					(void) sendfile (fp);
 #endif
-					fclose (fp);
+					(void) fclose (fp);
 				}
 				break;
 			case 'o':	/* open an old io file */
 			case 'f':	/* create a new io file */
 				if (here->out != NULLFILE)
-					fclose (here->out);
+					(void) fclose (here->out);
 				here->out = NULLFILE;
 				if (*cp == '~' && isdigit (cp[1]))
 					cp = here->variables[cp[1] - '0'];
@@ -1069,7 +1070,7 @@ long pos;
 	}
 lost:
 	if (here->out != NULLFILE) {
-		fclose (here->out);
+		(void) fclose (here->out);
 		here->out = NULLFILE;
 	}
 	free (label);
@@ -1139,7 +1140,7 @@ struct ffblk ff;
 					(void) fgets (buf2, 256, fp);
 					cp = skipwhite (buf2);
 				} while (*cp == '\n');
-				fclose (fp);
+				(void) fclose (fp);
 				if (*cp == '~')	/* subdir directing file */
 					cp = skipwhite (skipnonwhite (skipwhite (++cp)));
 				here->Tutors[here->maxselect++] = strdup (ff.ff_name);
@@ -1206,7 +1207,7 @@ char tmp[AXBUF];
 	here.Error[0] = 0;
 	here.out = NULLFILE;
 	here.whichsocket = here.background = 0;
-	here.socket = calloc (10, sizeof (int));
+	here.socket = callocw (10, sizeof (int));
 
 	here.user = here.socket[0] = Curproc->output;
 	filename[0] = 0;
@@ -1292,7 +1293,7 @@ char tmp[AXBUF];
 			}
 			process (&here);
 		}
-		fclose (here.fp);
+		(void) fclose (here.fp);
 	}
 	mycolorchange (&here, "09", here.last);
 	tprintf ("\n\nThanks for using the ");
@@ -1351,12 +1352,13 @@ int usesession = 0, k;
 struct world here;
 
 	/* Use a session if this comes from console - WG7J*/
-	if (!scriptmode && Curproc->input == Command->input) {
+	if (!inMbScriptHook && !scriptmode && Curproc->input == Command->input) {
 		usesession = 1;
 		if ((sp = newsession (name, SCRIPT, 0)) == NULLSESSION) {
 			return 1;
 		}
 	}
+	inMbScriptHook = 0;
 	here.background = scriptmode;
 	if (!inherited || inherited == (struct world *) -1) {
 		here.ischild = 0;
@@ -1364,7 +1366,7 @@ struct world here;
 		here.whichsocket = 0;
 		here.ipconnect = 1;
 		here.goback = 0;
-		here.socket = calloc (10, sizeof (int));
+		here.socket = callocw (10, sizeof (int));
 
 #ifndef STANDALONE
 		if (inherited == (struct world *) -1) {
@@ -1426,7 +1428,7 @@ struct world here;
 	here.last[0] = 0;
 	here.Error[0] = 0;
 	process (&here);
-	fclose (here.fp);
+	(void) fclose (here.fp);
 	if (inherited) {
 		for (k = 5; k < 10; k++) {
 			if (!here.goback) {
@@ -1462,9 +1464,9 @@ int thisscriptmode;
 	{
 	char fname[256];
 
-		strcpy (fname, make_fname (Command->curdirs->dir, argv[1]));
+		strncpy (fname, make_fname (Command->curdirs->dir, argv[1]), 256);
 		if ((fp = fopen (fname, READ_TEXT)) == NULLFILE) {
-			tprintf (NoRead, fname, sys_errlist[errno]);
+			tprintf (NoRead, fname, SYS_ERRLIST(errno));
 			return 1;
 		}
 	}
@@ -1476,7 +1478,7 @@ int thisscriptmode;
 	if (scriptmode)
 		Curproc->input = Curproc->output = -1;
 	thisscriptmode = scriptmode;
-	if (scriptmode || Curproc->input == Command->input)
+	if (!inMbScriptHook && (scriptmode || Curproc->input == Command->input))
 		(void) newproc ("script", 2048, (void (*)(int, void *, void *)) scriptcmd, (argc > 1) ? 0 : (int) p, (void *) fp, (void *) strdup (argv[1]), 2);
 	else
 		(void) scriptcmd ((argc > 1) ? 0 : (struct world *) p, fp, argv[1]);
@@ -1535,6 +1537,7 @@ char *args2[2];
 		args2[0] = m->line;
 		argsCalling = (char const **) &args2[0];	/*lint !e789 */
 		MbxCalling = m;
+		inMbScriptHook = 1;
 		(void) doscript (1, args, (void *) -1);
 	}
 }
@@ -1571,7 +1574,7 @@ int skiptoblank = 0;
 				fputs (buf, out);
 		}
 	}
-	fclose (out);
+	(void) fclose (out);
 	numCalling = 1;
 	argsCalling = args;	/*lint !e789 */
 	args[0] = "import";
