@@ -12,12 +12,22 @@
 #include "udp.h"
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: udpcmd.c,v 1.15 1997/06/01 22:10:46 root Exp root $";
+static char rcsid[] OPTIONAL = "$Id: udpcmd.c,v 1.16 1997/09/07 00:31:16 root Exp root $";
+#endif
+
+#ifdef  UDPACCESS
+static int douaccess (int argc, char *argv[], void *p);
+extern int do_tcpudp_access (int argc, char *argv[], void *p);
+
+struct rtaccess *UDPaccess = NULLACCESS;       /* access list */
 #endif
 
 static int doudpstat (int argc,char *argv[],void *p);
 
 static struct cmds Udpcmds[] = {
+#ifdef UDPACCESS
+	{ "access",	douaccess,	0, 0,	NULLCHAR },
+#endif
 	{ "status",	doudpstat,	0, 0,	NULLCHAR },
 	{ NULLCHAR,	NULL,		0, 0,	NULLCHAR }
 };
@@ -76,3 +86,20 @@ register int i;
 	}
 	return 0;
 }
+
+
+#ifdef  UDPACCESS
+
+static int
+douaccess (argc, argv, p)
+int argc;
+char *argv[];
+void *p OPTIONAL;
+{
+	return (do_tcpudp_access (argc, argv, &UDPaccess));
+}
+
+#endif
+
+
+
