@@ -19,39 +19,30 @@
 #include "global.h"
 #include "ctype.h"
 #ifdef linux
-#undef IS_LITTLE_ENDIAN
+#undef IS_LITTLE_ENDIAN	/* to avoid a re-definition warning */
 #endif
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <signal.h>
 #include "hardware.h"
 #include "commands.h"
-#ifdef M_UNIX
-#include <time.h>
-#include <sys/select.h>
-#else
 #ifndef _SYS_TIME_H
 #include <sys/time.h>
 #endif
-#endif
 #include "timer.h"
 #include "proc.h"
 #include "socket.h"
 #include "session.h"
+#ifdef ALPHATEST
+#include "main.h"
+#endif
 
 #ifdef NO_GETTOD
 #include <sys/timeb.h>
 #endif
 
 #if !defined(_lint)
-static char rcsid[] OPTIONAL = "$Id: unix.c,v 1.25 1997/05/24 18:29:20 root Exp root $";
-#endif
-
-#ifdef M_UNIX
-#ifndef bzero
-/* stupid bug in SCO 3.2.2 (is there any other kind for them?) */
-#define bzero(s,n) memset((s),0,(n))
-#endif
+static char rcsid[] OPTIONAL = "$Id: unix.c,v 1.29 1997/09/14 14:37:46 root Exp root $";
 #endif
 
 /* and undo a collision avoidance, since we need the real one */
@@ -80,10 +71,6 @@ static int time_in_a_free_call = 0;
 #define MAX_TIME_INA_FREE_CALL	60
 
 
-#ifdef  _lint
-void _exit (int);
-#endif
-
 /* Don't use this yet... something's still calling malloc() directly */
 #undef LMCHECK			/* TEST memory allocation checker */
 
@@ -314,9 +301,24 @@ void *p;
 }
 
 
+
+static jmp_buf free_trap;
+
+
+static void
+j_free_trap (int sig OPTIONAL)
+{
+	++Ksig.kfreesegvs;
+	longjmp (free_trap, 1);
+}
+
+
+
 void
 j_free(IFLINT(const) void *p)
 {
+struct sigaction sa, saold;
+
 	if (p)	{
 #ifdef LMCHECK
 		if (!mwunhash (p))	{
@@ -326,7 +328,17 @@ j_free(IFLINT(const) void *p)
 #endif
 		time_in_a_free_call = 0;
 		within_a_free_call = 1;
-		free((void *)p);
+
+		/* protect free() from SEGV signals */
+		sa.sa_handler = j_free_trap;
+		sa.sa_flags = 0;
+		(void) sigaction (SIGSEGV, &sa, &saold);
+
+		if (setjmp (free_trap) == 0)
+			free((void *)p);
+
+		/* restore previous value for SIGV signals */
+		(void) sigaction (SIGSEGV, &saold, (struct sigaction *) 0);
 		within_a_free_call = 0;
 	}
 }
@@ -476,7 +488,7 @@ static struct timezone tz;
 
 
 void
-getdate(dp)
+tnos_getdate(dp)
 struct date *dp;
 {
 struct tm *tm;
@@ -566,29 +578,76 @@ struct io *evp, *evc;
 
 
 #ifndef DUMP_CORE
+#ifdef ALPHATEST
+extern void proc_launch (void);
+#endif
+
 static void
 ouch(int sig OPTIONAL)
 {
-#if 0
 struct sigaction sa;
 
 	sa.sa_handler = SIG_DFL;
 	sa.sa_flags = 0;
-	sigaction (SIGSEGV, &sa, (struct sigaction *) 0);
+	(void) sigaction (SIGSEGV, &sa, (struct sigaction *) 0);
 #ifdef SIGBUS
-	sigaction (SIGBUS, &sa, (struct sigaction *) 0);
+	(void) sigaction (SIGBUS, &sa, (struct sigaction *) 0);
 #endif
+
+#if 0
 	if (fork() == 0)	{
 		sigaction(sig, &sa, (struct sigaction *) 0);
-#ifdef sun
-		kill(getpid(), sig);
-#else
-		raise(sig);
-#endif
+		(void) kill(getpid(), sig);
 	}
 	detach_all_asy();
 #endif
-	exit(1);
+
+	if (shall_we_crash())
+		crash_it_already ("SIGSEGV signal");
+#ifdef ALPHATEST
+	if (++Ksig.kresumes == RESTART_COUNT || Command->proc == Curproc)
+		exit (1);
+
+	/* Stop alarm clock in case it's running */
+	stop_timer (&Curproc->alarm);
+
+	/* attempt to simply kill off the process running, not TNOS */
+	if (main_exit != TRUE)	{
+		time_t now;
+		
+		log (-1, "signal SIGSEGV caught - killing process '%s' and resuming",
+			Curproc->name);
+		now = time ((time_t *)0);
+		tcmdprintf ("*** signal SIGSEGV caught at %s*** killing process '%s' and resuming\n",
+			ctime (&now), Curproc->name);
+		tprintf ("\n!!! System error occurred !!!\n");
+	}
+	/* these were done without a signal handler, to allow dying if logging fails */
+
+	/* reset the handler */
+	sa.sa_handler = ouch;
+	sa.sa_flags = 0;
+	(void) sigaction (SIGSEGV, &sa, (struct sigaction *) 0);
+#ifdef SIGBUS
+	(void) sigaction (SIGBUS, &sa, (struct sigaction *) 0);
+#endif
+	switch (Curproc->ptype)	{
+		case PTYPE_IO:
+		case PTYPE_DAEMON:
+		case PTYPE_SERVER:	Ksig.krestarts++;
+					if (main_exit == TRUE)
+						killself ();
+					else
+						proc_launch ();		/* restart the process ;-) */
+					break;
+		case PTYPE_NORMAL:
+		default:		if (main_exit != TRUE)	{
+						tflush ();
+						kpause (3000);
+					}
+					killself ();		/* let the errant process be the martyr */
+	}
+#endif
 }
 #endif
 
@@ -687,13 +746,7 @@ struct itimerval it;
 static void
 cleanup (int sig OPTIONAL)
 {
-#if 0
-	if (sig == SIGHUP)
-		log (-1, "Received SIGHUP");
-	where_outta_here ((sig == SIGHUP) ? 2 : 0, "cleanup");
-#else
 	where_outta_here (0, "cleanup");
-#endif
 }
 
 
@@ -704,6 +757,7 @@ struct sigaction sa;
 
 	init_time();
 	register_io (0, &Keyboard);
+
 #ifndef DUMP_CORE
 	sa.sa_handler = ouch;
 	sa.sa_flags = 0;
@@ -712,12 +766,11 @@ struct sigaction sa;
 	(void) sigaction (SIGBUS, &sa, (struct sigaction *) 0);
 #endif
 #endif /* DUMP_CORE */
+
 	sa.sa_handler = cleanup;
 	sa.sa_flags = 0;
 	(void) sigaction (SIGTERM, &sa, (struct sigaction *) 0);
-#if 0
-	sigaction (SIGHUP, &sa, (struct sigaction *) 0);
-#endif
+
 	sa.sa_handler = SIG_IGN;
 	sa.sa_flags = 0;
 	(void) sigaction (SIGWINCH, &sa, (struct sigaction *) 0);
@@ -773,7 +826,7 @@ _putenv (const char *envstr)
 {
 char **new_envp;
 
-	new_envp = (char **) calloc ((unsigned int)(tnos_envp_size + 2), sizeof (char *));
+	new_envp = (char **) callocw ((unsigned int)(tnos_envp_size + 2), sizeof (char *));
 	if (new_envp == NULLCHARP)
 		return -1;
 
@@ -841,7 +894,7 @@ char *buf;
 int retval = 0;
 
 	/* create the new entry */
-	buf = (char *) calloc (1, strlen(name) + strlen(value) + 2);
+	buf = (char *) callocw (1, strlen(name) + strlen(value) + 2);
 	if (buf == NULLCHAR)
 		return -1;
 	sprintf (buf, "%s=%s", name, value);
@@ -939,7 +992,7 @@ char const *av[4];
 				av[ac++] = argv[1];
 			}
 			av[ac] = 0;
-			(void) execve (av[0], av, (const char **)tnos_envp);
+			(void) execve (av[0], (char * const *)av, (char * const *)tnos_envp);	/*lint !e605 */
 			_exit(1);
 		default:
 			close (pi[1]);
